@@ -175,11 +175,11 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 			} else {
 				if(res->ai_family == AF_INET) {
 					req_obj->dst = res;
-					ptr = &((struct sockaddr_in *) res->ai_addr)->sin_addr;
+					ptr = &PTR_CAST(struct sockaddr_in, res->ai_addr)->sin_addr;
 					inet_ntop (res->ai_family, ptr, req_obj->ipaddress, INET_ADDRSTRLEN);
 				} else if (res->ai_family == AF_INET6) {
 					req_obj->dst = res;
-					ptr = &((struct sockaddr_in6 *) res->ai_addr)->sin6_addr;
+					ptr = &PTR_CAST(struct sockaddr_in6, res->ai_addr)->sin6_addr;
 					inet_ntop (res->ai_family, ptr, req_obj->ipaddress, INET6_ADDRSTRLEN);
 				} else {
 					fprintf(stderr, "server should be an IP, not %s\n", optarg);
@@ -281,7 +281,7 @@ main(int argc, char **argv)
 #endif
 
 	/* Allocate the room */
-	req = (REQ *) MALLOC(sizeof (REQ));
+	PMALLOC(req);
 
 	/* Preset (potentially) non-zero defaults */
 	req->hash = hash_default;
