@@ -38,10 +38,10 @@ bfd_data_t *old_bfd_data;
 char *bfd_buffer;
 
 /* Local vars */
-static const char *dump_file = "/tmp/keepalived_bfd.data";
+static const char *dump_file = KA_TMP_DIR "/keepalived_bfd.data";
 
 /*
- * bfd_t functions
+ *	bfd_t functions
  */
 /* Initialize bfd_t */
 bool
@@ -66,13 +66,14 @@ alloc_bfd(const char *name)
 		return false;
 	}
 
-	bfd = (bfd_t *) MALLOC(sizeof (bfd_t));
+	PMALLOC(bfd);
+	INIT_LIST_HEAD(&bfd->e_list);
 	strcpy(bfd->iname, name);
 
 	/* Set defaults */
-	bfd->local_min_rx_intv = BFD_MINRX_DEFAULT * 1000;
-	bfd->local_min_tx_intv = BFD_MINTX_DEFAULT * 1000;
-	bfd->local_idle_tx_intv = BFD_IDLETX_DEFAULT * 1000;
+	bfd->local_min_rx_intv = BFD_MINRX_DEFAULT * TIMER_HZ / 1000;
+	bfd->local_min_tx_intv = BFD_MINTX_DEFAULT * TIMER_HZ / 1000;
+	bfd->local_idle_tx_intv = BFD_IDLETX_DEFAULT * TIMER_HZ / 1000;
 	bfd->local_detect_mult = BFD_MULTIPLIER_DEFAULT;
 
 	bfd->ttl = 0;
@@ -87,16 +88,25 @@ alloc_bfd(const char *name)
 	bfd->sands_exp = TIMER_NEVER;
 	bfd->sands_rst = TIMER_NEVER;
 
-	list_add(bfd_data->bfd, bfd);
+	list_add_tail(&bfd->e_list, &bfd_data->bfd);
 
 	return true;
 }
 
-static void
-free_bfd(void *data)
+void
+free_bfd(bfd_t *bfd)
 {
-	assert(data);
-	FREE(data);
+	list_del_init(&bfd->e_list);
+	FREE(bfd);
+
+}
+static void
+free_bfd_list(list_head_t *l)
+{
+	bfd_t *bfd, *bfd_tmp;
+
+	list_for_each_entry_safe(bfd, bfd_tmp, l, e_list)
+		free_bfd(bfd);
 }
 
 static void
@@ -118,14 +128,10 @@ conf_write_sands(FILE *fp, const char *text, unsigned long sands)
 
 /* Dump BFD instance configuration parameters */
 static void
-dump_bfd(FILE *fp, const void *data)
+dump_bfd(FILE *fp, const bfd_t *bfd)
 {
-	const bfd_t *bfd;
 	char time_str[26];
 
-	assert(data);
-	bfd = (const bfd_t *)data;
-
 	conf_write(fp, " BFD Instance = %s", bfd->iname);
 	conf_write(fp, "   Neighbor IP = %s",
 		    inet_sockaddrtos(&bfd->nbr_addr));
@@ -134,14 +140,10 @@ dump_bfd(FILE *fp, const void *data)
 		conf_write(fp, "   Source IP = %s",
 			    inet_sockaddrtos(&bfd->src_addr));
 
-	conf_write(fp, "   Required min RX interval = %u ms",
-		    bfd->local_min_rx_intv / (TIMER_HZ / 1000));
-	conf_write(fp, "   Desired min TX interval = %u ms",
-		    bfd->local_min_tx_intv / (TIMER_HZ / 1000));
-	conf_write(fp, "   Desired idle TX interval = %u ms",
-		    bfd->local_idle_tx_intv / (TIMER_HZ / 1000));
-	conf_write(fp, "   Detection multiplier = %d",
-		    bfd->local_detect_mult);
+	conf_write(fp, "   Required min RX interval = %u us", bfd->local_min_rx_intv);
+	conf_write(fp, "   Desired min TX interval = %u us", bfd->local_min_tx_intv);
+	conf_write(fp, "   Desired idle TX interval = %u us", bfd->local_idle_tx_intv);
+	conf_write(fp, "   Detection multiplier = %d", bfd->local_detect_mult);
 	conf_write(fp, "   %s = %d",
 		    bfd->nbr_addr.ss_family == AF_INET ? "TTL" : "hoplimit",
 		    bfd->ttl);
@@ -172,16 +174,16 @@ dump_bfd(FILE *fp, const void *data)
 		conf_write(fp, "   remote discriminator = 0x%x", bfd->remote_discr);
 		conf_write(fp, "   local diag = %s", BFD_DIAG_STR(bfd->local_diag));
 		conf_write(fp, "   remote diag = %s", BFD_DIAG_STR(bfd->remote_diag));
-		conf_write(fp, "   remote min tx intv = %u ms", bfd->remote_min_tx_intv / (TIMER_HZ / 1000));
-		conf_write(fp, "   remote min rx intv = %u ms", bfd->remote_min_rx_intv / (TIMER_HZ / 1000));
+		conf_write(fp, "   remote min tx intv = %u us", bfd->remote_min_tx_intv);
+		conf_write(fp, "   remote min rx intv = %u us", bfd->remote_min_rx_intv);
 		conf_write(fp, "   local demand = %u", bfd->local_demand);
 		conf_write(fp, "   remote demand = %u", bfd->remote_demand);
 		conf_write(fp, "   remote detect multiplier = %u", bfd->remote_detect_mult);
 		conf_write(fp, "   %spoll, %sfinal", bfd->poll ? "" : "!", bfd->final ? "" : "!");
-		conf_write(fp, "   local tx intv = %u ms", bfd->local_tx_intv / (TIMER_HZ / 1000));
-		conf_write(fp, "   remote tx intv = %u ms", bfd->remote_tx_intv / (TIMER_HZ / 1000));
-		conf_write(fp, "   local detection time = %" PRIu64 " ms", bfd->local_detect_time / (TIMER_HZ / 1000));
-		conf_write(fp, "   remote detection time = %" PRIu64 " ms", bfd->remote_detect_time / (TIMER_HZ / 1000));
+		conf_write(fp, "   local tx intv = %u us", bfd->local_tx_intv);
+		conf_write(fp, "   remote tx intv = %u us", bfd->remote_tx_intv);
+		conf_write(fp, "   local detection time = %" PRIu64 " us", bfd->local_detect_time);
+		conf_write(fp, "   remote detection time = %" PRIu64 " us", bfd->remote_detect_time);
 		if (bfd->last_seen.tv_sec == 0)
 			conf_write(fp, "   last_seen = [never]");
 		else {
@@ -190,23 +192,27 @@ dump_bfd(FILE *fp, const void *data)
 		}
 	}
 }
+static void
+dump_bfd_list(FILE *fp, const list_head_t *l)
+{
+	bfd_t *bfd;
 
-/* Looks up bfd instance by name */
+	list_for_each_entry(bfd, l, e_list)
+		dump_bfd(fp, bfd);
+}
+
+/*
+ *	Looks up bfd instance by name
+ */
 static bfd_t * __attribute__ ((pure))
 find_bfd_by_name2(const char *name, const bfd_data_t *data)
 {
-	element e;
 	bfd_t *bfd;
 
 	assert(name);
 	assert(data);
-	assert(data->bfd);
 
-	if (LIST_ISEMPTY(data->bfd))
-		return NULL;
-
-	for (e = LIST_HEAD(data->bfd); e; ELEMENT_NEXT(e)) {
-		bfd = ELEMENT_DATA(e);
+	list_for_each_entry(bfd, &data->bfd, e_list) {
 		if (!strcmp(name, bfd->iname))
 			return bfd;
 	}
@@ -229,15 +235,15 @@ bfd_cmp_timers(bfd_t * old_bfd, bfd_t * bfd)
 }
 
 /*
- * bfd_data_t functions
+ *	bfd_data_t functions
  */
 bfd_data_t *
 alloc_bfd_data(void)
 {
 	bfd_data_t *data;
 
-	data = (bfd_data_t *) MALLOC(sizeof (bfd_data_t));
-	data->bfd = alloc_list(free_bfd, dump_bfd);
+	PMALLOC(data);
+	INIT_LIST_HEAD(&data->bfd);
 
 	/* Initialize internal variables */
 	data->thread_in = NULL;
@@ -247,11 +253,11 @@ alloc_bfd_data(void)
 }
 
 void
-free_bfd_data(bfd_data_t * data)
+free_bfd_data(bfd_data_t *data)
 {
 	assert(data);
 
-	free_list(&data->bfd);
+	free_bfd_list(&data->bfd);
 	FREE(data);
 }
 
@@ -268,9 +274,9 @@ dump_bfd_data(FILE *fp, const bfd_data_t *data)
 		conf_write(fp, " thread_in = 0x%p", data->thread_in);
 	}
 
-	if (!LIST_ISEMPTY(data->bfd)) {
+	if (!list_empty(&data->bfd)) {
 		conf_write(fp, "------< BFD Topology >------");
-		dump_list(fp, data->bfd);
+		dump_bfd_list(fp, &data->bfd);
 	}
 }
 
@@ -280,8 +286,7 @@ bfd_print_data(void)
 	FILE *file = fopen_safe(dump_file, "w");
 
 	if (!file) {
-		log_message(LOG_INFO, "Can't open %s (%d: %s)",
-			dump_file, errno, strerror(errno));
+		log_message(LOG_INFO, "Can't open %s (%d: %m)", dump_file, errno);
 		return;
 	}
 
@@ -294,13 +299,11 @@ void
 bfd_complete_init(void)
 {
 	bfd_t *bfd, *bfd_old;
-	element e;
 
 	assert(bfd_data);
-	assert(bfd_data->bfd);
 
 	/* Build configuration */
-	LIST_FOREACH(bfd_data->bfd, bfd, e) {
+	list_for_each_entry(bfd, &bfd_data->bfd, e_list) {
 		/* If there was an old instance with the same name
 		   copy its state and thread sands during reload */
 		if (reload && (bfd_old = find_bfd_by_name2(bfd->iname, old_bfd_data))) {
@@ -318,13 +321,13 @@ bfd_complete_init(void)
 }
 
 /*
- * bfd_buffer functions
+ *	bfd_buffer functions
  */
 void
 alloc_bfd_buffer(void)
 {
 	if (!bfd_buffer)
-		bfd_buffer = (char *) MALLOC(BFD_BUFFER_SIZE);
+		bfd_buffer = (char *)MALLOC(BFD_BUFFER_SIZE);
 }
 
 void
@@ -335,7 +338,7 @@ free_bfd_buffer(void)
 }
 
 /*
- * Lookup functions
+ *	Lookup functions
  */
 /* Looks up bfd instance by neighbor address, and optional local address.
  * If local address is not set, then it is a configuration time check and
@@ -343,13 +346,12 @@ free_bfd_buffer(void)
 bfd_t * __attribute__ ((pure))
 find_bfd_by_addr(const struct sockaddr_storage *nbr_addr, const struct sockaddr_storage *local_addr)
 {
-	element e;
 	bfd_t *bfd;
 	assert(nbr_addr);
 	assert(local_addr);
 	assert(bfd_data);
 
-	LIST_FOREACH(bfd_data->bfd, bfd, e) {
+	list_for_each_entry(bfd, &bfd_data->bfd, e_list) {
 		if (&bfd->nbr_addr == nbr_addr)
 			continue;
 
@@ -376,14 +378,9 @@ find_bfd_by_addr(const struct sockaddr_storage *nbr_addr, const struct sockaddr_
 bfd_t * __attribute__ ((pure))
 find_bfd_by_discr(const uint32_t discr)
 {
-	element e;
 	bfd_t *bfd;
 
-	if (LIST_ISEMPTY(bfd_data->bfd))
-		return NULL;
-
-	for (e = LIST_HEAD(bfd_data->bfd); e; ELEMENT_NEXT(e)) {
-		bfd = ELEMENT_DATA(e);
+	list_for_each_entry(bfd, &bfd_data->bfd, e_list) {
 		if (bfd->local_discr == discr)
 			return bfd;
 	}
@@ -392,7 +389,7 @@ find_bfd_by_discr(const uint32_t discr)
 }
 
 /*
- * Utility functions
+ *	Utility functions
  */
 
 /* Check if uint64_t is large enough to hold uint32_t * int */
@@ -418,7 +415,6 @@ bfd_get_random_discr(bfd_data_t *data)
 {
 	bfd_t *bfd;
 	uint32_t discr;
-	element e;
 
 	assert(data);
 
@@ -428,7 +424,7 @@ bfd_get_random_discr(bfd_data_t *data)
 		discr = (rand_intv(1, UINT32_MAX) & ~1) | (time_now.tv_sec & 1);
 
 		/* Check for collisions */
-		LIST_FOREACH(data->bfd, bfd, e) {
+		list_for_each_entry(bfd, &data->bfd, e_list) {
 			if (bfd->local_discr == discr) {
 				discr = 0;
 				break;
