@@ -48,9 +48,6 @@ static int bfd_send_packet(int, bfdpkt_t *, bool);
 static void bfd_sender_schedule(bfd_t *);
 
 static void bfd_state_down(bfd_t *, uint8_t diag);
-static void bfd_state_admindown(bfd_t *);
-static void bfd_state_up(bfd_t *);
-static void bfd_dump_timers(FILE *fp, bfd_t *);
 
 /*
  * Session sender thread
@@ -92,7 +89,7 @@ bfd_sender_thread(thread_ref_t thread)
 	bfd_build_packet(&pkt, bfd, bfd_buffer, BFD_BUFFER_SIZE);
 	if (bfd_send_packet(bfd->fd_out, &pkt, !bfd->send_error) == -1) {
 		if (!bfd->send_error) {
-			log_message(LOG_ERR, "BFD_Instance(%s) Error sending packet", bfd->iname);
+			log_message(LOG_ERR, "(%s) Error sending packet", bfd->iname);
 			bfd->send_error = true;
 		}
 	} else
@@ -251,7 +248,7 @@ bfd_expire_thread(thread_ref_t thread)
 
 	if (bfd->local_state == BFD_STATE_UP ||
 	    __test_bit(LOG_EXTRA_DETAIL_BIT, &debug))
-		log_message(LOG_WARNING, "BFD_Instance(%s) Expired after"
+		log_message(LOG_WARNING, "(%s) Expired after"
 			    " %" PRIu32 " ms (%" PRIu32 " usec overdue)",
 			    bfd->iname, dead_time / 1000, overdue_time);
 
@@ -487,7 +484,7 @@ bfd_state_down(bfd_t *bfd, uint8_t diag)
 
 	if (bfd->local_state == BFD_STATE_UP ||
 	    __test_bit(LOG_EXTRA_DETAIL_BIT, &debug))
-		log_message(LOG_WARNING, "BFD_Instance(%s) Entering %s state"
+		log_message(LOG_WARNING, "(%s) Entering %s state"
 			    " (Local diagnostic - %s, Remote diagnostic - %s)",
 			    bfd->iname, BFD_STATE_STR(BFD_STATE_DOWN),
 			    BFD_DIAG_STR(diag),
@@ -516,7 +513,7 @@ bfd_state_admindown(bfd_t *bfd)
 	if (bfd_sender_scheduled(bfd))
 		bfd_sender_cancel(bfd);
 
-	log_message(LOG_WARNING, "BFD_Instance(%s) Entering %s state",
+	log_message(LOG_WARNING, "(%s) Entering %s state",
 		    bfd->iname, BFD_STATE_STR(bfd->local_state));
 
 	bfd_state_fall(bfd, false);
@@ -531,7 +528,7 @@ bfd_state_rise(bfd_t *bfd)
 
 	if (bfd->local_state == BFD_STATE_UP ||
 	    __test_bit(LOG_EXTRA_DETAIL_BIT, &debug))
-		log_message(LOG_INFO, "BFD_Instance(%s) Entering %s state",
+		log_message(LOG_INFO, "(%s) Entering %s state",
 			    bfd->iname, BFD_STATE_STR(bfd->local_state));
 
 	if (bfd_reset_scheduled(bfd))
@@ -577,25 +574,28 @@ bfd_state_init(bfd_t *bfd)
 static void
 bfd_dump_timers(FILE *fp, bfd_t *bfd)
 {
+	int indent;
+
 	assert(bfd);
 
-	conf_write(fp, "BFD_Instance(%s)"
-		    " --------------< Session parameters >-------------",
+	indent = 2 + strlen(bfd->iname);
+	conf_write(fp, "(%s)"
+		    " ---------------< Session parameters >--------------",
 		    bfd->iname);
-	conf_write(fp, "BFD_Instance(%s)"
-		    "        min_tx  min_rx  tx_intv  mult  detect_time",
-		    bfd->iname);
-	conf_write(fp, "BFD_Instance(%s)"
-		    " local %7u %7u %8u %5u %12" PRIu64,
-		    bfd->iname, (bfd->local_state == BFD_STATE_UP ? bfd->local_min_tx_intv : bfd->local_idle_tx_intv) / 1000,
-		    bfd->local_min_rx_intv / 1000,
-		    bfd->local_tx_intv / 1000, bfd->local_detect_mult,
-		    bfd->local_detect_time / 1000);
-	conf_write(fp, "BFD_Instance(%s)" " remote %6u %7u %8u %5u %12" PRIu64,
-		    bfd->iname, bfd->remote_min_tx_intv / 1000,
-		    bfd->remote_min_rx_intv / 1000,
-		    bfd->remote_tx_intv / 1000, bfd->remote_detect_mult,
-		    bfd->remote_detect_time / 1000);
+	conf_write(fp, "%*s"
+		    "          min_tx  min_rx  tx_intv  mult  detect_time",
+		    indent, "");
+	conf_write(fp, "%*s"
+		    " local  %8u %7u %8u %5u %12" PRIu64,
+		    indent, "", (bfd->local_state == BFD_STATE_UP ? bfd->local_min_tx_intv : bfd->local_idle_tx_intv),
+		    bfd->local_min_rx_intv,
+		    bfd->local_tx_intv, bfd->local_detect_mult,
+		    bfd->local_detect_time);
+	conf_write(fp, "%*s" " remote %8u %7u %8u %5u %12" PRIu64,
+		    indent, "", bfd->remote_min_tx_intv,
+		    bfd->remote_min_rx_intv,
+		    bfd->remote_tx_intv, bfd->remote_detect_mult,
+		    bfd->remote_detect_time);
 }
 
 /*
@@ -620,7 +620,7 @@ bfd_send_packet(int fd, bfdpkt_t *pkt, bool log_error)
 
 	ret =
 	    sendto(fd, pkt->buf, pkt->len, 0,
-		   (struct sockaddr *) &pkt->dst_addr, dstlen);
+		   PTR_CAST(struct sockaddr, &pkt->dst_addr), dstlen);
 	if (ret == -1 && log_error)
 		log_message(LOG_ERR, "sendto() error (%m)");
 
@@ -751,11 +751,13 @@ bfd_handle_packet(bfdpkt_t *pkt)
 		bfd_sender_reschedule(bfd);
 
 	/* Report detection time changes */
-	if (bfd->local_detect_time != old_local_detect_time)
-		log_message(LOG_INFO, "BFD_Instance(%s) Detection time"
-			    " is %" PRIu64 " ms (was %" PRIu64 " ms)", bfd->iname,
-			    bfd->local_detect_time / 1000,
-			    old_local_detect_time / 1000);
+	if (bfd->local_detect_time != old_local_detect_time) {
+		int len = strlen(bfd->iname) + 2;
+		log_message(LOG_INFO, "%*s Detection time"
+			    " is %" PRIu64 " us (was %" PRIu64 " us)", len, "",
+			    bfd->local_detect_time,
+			    old_local_detect_time);
+	}
 
 	/* BFD state machine */
 	if (bfd->remote_state == BFD_STATE_ADMINDOWN &&
@@ -810,9 +812,9 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 	unsigned int ttl = 0;
 	struct msghdr msg;
 	struct cmsghdr *cmsg = NULL;
-	char cbuf[CMSG_SPACE(sizeof (struct in6_pktinfo)) + CMSG_SPACE(sizeof(ttl))];
+	char cbuf[CMSG_SPACE(sizeof (struct in6_pktinfo)) + CMSG_SPACE(sizeof(ttl))] __attribute__((aligned(__alignof__(struct cmsghdr))));
 	struct iovec iov[1];
-	struct in6_pktinfo *pktinfo;
+	const struct in6_pktinfo *pktinfo;
 
 	assert(pkt);
 	assert(fd >= 0);
@@ -850,12 +852,12 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 		    (cmsg->cmsg_level == IPPROTO_IPV6 && cmsg->cmsg_type == IPV6_HOPLIMIT))
 			ttl = *CMSG_DATA(cmsg);
 		else if (cmsg->cmsg_level == IPPROTO_IPV6 && cmsg->cmsg_type == IPV6_PKTINFO) {
-			pktinfo = (struct in6_pktinfo *)CMSG_DATA(cmsg);
+			pktinfo = PTR_CAST_CONST(struct in6_pktinfo, CMSG_DATA(cmsg));
 			if (IN6_IS_ADDR_V4MAPPED(&pktinfo->ipi6_addr)) {
-				((struct sockaddr_in *)&pkt->dst_addr)->sin_addr.s_addr = pktinfo->ipi6_addr.s6_addr32[3];
+				PTR_CAST(struct sockaddr_in, &pkt->dst_addr)->sin_addr.s_addr = pktinfo->ipi6_addr.s6_addr32[3];
 				pkt->dst_addr.ss_family = AF_INET;
 			} else {
-				memcpy(&((struct sockaddr_in6 *)&pkt->dst_addr)->sin6_addr, &pktinfo->ipi6_addr, sizeof(pktinfo->ipi6_addr));
+				memcpy(&PTR_CAST(struct sockaddr_in6, &pkt->dst_addr)->sin6_addr, &pktinfo->ipi6_addr, sizeof(pktinfo->ipi6_addr));
 				pkt->dst_addr.ss_family = AF_INET6;
 			}
 		}
@@ -868,13 +870,13 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
 	if (!ttl)
 		log_message(LOG_WARNING, "recvmsg() returned no TTL control message");
 
-	pkt->hdr = (bfdhdr_t *) buf;
+	pkt->hdr = PTR_CAST(bfdhdr_t, buf);
 	pkt->len = len;
 	pkt->ttl = ttl;
 
 	/* Convert an IPv4-mapped IPv6 address to a real IPv4 address */
-	if (IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6 *)&pkt->src_addr)->sin6_addr)) {
-		((struct sockaddr_in *)&pkt->src_addr)->sin_addr.s_addr = ((struct sockaddr_in6 *)&pkt->src_addr)->sin6_addr.s6_addr32[3];
+	if (IN6_IS_ADDR_V4MAPPED(&PTR_CAST(struct sockaddr_in6, &pkt->src_addr)->sin6_addr)) {
+		PTR_CAST(struct sockaddr_in, &pkt->src_addr)->sin_addr.s_addr = PTR_CAST(struct sockaddr_in6, &pkt->src_addr)->sin6_addr.s6_addr32[3];
 		pkt->src_addr.ss_family = AF_INET;
 	}
 
@@ -1008,7 +1010,7 @@ bfd_open_fd_out(bfd_t *bfd)
 
 	bfd->fd_out = socket(bfd->nbr_addr.ss_family, SOCK_DGRAM, IPPROTO_UDP);
 	if (bfd->fd_out == -1) {
-		log_message(LOG_ERR, "BFD_Instance(%s) socket() error (%m)",
+		log_message(LOG_ERR, "(%s) socket() error (%m)",
 			    bfd->iname);
 		return 1;
 	}
@@ -1034,11 +1036,11 @@ bfd_open_fd_out(bfd_t *bfd)
 		do {
 			/* Try binding socket to the address until we find one available */
 			if (bfd->src_addr.ss_family == AF_INET)
-				((struct sockaddr_in *)&bfd->src_addr)->sin_port = htons(port);
+				PTR_CAST(struct sockaddr_in, &bfd->src_addr)->sin_port = htons(port);
 			else
-				((struct sockaddr_in6 *)&bfd->src_addr)->sin6_port = htons(port);
+				PTR_CAST(struct sockaddr_in6, &bfd->src_addr)->sin6_port = htons(port);
 
-			ret = bind(bfd->fd_out, (struct sockaddr *) &bfd->src_addr, sockaddr_len);
+			ret = bind(bfd->fd_out, PTR_CAST(struct sockaddr, &bfd->src_addr), sockaddr_len);
 
 			if (ret == -1 && errno == EADDRINUSE) {
 				/* Port already in use, try next */
@@ -1054,7 +1056,7 @@ bfd_open_fd_out(bfd_t *bfd)
 
 		if (ret == -1) {
 			log_message(LOG_ERR,
-				    "BFD_Instance(%s) bind() error (%m)",
+				    "(%s) bind() error (%m)",
 				    bfd->iname);
 			return 1;
 		}
@@ -1072,7 +1074,7 @@ bfd_open_fd_out(bfd_t *bfd)
 		ret = setsockopt(bfd->fd_out, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &ttl, sizeof (ttl));
 
 	if (ret == -1) {
-		log_message(LOG_ERR, "BFD_Instance(%s) setsockopt() "
+		log_message(LOG_ERR, "(%s) setsockopt() "
 			    " error (%m)", bfd->iname);
 		return 1;
 	}
@@ -1100,7 +1102,7 @@ bfd_open_fds(bfd_data_t *data)
 
 	list_for_each_entry(bfd, &data->bfd, e_list) {
 		if (bfd_open_fd_out(bfd)) {
-			log_message(LOG_ERR, "BFD_Instance(%s) Unable to"
+			log_message(LOG_ERR, "(%s) Unable to"
 				    " open output socket, disabling instance",
 				    bfd->iname);
 			bfd_state_admindown(bfd);
