@@ -107,7 +107,7 @@ static pcre2_match_context *mcontext;
 static pcre2_jit_stack *jit_stack;
 #endif
 
-static list regexs;	/* list of regex_t */
+static LIST_HEAD_INITIALIZE(regexs);	/* regex_t */
 
 #ifdef _WITH_REGEX_TIMERS_
 struct timespec total_regex_times;
@@ -134,15 +134,14 @@ static const char *request_template_ipv6 =
 			"%s"
 			"Host: [%s]%s\r\n\r\n";
 
-static int http_connect_thread(thread_ref_t);
+static void http_connect_thread(thread_ref_t);
 
 #ifdef _WITH_REGEX_CHECK_
 static void
-free_regex(void *data)
+free_regex(regex_t *regex)
 {
-	regex_t *regex = data;
-
-	// Free up the regular expression.
+	list_del_init(&regex->e_list);
+	/* Free up the regular expression. */
 	FREE_CONST_PTR(regex->pattern);
 	pcre2_code_free(regex->pcre2_reCompiled);
 	pcre2_match_data_free(regex->pcre2_match_data);
@@ -163,22 +162,18 @@ free_regex(void *data)
 #endif
 
 static void
-free_url(void *data)
+free_url(url_t *url)
 {
-	url_t *url = data;
-
+	list_del_init(&url->e_list);
 	FREE_CONST_PTR(url->path);
 	FREE_CONST_PTR(url->digest);
 	FREE_CONST_PTR(url->virtualhost);
 #ifdef _WITH_REGEX_CHECK_
 	if (url->regex) {
-		if (!--url->regex->use_count) {
-			free_list_data(regexs, url->regex);
-
-			if (LIST_ISEMPTY(regexs)) {
-				/* This is the last regex to be freed, so free up one-off resources */
-				free_list(&regexs);
+		if (!--url->regex->refcnt) {
+			free_regex(url->regex);
 
+			if (list_empty(&regexs)) {
 #ifndef PCRE2_DONT_USE_JIT
 				if (mcontext) {
 					pcre2_match_context_free(mcontext);
@@ -200,6 +195,14 @@ free_url(void *data)
 #endif
 	FREE(url);
 }
+static void
+free_url_list(list_head_t *l)
+{
+	url_t *url, *url_tmp;
+
+	list_for_each_entry_safe(url, url_tmp, l, e_list)
+		free_url(url);
+}
 
 static char *
 format_digest(const uint8_t *digest, char *buf)
@@ -213,9 +216,8 @@ format_digest(const uint8_t *digest, char *buf)
 }
 
 static void
-dump_url(FILE *fp, const void *data)
+dump_url(FILE *fp, const url_t *url)
 {
-	const url_t *url = data;
 	char digest_buf[2 * MD5_DIGEST_LENGTH + 1];
 	unsigned int i = 0;
 	unsigned min = 0;
@@ -226,7 +228,7 @@ dump_url(FILE *fp, const void *data)
 
 	conf_write(fp, "     HTTP Status Code(s)");
 	for (i = HTTP_STATUS_CODE_MIN; i <= HTTP_STATUS_CODE_MAX; i++) {
-		if (__test_bit(i - HTTP_STATUS_CODE_MIN, url->status_code)) {
+		if (__test_bit_array(i - HTTP_STATUS_CODE_MIN, url->status_code)) {
 			if (!min)
 				min = i;
 		} else {
@@ -274,7 +276,7 @@ dump_url(FILE *fp, const void *data)
 		else
 			options_buf[0] = '\0';
 		conf_write(fp, "     Regex options:%s", options_buf);
-		conf_write(fp, "     Regex use count = %u", url->regex->use_count);
+		conf_write(fp, "     Regex ref count = %u", url->regex->refcnt);
 #ifndef PCRE2_DONT_USE_JIT
 		if (url->regex_use_stack)
 			conf_write(fp, "     Regex stack start %zu, max %zu", jit_stack_start, jit_stack_max);
@@ -282,6 +284,14 @@ dump_url(FILE *fp, const void *data)
 	}
 #endif
 }
+static void
+dump_url_list(FILE *fp, const list_head_t *l)
+{
+	url_t *url;
+
+	list_for_each_entry(url, l, e_list)
+		dump_url(fp, url);
+}
 
 static void
 free_http_request(request_t *req)
@@ -290,17 +300,16 @@ free_http_request(request_t *req)
 		return;
 	if (req->ssl)
 		SSL_free(req->ssl);
-	if (req->buffer)
-		FREE(req->buffer);
+	FREE_PTR(req->buffer);
 	FREE(req);
 }
 
 static void
-free_http_get_check(checker_t *checker)
+free_http_check(checker_t *checker)
 {
 	http_checker_t *http_get_chk = checker->data;
 
-	free_list(&http_get_chk->url);
+	free_url_list(&http_get_chk->url);
 	free_http_request(http_get_chk->req);
 	FREE_CONST_PTR(http_get_chk->virtualhost);
 	FREE_PTR(http_get_chk);
@@ -309,7 +318,7 @@ free_http_get_check(checker_t *checker)
 }
 
 static void
-dump_http_get_check(FILE *fp, const checker_t *checker)
+dump_http_check(FILE *fp, const checker_t *checker)
 {
 	const http_checker_t *http_get_chk = checker->data;
 
@@ -317,55 +326,64 @@ dump_http_get_check(FILE *fp, const checker_t *checker)
 			http_get_chk->proto == PROTO_HTTP ? "HTTP" : "SSL",
 			http_get_chk->http_protocol == HTTP_PROTOCOL_1_0C ? "1.0C" :
 			  http_get_chk->http_protocol == HTTP_PROTOCOL_1_1 ? "1.1" : "1.0");
-	dump_checker_opts(fp, checker);
 	if (http_get_chk->virtualhost)
 		conf_write(fp, "   Virtualhost = %s", http_get_chk->virtualhost);
 #ifdef _HAVE_SSL_SET_TLSEXT_HOST_NAME_
 	conf_write(fp, "   Enable SNI %sset", http_get_chk->enable_sni ? "" : "un");
 #endif
- 	conf_write(fp, "   Fast recovery %sset", http_get_chk->fast_recovery ? "" : "un");
-	dump_list(fp, http_get_chk->url);
+	conf_write(fp, "   Fast recovery %sset", http_get_chk->fast_recovery ? "" : "un");
+	dump_url_list(fp, &http_get_chk->url);
 	if (http_get_chk->failed_url)
 		conf_write(fp, "   Failed URL = %s", http_get_chk->failed_url->path);
 }
 static http_checker_t *
 alloc_http_get(const char *proto)
 {
-	http_checker_t *http_get_chk;
+	http_checker_t *new;
 
-	http_get_chk = (http_checker_t *) MALLOC(sizeof (http_checker_t));
-	http_get_chk->proto =
-	    (!strcmp(proto, "HTTP_GET")) ? PROTO_HTTP : PROTO_SSL;
-	http_get_chk->http_protocol = HTTP_PROTOCOL_1_0;
-	http_get_chk->url = alloc_list(free_url, dump_url);
-	http_get_chk->virtualhost = NULL;
+	PMALLOC(new);
+	INIT_LIST_HEAD(&new->url);
+	new->proto = (!strcmp(proto, "HTTP_GET")) ? PROTO_HTTP : PROTO_SSL;
+	new->http_protocol = HTTP_PROTOCOL_1_0;
+	new->virtualhost = NULL;
 
-	if (http_get_chk->proto == PROTO_SSL)
+	if (new->proto == PROTO_SSL)
 		check_data->ssl_required = true;
 
-	return http_get_chk;
+	return new;
+}
+
+static size_t __attribute__ ((pure))
+url_list_size(const list_head_t *l)
+{
+	size_t cnt = 0;
+	list_head_t *e;
+
+	list_for_each(e, l)
+		cnt++;
+	return cnt;
 }
 
 static bool __attribute__((pure))
-http_get_check_compare(const checker_t *old_c, const checker_t *new_c)
+compare_http_check(const checker_t *old_c, checker_t *new_c)
 {
 	const http_checker_t *old = old_c->data;
 	const http_checker_t *new = new_c->data;
-	size_t n;
-	const url_t *u1, *u2;
+	url_t *u1, *u2 = NULL;
 	unsigned i;
 
 	if (!compare_conn_opts(old_c->co, new_c->co))
 		return false;
-	if (LIST_SIZE(old->url) != LIST_SIZE(new->url))
+	if (url_list_size(&old->url) != url_list_size(&new->url))
 		return false;
 	if (!old->virtualhost != !new->virtualhost)
 		return false;
 	if (old->virtualhost && strcmp(old->virtualhost, new->virtualhost))
 		return false;
-	for (n = 0; n < LIST_SIZE(new->url); n++) {
-		u1 = (const url_t *)list_element(old->url, n);
-		u2 = (const url_t *)list_element(new->url, n);
+
+	list_for_each_entry(u1, &old->url, e_list) {
+		u2 = (!u2) ? list_first_entry(&new->url, url_t, e_list) :
+			     list_entry(u2->e_list.next, url_t, e_list);
 		if (strcmp(u1->path, u2->path))
 			return false;
 		if (!u1->digest != !u2->digest)
@@ -384,7 +402,7 @@ http_get_check_compare(const checker_t *old_c, const checker_t *new_c)
 		if (!u1->regex != !u2->regex)
 			return false;
 		if (u1->regex) {
-			if (strcmp((const char *)u1->regex->pattern, (const char *)u2->regex->pattern))
+			if (strcmp(PTR_CAST_CONST(char, u1->regex->pattern), PTR_CAST_CONST(char, u2->regex->pattern)))
 				return false;
 			if (u1->regex->pcre2_options != u2->regex->pcre2_options)
 				return false;
@@ -400,6 +418,8 @@ http_get_check_compare(const checker_t *old_c, const checker_t *new_c)
 	return true;
 }
 
+static const checker_funcs_t http_checker_funcs = { CHECKER_HTTP, free_http_check, dump_http_check, compare_http_check, NULL };
+
 /* Configuration stream handling */
 static void
 http_get_handler(const vector_t *strvec)
@@ -410,16 +430,14 @@ http_get_handler(const vector_t *strvec)
 
 	/* queue new checker */
 	http_get_chk = alloc_http_get(str);
-	checker = queue_checker(free_http_get_check, dump_http_get_check,
-		      http_connect_thread, http_get_check_compare,
-		      http_get_chk, CHECKER_NEW_CO(), true);
+	checker = queue_checker(&http_checker_funcs, http_connect_thread, http_get_chk, CHECKER_NEW_CO(), true);
 	checker->default_delay_before_retry = 3 * TIMER_HZ;
 }
 
 static void
 http_get_retry_handler(const vector_t *strvec)
 {
-	checker_t *checker = LIST_TAIL_DATA(checkers_queue);
+	checker_t *checker = CHECKER_GET_CURRENT();
 	unsigned retry;
 
 	report_config_error(CONFIG_GENERAL_ERROR, "nb_get_retry is deprecated - please use 'retry'");
@@ -450,7 +468,7 @@ http_get_check_end(void)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
 
-	if (LIST_ISEMPTY(http_get_chk->url)) {
+	if (list_empty(&http_get_chk->url)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "HTTP/SSL_GET checker has no urls specified - ignoring");
 		dequeue_new_checker();
 	}
@@ -467,22 +485,22 @@ url_handler(__attribute__((unused)) const vector_t *strvec)
 	url_t *new;
 
 	/* allocate the new URL */
-	new = (url_t *) MALLOC(sizeof (url_t));
-
-	list_add(http_get_chk->url, new);
+	PMALLOC(new);
+	INIT_LIST_HEAD(&new->e_list);
+	list_add_tail(&new->e_list, &http_get_chk->url);
 
 #ifdef _WITH_REGEX_CHECK_
 	conf_regex_options = 0;
 #endif
 
-	http_get_chk->url_it = LIST_HEAD(http_get_chk->url);
+	http_get_chk->url_it = list_first_entry(&http_get_chk->url, url_t, e_list);
 }
 
 static void
 path_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 
 	url->path = set_value(strvec);
 }
@@ -491,7 +509,7 @@ static void
 digest_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 	char *digest;
 	char *endptr;
 	int i;
@@ -533,7 +551,7 @@ static void
 status_code_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 	const char *str;
 	unsigned int i, j;
 	char *endptr;
@@ -556,7 +574,7 @@ status_code_handler(const vector_t *strvec)
 		}
 
 		for (j = min; j <= max; j++)
-			__set_bit(j - HTTP_STATUS_CODE_MIN, url->status_code);
+			__set_bit_array(j - HTTP_STATUS_CODE_MIN, url->status_code);
 	}
 }
 
@@ -564,7 +582,7 @@ static void
 url_virtualhost_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 
 	if (vector_size(strvec) < 2) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Missing HTTP_GET virtualhost name");
@@ -609,7 +627,7 @@ regex_handler(__attribute__((unused)) const vector_t *strvec)
 		return;
 	}
 
-	conf_regex_pattern = (const unsigned char *)set_value(strvec_qe);
+	conf_regex_pattern = PTR_CAST_CONST(unsigned char, set_value(strvec_qe));
 	free_strvec(strvec_qe);
 }
 
@@ -617,7 +635,7 @@ static void
 regex_no_match_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 
 	url->regex_no_match = true;
 }
@@ -664,7 +682,7 @@ static void
 regex_min_offset_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 
 	url->regex_min_offset = regex_offset_handler(strvec, "min");
 }
@@ -673,7 +691,7 @@ static void
 regex_max_offset_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 
 	/* regex_max_offset is one beyond last acceptable position */
 	url->regex_max_offset = regex_offset_handler(strvec, "max") + 1;
@@ -684,7 +702,7 @@ static void
 regex_stack_handler(const vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 	unsigned long stack_start, stack_max;
 	char *endptr;
 
@@ -725,57 +743,59 @@ prepare_regex(url_t *url)
 	PCRE2_SIZE pcreErrorOffset;
 	PCRE2_UCHAR buffer[256];
 	regex_t *r;
-	element e;
-
-	if (!LIST_EXISTS(regexs))
-		regexs = alloc_list(free_regex, NULL);
 
 	/* See if this regex has already been specified */
-	LIST_FOREACH(regexs, r, e) {
+	list_for_each_entry(r, &regexs, e_list) {
 		if (r->pcre2_options == conf_regex_options &&
-		    !strcmp((const char *)r->pattern, (const char *)conf_regex_pattern)) {
+		    !strcmp(PTR_CAST_CONST(char, r->pattern), PTR_CAST_CONST(char, conf_regex_pattern))) {
 			url->regex = r;
 			FREE_CONST_PTR(conf_regex_pattern);
-
-			url->regex->use_count++;
+			url->regex->refcnt++;
 
 			return;
 		}
 	}
 
 	/* This is a new regex */
-	url->regex = MALLOC(sizeof *r);
-	url->regex->pattern = conf_regex_pattern;
-	url->regex->pcre2_options = conf_regex_options;
+	PMALLOC(r);
+	INIT_LIST_HEAD(&r->e_list);
+	r->pattern = conf_regex_pattern;
+	r->pcre2_options = conf_regex_options;
 	conf_regex_pattern = NULL;
-	url->regex->use_count = 1;
-
-	url->regex->pcre2_reCompiled = pcre2_compile(url->regex->pattern, PCRE2_ZERO_TERMINATED, url->regex->pcre2_options, &pcreErrorNumber, &pcreErrorOffset, NULL);
+	r->refcnt = 1;
+	r->pcre2_reCompiled = pcre2_compile(r->pattern, PCRE2_ZERO_TERMINATED, r->pcre2_options,
+					    &pcreErrorNumber, &pcreErrorOffset, NULL);
+	url->regex = r;
 
 	/* pcre_compile returns NULL on error, and sets pcreErrorOffset & pcreErrorStr */
-	if(url->regex->pcre2_reCompiled == NULL) {
+	if(r->pcre2_reCompiled == NULL) {
 		pcre2_get_error_message(pcreErrorNumber, buffer, sizeof buffer);
-		log_message(LOG_INFO, "Invalid regex: '%s' at offset %zu: %s\n", url->regex->pattern, pcreErrorOffset, (char *)buffer);
+		log_message(LOG_INFO, "Invalid regex: '%s' at offset %zu: %s\n"
+				    , r->pattern, pcreErrorOffset, PTR_CAST(char, buffer));
 
-		FREE_CONST_PTR(url->regex->pattern);
-		FREE_PTR(url->regex);
+		FREE_CONST_PTR(r->pattern);
+		FREE(r);
 
 		return;
 	}
 
-	url->regex->pcre2_match_data = pcre2_match_data_create_from_pattern(url->regex->pcre2_reCompiled, NULL);
-	pcre2_pattern_info(url->regex->pcre2_reCompiled, PCRE2_INFO_MAXLOOKBEHIND, &url->regex->pcre2_max_lookbehind);
+	r->pcre2_match_data = pcre2_match_data_create_from_pattern(r->pcre2_reCompiled, NULL);
+	pcre2_pattern_info(r->pcre2_reCompiled, PCRE2_INFO_MAXLOOKBEHIND, &r->pcre2_max_lookbehind);
 
 #ifndef PCRE2_DONT_USE_JIT
-	if ((pcreErrorNumber = pcre2_jit_compile(url->regex->pcre2_reCompiled, PCRE2_JIT_PARTIAL_HARD /* | PCRE2_JIT_COMPLETE */))) {
+	if ((pcreErrorNumber = pcre2_jit_compile(r->pcre2_reCompiled, PCRE2_JIT_PARTIAL_HARD /* | PCRE2_JIT_COMPLETE */))) {
 		pcre2_get_error_message(pcreErrorNumber, buffer, sizeof buffer);
-		log_message(LOG_INFO, "Regex JIT compilation failed: '%s': %s\n", url->regex->pattern, (char *)buffer);
+		log_message(LOG_INFO, "Regex JIT compilation failed: '%s': %s\n"
+				    , r->pattern, PTR_CAST(char, buffer));
+
+		FREE_CONST_PTR(r->pattern);
+		FREE(r);
 
 		return;
 	}
 #endif
 
-	list_add(regexs, url->regex);
+	list_add_tail(&r->e_list, &regexs);
 }
 #endif
 
@@ -817,12 +837,12 @@ static void
 url_check(void)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
-	url_t *url = LIST_TAIL_DATA(http_get_chk->url);
+	url_t *url = list_last_entry(&http_get_chk->url, url_t, e_list);
 	unsigned i;
 
 	if (!url->path) {
 		report_config_error(CONFIG_GENERAL_ERROR, "HTTP/SSL_GET checker url has no path - ignoring");
-		free_list_element(http_get_chk->url, http_get_chk->url->tail);
+		free_url(url);
 		return;
 	}
 
@@ -833,7 +853,7 @@ url_check(void)
 	}
 	if (i >= sizeof(url->status_code) / sizeof(url->status_code[0])) {
 		for (i = HTTP_DEFAULT_STATUS_CODE_MIN; i <= HTTP_DEFAULT_STATUS_CODE_MAX; i++)
-			__set_bit(i - HTTP_STATUS_CODE_MIN, url->status_code);
+			__set_bit_array(i - HTTP_STATUS_CODE_MIN, url->status_code);
 	}
 
 #ifdef _WITH_REGEX_CHECK_
@@ -944,7 +964,7 @@ install_ssl_check_keyword(void)
  * method == REGISTER_CHECKER_RETRY => register a retry on url checker thread
  * method == REGISTER_CHECKER_FAILED => register a checker on the failed URL
  */
-static int
+static void
 epilog(thread_ref_t thread, register_checker_t method)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -955,7 +975,10 @@ epilog(thread_ref_t thread, register_checker_t method)
 	bool rs_was_alive;
 
 	if (method == REGISTER_CHECKER_NEW) {
-		ELEMENT_NEXT(http_get_check->url_it);
+		if (list_is_last(&http_get_check->url_it->e_list, &http_get_check->url))
+			http_get_check->url_it = NULL;
+		else
+			http_get_check->url_it = list_entry(http_get_check->url_it->e_list.next, url_t, e_list);
 		checker->retry_it = 0;
 	} else if (method == REGISTER_CHECKER_RETRY)
 		checker->retry_it++;
@@ -980,7 +1003,7 @@ epilog(thread_ref_t thread, register_checker_t method)
 		}
 
 		/* Reset it counters */
-		http_get_check->url_it = LIST_HEAD(http_get_check->url);
+		http_get_check->url_it = list_first_entry(&http_get_check->url, url_t, e_list);
 		checker->retry_it = 0;
 	}
 	/*
@@ -993,7 +1016,7 @@ epilog(thread_ref_t thread, register_checker_t method)
 		if (checker->is_up || !checker->has_run) {
 			if (checker->has_run && checker->retry)
 				log_message(LOG_INFO
-				   , "HTTP_CHECK on service %s failed after %u retry."
+				   , "HTTP_CHECK on service %s failed after %u retries."
 				   , FMT_CHK(checker)
 				   , checker->retry_it - 1);
 			else
@@ -1011,12 +1034,12 @@ epilog(thread_ref_t thread, register_checker_t method)
 		}
 
 		/* Mark we have a failed URL */
-		http_get_check->failed_url = ELEMENT_DATA(http_get_check->url_it);
+		http_get_check->failed_url = http_get_check->url_it;
 	}
 	else if (method == REGISTER_CHECKER_NEW && http_get_check->failed_url) {
 		/* If the failed URL is now up, check all the URLs */
 		http_get_check->failed_url = NULL;
-		http_get_check->url_it = LIST_HEAD(http_get_check->url);
+		http_get_check->url_it = list_first_entry(&http_get_check->url, url_t, e_list);
 		checker->retry_it = 0;
 	}
 
@@ -1048,10 +1071,10 @@ epilog(thread_ref_t thread, register_checker_t method)
 	else
 		thread_add_timer(thread->master, http_connect_thread, checker, delay);
 
-	return 0;
+	return;
 }
 
-int
+void
 timeout_epilog(thread_ref_t thread, const char *debug_msg)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1063,18 +1086,19 @@ timeout_epilog(thread_ref_t thread, const char *debug_msg)
 					    , debug_msg
 					    , FMT_CHK(checker));
 		checker->has_run = true;
-		return epilog(thread, REGISTER_CHECKER_RETRY);
+		epilog(thread, REGISTER_CHECKER_RETRY);
+		return;
 	}
 
 	/* do not retry if server is already known as dead */
-	return epilog(thread, REGISTER_CHECKER_FAILED);
+	epilog(thread, REGISTER_CHECKER_FAILED);
 }
 
 /* return the url pointer of the current url iterator  */
 static inline url_t *
 fetch_next_url(http_checker_t * http_get_check)
 {
-	return ELEMENT_DATA(http_get_check->url_it);
+	return http_get_check->url_it;
 }
 
 #ifdef _WITH_REGEX_CHECK_
@@ -1148,7 +1172,7 @@ check_regex(url_t *url, request_t *req)
 	pcreExecRet = pcre2_match
 #endif
 				(url->regex->pcre2_reCompiled,
-				 (unsigned char *)req->buffer,
+				 PTR_CAST(unsigned char, req->buffer),
 				 req->len,
 				 start_offset,
 				 PCRE2_PARTIAL_HARD,
@@ -1253,7 +1277,7 @@ check_regex(url_t *url, request_t *req)
 #endif
 
 /* Handle response */
-int
+void
 http_handle_response(thread_ref_t thread, unsigned char digest[MD5_DIGEST_LENGTH]
 		     , bool empty_buffer)
 {
@@ -1265,14 +1289,18 @@ http_handle_response(thread_ref_t thread, unsigned char digest[MD5_DIGEST_LENGTH
 	const char *msg = "HTTP status code";
 
 	/* First check if remote webserver returned data */
-	if (empty_buffer)
-		return timeout_epilog(thread, "Read, no data received from ");
+	if (empty_buffer) {
+		timeout_epilog(thread, "Read, no data received from ");
+		return;
+	}
 
 	/* Next check the HTTP status code */
 	if (req->status_code < HTTP_STATUS_CODE_MIN ||
-		req->status_code > HTTP_STATUS_CODE_MAX ||
-		!__test_bit(req->status_code - HTTP_STATUS_CODE_MIN, url->status_code))
-		return timeout_epilog(thread, "HTTP status code error to");
+	    req->status_code > HTTP_STATUS_CODE_MAX ||
+	    !__test_bit_array(req->status_code - HTTP_STATUS_CODE_MIN, url->status_code)) {
+		timeout_epilog(thread, "HTTP status code error to");
+		return;
+	}
 
 	/* Report a length mismatch the first time we get the specific difference */
 	if (req->content_len != SIZE_MAX && req->content_len != req->rx_bytes) {
@@ -1291,8 +1319,10 @@ http_handle_response(thread_ref_t thread, unsigned char digest[MD5_DIGEST_LENGTH
 		/* Compute MD5SUM */
 		r = memcmp(url->digest, digest, MD5_DIGEST_LENGTH);
 
-		if (r)
-			return timeout_epilog(thread, "MD5 digest error to");
+		if (r) {
+			timeout_epilog(thread, "MD5 digest error to");
+			return;
+		}
 		msg = "MD5 digest";
 	}
 
@@ -1310,8 +1340,10 @@ http_handle_response(thread_ref_t thread, unsigned char digest[MD5_DIGEST_LENGTH
 		url->regex->num_regex_urls++;
 #endif
 
-		if (req->regex_matched == url->regex_no_match)
-			return timeout_epilog(thread, "Regex match failed");
+		if (req->regex_matched == url->regex_no_match) {
+			timeout_epilog(thread, "Regex match failed");
+			return;
+		}
 		msg = "Regex match";
 	}
 #endif
@@ -1321,10 +1353,11 @@ http_handle_response(thread_ref_t thread, unsigned char digest[MD5_DIGEST_LENGTH
 			"%s success to %s url(%s)", msg
 			, FMT_CHK(checker)
 			, url->path);
-		return epilog(thread, REGISTER_CHECKER_NEW) + 1;
+		epilog(thread, REGISTER_CHECKER_NEW);
+		return;
 	}
 
-	return epilog(thread, REGISTER_CHECKER_NEW) + 1;
+	epilog(thread, REGISTER_CHECKER_NEW);
 }
 
 /* Handle response stream performing MD5 updates */
@@ -1369,7 +1402,7 @@ http_process_response(request_t *req, size_t r, url_t *url)
 }
 
 /* Asynchronous HTTP stream reader */
-static int
+static void
 http_read_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1381,8 +1414,10 @@ http_read_thread(thread_ref_t thread)
 	ssize_t r = 0;
 
 	/* Handle read timeout */
-	if (thread->type == THREAD_READ_TIMEOUT)
-		return timeout_epilog(thread, "Timeout HTTP read");
+	if (thread->type == THREAD_READ_TIMEOUT) {
+		timeout_epilog(thread, "Timeout HTTP read");
+		return;
+	}
 
 	/* read the HTTP stream */
 	r = read(thread->u.f.fd, req->buffer + req->len,
@@ -1395,7 +1430,7 @@ http_read_thread(thread_ref_t thread)
 				    , strerror(errno));
 		thread_add_read(thread->master, http_read_thread, checker,
 				thread->u.f.fd, timeout, true);
-		return 0;
+		return;
 	}
 
 	if (r <= 0) {	/* -1:error , 0:EOF */
@@ -1405,7 +1440,8 @@ http_read_thread(thread_ref_t thread)
 
 		if (r == -1) {
 			/* We have encountered a real read error */
-			return timeout_epilog(thread, "Read error with");
+			timeout_epilog(thread, "Read error with");
+			return;
 		}
 
 		/* Handle response stream */
@@ -1421,15 +1457,13 @@ http_read_thread(thread_ref_t thread)
 		thread_add_read(thread->master, http_read_thread, checker,
 				thread->u.f.fd, timeout, true);
 	}
-
-	return 0;
 }
 
 /*
  * Read get result from the remote web server.
  * Apply trigger check to this result.
  */
-static int
+static void
 http_response_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1439,11 +1473,13 @@ http_response_thread(thread_ref_t thread)
 	unsigned timeout = checker->co->connection_to;
 
 	/* Handle read timeout */
-	if (thread->type == THREAD_READ_TIMEOUT)
-		return timeout_epilog(thread, "Timeout WEB read");
+	if (thread->type == THREAD_READ_TIMEOUT) {
+		timeout_epilog(thread, "Timeout WEB read");
+		return;
+	}
 
 	/* Allocate & clean the get buffer */
-	req->buffer = (char *) MALLOC(MAX_BUFFER_LENGTH);
+	req->buffer = PTR_CAST(char, MALLOC(MAX_BUFFER_LENGTH));
 	req->extracted = NULL;
 	req->len = 0;
 	req->error = 0;
@@ -1464,11 +1500,10 @@ http_response_thread(thread_ref_t thread)
 	else
 		thread_add_read(thread->master, http_read_thread, checker,
 				thread->u.f.fd, timeout, true);
-	return 0;
 }
 
 /* remote Web server is connected, send it the get url query.  */
-static int
+static void
 http_request_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1484,11 +1519,13 @@ http_request_thread(thread_ref_t thread)
 	int ret = 0;
 
 	/* Handle write timeout */
-	if (thread->type == THREAD_WRITE_TIMEOUT)
-		return timeout_epilog(thread, "Timeout WEB write");
+	if (thread->type == THREAD_WRITE_TIMEOUT) {
+		timeout_epilog(thread, "Timeout WEB write");
+		return;
+	}
 
 	/* Allocate & clean the GET string */
-	str_request = (char *) MALLOC(GET_BUFFER_LENGTH);
+	str_request = PTR_CAST(char, MALLOC(GET_BUFFER_LENGTH));
 
 	fetched_url = fetch_next_url(http_get_check);
 
@@ -1534,18 +1571,20 @@ http_request_thread(thread_ref_t thread)
 
 	FREE(str_request);
 
-	if (!ret)
-		return timeout_epilog(thread, "Cannot send get request to");
+	if (!ret) {
+		timeout_epilog(thread, "Cannot send get request to");
+		return;
+	}
 
 	/* Register read timeouted thread */
 	thread_add_read(thread->master, http_response_thread, checker,
 			thread->u.f.fd, timeout, true);
 	thread_del_write(thread);
-	return 1;
+	return;
 }
 
 /* WEB checkers threads */
-static int
+static void
 http_check_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1559,20 +1598,23 @@ http_check_thread(thread_ref_t thread)
 	status = tcp_socket_state(thread, http_check_thread);
 	switch (status) {
 	case connect_error:
-		return timeout_epilog(thread, "Error connecting");
+		timeout_epilog(thread, "Error connecting");
+		return;
 		break;
 
 	case connect_timeout:
-		return timeout_epilog(thread, "Timeout connecting");
+		timeout_epilog(thread, "Timeout connecting");
+		return;
 		break;
 
 	case connect_fail:
-		return timeout_epilog(thread, "Connection failed");
+		timeout_epilog(thread, "Connection failed");
+		return;
 		break;
 
 	case connect_success:
 		if (!http_get_check->req) {
-			http_get_check->req = (request_t *) MALLOC(sizeof (request_t));
+			PMALLOC(http_get_check->req);
 			new_req = true;
 		} else
 			new_req = false;
@@ -1582,8 +1624,10 @@ http_check_thread(thread_ref_t thread)
 			if (thread->type != THREAD_WRITE_TIMEOUT &&
 			    thread->type != THREAD_READ_TIMEOUT)
 				ret = ssl_connect(thread, new_req);
-			else
-				return timeout_epilog(thread, "Timeout connecting");
+			else {
+				timeout_epilog(thread, "Timeout connecting");
+				return;
+			}
 
 			if (ret == -1) {
 				switch ((ssl_err = SSL_get_error(http_get_check->req->ssl,
@@ -1633,16 +1677,15 @@ http_check_thread(thread_ref_t thread)
 			if (http_get_check->proto == PROTO_SSL)
 				ssl_printerr(SSL_get_error (http_get_check->req->ssl, ret));
 #endif
-			return timeout_epilog(thread, "SSL handshake/communication error"
+			timeout_epilog(thread, "SSL handshake/communication error"
 						 " connecting to");
+			return;
 		}
 		break;
 	}
-
-	return 0;
 }
 
-static int
+static void
 http_connect_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -1659,13 +1702,15 @@ http_connect_thread(thread_ref_t thread)
 	if (!checker->enabled) {
 		thread_add_timer(thread->master, http_connect_thread, checker,
 				 checker->delay_loop);
-		return 0;
+		return;
 	}
 
 	/* if there are no URLs in list, enable server w/o checking */
 	fetched_url = fetch_next_url(http_get_check);
-	if (!fetched_url)
-		return epilog(thread, REGISTER_CHECKER_NEW) + 1;
+	if (!fetched_url) {
+		epilog(thread, REGISTER_CHECKER_NEW);
+		return;
+	}
 
 	/* Create the socket */
 	if ((fd = socket(co->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_TCP)) == -1) {
@@ -1673,7 +1718,7 @@ http_connect_thread(thread_ref_t thread)
 		thread_add_timer(thread->master, http_connect_thread, checker,
 				checker->delay_loop);
 
-		return 0;
+		return;
 	}
 
 #if !HAVE_DECL_SOCK_NONBLOCK
@@ -1700,8 +1745,6 @@ http_connect_thread(thread_ref_t thread)
 					checker->delay_loop);
 		}
 	}
-
-	return 0;
 }
 
 #ifdef THREAD_DUMP
