@@ -48,6 +48,11 @@ static void udp_connect_thread(thread_ref_t);
 static void
 free_udp_check(checker_t *checker)
 {
+	udp_check_t *udp_check = CHECKER_ARG(checker);
+
+	FREE_PTR(udp_check->payload);
+	FREE_PTR(udp_check->reply_data);
+	FREE_PTR(udp_check->reply_mask);
 	FREE(checker->co);
 	FREE(checker->data);
 	FREE(checker);
@@ -59,38 +64,123 @@ dump_udp_check(FILE *fp, const checker_t *checker)
 	udp_check_t *udp_check = CHECKER_ARG(checker);
 
 	conf_write(fp, "   Keepalive method = UDP_CHECK");
-	dump_checker_opts(fp, checker);
+
+	if (udp_check->payload)
+		conf_write(fp, "   Payload len = %u", udp_check->payload_len);
+	else
+		conf_write(fp, "   Payload specified = no");
+
 	conf_write(fp, "   Require reply = %s", udp_check->require_reply ? "yes" : "no");
+	if (udp_check->require_reply) {
+		conf_write(fp, "     Min reply length = %u", udp_check->min_reply_len);
+		conf_write(fp, "     Max reply length = %u", udp_check->max_reply_len);
+		conf_write(fp, "     Reply data len = %u", udp_check->reply_len);
+		if (udp_check->reply_data)
+			conf_write(fp, "     Reply data mask = %s", udp_check->reply_mask ? "yes" : "no");
+	}
 }
 
 static bool
-udp_check_compare(const checker_t *a, checker_t *b)
+compare_udp_check(const checker_t *a, checker_t *b)
 {
 	return compare_conn_opts(a->co, b->co);
 }
 
+static const checker_funcs_t udp_checker_funcs = { CHECKER_UDP, free_udp_check, dump_udp_check, compare_udp_check, NULL };
+
 static void
 udp_check_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	udp_check_t *udp_check = MALLOC(sizeof (udp_check_t));
 
+	udp_check->min_reply_len = 0;
+	udp_check->max_reply_len = UINT8_MAX;
+
 	/* queue new checker */
-	queue_checker(free_udp_check, dump_udp_check, udp_connect_thread,
-		      udp_check_compare, udp_check, CHECKER_NEW_CO(), true);
+	queue_checker(&udp_checker_funcs, udp_connect_thread, udp_check, CHECKER_NEW_CO(), true);
 }
+
 static void
-require_reply_handler(__attribute__((unused)) const vector_t *strvec)
+payload_handler(const vector_t *strvec)
 {
 	udp_check_t *udp_check = CHECKER_GET();
+	char *hex_str;
+
+	if (vector_size(strvec) == 1) {
+		report_config_error(CONFIG_GENERAL_ERROR, "UDP_CHECK payload requires a payload");
+		return;
+	}
+
+	hex_str = make_strvec_str(strvec, 1);
+
+	udp_check->payload_len = read_hex_str(hex_str, &udp_check->payload, NULL);
+	if (!udp_check->payload_len)
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid hex string for UDP_CHECK payload");
+
+	FREE_ONLY(hex_str);
+}
+
+static void
+require_reply_handler(const vector_t *strvec)
+{
+	udp_check_t *udp_check = CHECKER_GET();
+	char *hex_str;
 
 	udp_check->require_reply = true;
+
+	if (vector_size(strvec) == 1)
+		return;
+
+	hex_str = make_strvec_str(strvec, 1);
+
+	udp_check->reply_len = read_hex_str(hex_str, &udp_check->reply_data, &udp_check->reply_mask);
+	if (!udp_check->reply_len)
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid hex string for UDP_CHECK reply");
+
+	FREE_ONLY(hex_str);
+}
+
+static void
+min_length_handler(const vector_t *strvec)
+{
+	udp_check_t *udp_check = CHECKER_GET();
+	unsigned len;
+
+	if (!read_unsigned_strvec(strvec, 1, &len, 0, UINT16_MAX, false)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "UDP_CHECK min length %s not valid - must be between 0 & %d", strvec_slot(strvec, 1), UINT16_MAX);
+                return;
+        }
+
+	udp_check->min_reply_len = len;
+}
+
+static void
+max_length_handler(const vector_t *strvec)
+{
+	udp_check_t *udp_check = CHECKER_GET();
+	unsigned len;
+
+	if (!read_unsigned_strvec(strvec, 1, &len, 0, UINT16_MAX, false)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "UDP_CHECK max length %s not valid - must be between 0 & %d", strvec_slot(strvec, 1), UINT16_MAX);
+                return;
+        }
+
+	udp_check->max_reply_len = len;
 }
 
 static void
 udp_check_end_handler(void)
 {
-	if (!check_conn_opts(CHECKER_GET_CO()))
+	udp_check_t *udp_check = CHECKER_GET();
+
+	if (!check_conn_opts(CHECKER_GET_CO())) {
 		dequeue_new_checker();
+		return;
+	}
+
+	if (udp_check->min_reply_len > udp_check->max_reply_len)
+		report_config_error(CONFIG_GENERAL_ERROR, "UDP_CHECK min_reply length %d > max_reply_length %d - will always fail",
+				    udp_check->min_reply_len, udp_check->max_reply_len);
 }
 
 void
@@ -100,7 +190,10 @@ install_udp_check_keyword(void)
 	install_keyword("UDP_CHECK", &udp_check_handler);
 	install_sublevel();
 	install_checker_common_keywords(true);
+	install_keyword("payload", &payload_handler);
 	install_keyword("require_reply", &require_reply_handler);
+	install_keyword("min_reply_length", &min_length_handler);
+	install_keyword("max_reply_length", &max_length_handler);
 	install_sublevel_end_handler(udp_check_end_handler);
 	install_sublevel_end();
 }
@@ -158,20 +251,58 @@ udp_epilog(thread_ref_t thread, bool is_success)
 	thread_add_timer(thread->master, udp_connect_thread, checker, delay);
 }
 
+static bool
+check_udp_reply(const uint8_t *recv_data, size_t len, const udp_check_t *udp_check)
+{
+	unsigned i;
+	unsigned check_len;
+
+	if (len < udp_check->min_reply_len ||
+	    len > udp_check->max_reply_len)
+		return true;
+
+	/* We only checker lesser of len and udp_check->reply_len octets */
+	check_len = udp_check->reply_len;
+	if (len < check_len)
+		check_len = len;
+
+	/* Check the received data matches */
+	for (i = 0; i < check_len; i++) {
+		if ((recv_data[i] ^ udp_check->reply_data[i]) & (udp_check->reply_mask ? ~udp_check->reply_mask[i] : ~0))
+			return true;
+	}
+
+	/* Success */
+	return false;
+}
+
 static void
 udp_check_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	udp_check_t *udp_check = CHECKER_ARG(checker);
 	int status;
+	uint8_t *recv_buf;
+	size_t len = udp_check->max_reply_len + 1;
 
-	status = udp_socket_state(thread->u.f.fd, thread, udp_check->require_reply);
+	recv_buf = udp_check->require_reply ?
+			MALLOC(udp_check->max_reply_len + 1) :
+			NULL;
+
+	status = udp_socket_state(thread->u.f.fd, thread, recv_buf, &len);
 
 	thread_close_fd(thread);
 
-	if (status == connect_success)
-		udp_epilog(thread, true);
-	else {
+	if (status == connect_success) {
+		if (udp_check->reply_data && check_udp_reply(recv_buf, len, udp_check)) {
+			if (checker->is_up &&
+			    (global_data->checker_log_all_failures || checker->log_all_failures))
+				log_message(LOG_INFO, "UDP check to %s reply data mismatch."
+						, FMT_CHK(checker));
+			udp_epilog(thread, false);
+		} else
+			udp_epilog(thread, true);
+	} else {
 		if (checker->is_up &&
 		    (global_data->checker_log_all_failures || checker->log_all_failures))
 			log_message(LOG_INFO, "UDP connection to %s failed."
@@ -179,6 +310,8 @@ udp_check_thread(thread_ref_t thread)
 		udp_epilog(thread, false);
 	}
 
+	FREE(recv_buf);
+
 	return;
 }
 
@@ -186,6 +319,7 @@ static void
 udp_connect_thread(thread_ref_t thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
+	udp_check_t *udp_check = CHECKER_ARG(checker);
 	conn_opts_t *co = checker->co;
 	int fd;
 	int status;
@@ -217,7 +351,7 @@ udp_connect_thread(thread_ref_t thread)
 		log_message(LOG_INFO, "Unable to set CLOEXEC on icmp_connect socket - %s (%d)", strerror(errno), errno);
 #endif
 
-	status = udp_bind_connect(fd, co);
+	status = udp_bind_connect(fd, co, udp_check->payload, udp_check->payload_len);
 
 	/* handle udp connection status & register check worker thread */
 	if (udp_icmp_check_state(fd, status, thread, udp_check_thread, co->connection_to)) {
