@@ -31,8 +31,9 @@
 #include <netlink/genl/genl.h>
 #include <netlink/genl/ctrl.h>
 
+//#define LIBNL_DEBUG
+
 #ifdef _HAVE_LIBNL1_
-#define nl_sock		nl_handle
 #ifndef _LIBNL_DYNAMIC_
 #define nl_socket_alloc	nl_handle_alloc
 #define nl_socket_free	nl_handle_destroy
@@ -53,6 +54,10 @@
 #include "logger.h"
 #include "utils.h"
 
+#if HAVE_DECL_CLONE_NEWNET
+#include "namespaces.h"
+#include "global_data.h"
+#endif
 
 typedef struct ipvs_servicedest_s {
 	struct ip_vs_service_user	svc;
@@ -61,15 +66,16 @@ typedef struct ipvs_servicedest_s {
 
 static int sockfd = -1;
 static void* ipvs_func = NULL;
-
+static ipvs_timeout_t orig_ipvs_timeouts;
 
 #ifdef LIBIPVS_USE_NL
 static struct nl_sock *sock = NULL;
+static nl_recvmsg_msg_cb_t cur_nl_sock_cb_func;
 static int family;
 static bool try_nl = true;
+static int nl_ack_flag;
 
 /* Policy definitions */
-#ifdef _WITH_SNMP_CHECKER_
 static struct nla_policy ipvs_cmd_policy[IPVS_CMD_ATTR_MAX + 1] = {
 	[IPVS_CMD_ATTR_SERVICE]		= { .type = NLA_NESTED },
 	[IPVS_CMD_ATTR_DEST]		= { .type = NLA_NESTED },
@@ -79,6 +85,7 @@ static struct nla_policy ipvs_cmd_policy[IPVS_CMD_ATTR_MAX + 1] = {
 	[IPVS_CMD_ATTR_TIMEOUT_UDP]	= { .type = NLA_U32 },
 };
 
+#ifdef _WITH_SNMP_CHECKER_
 static struct nla_policy ipvs_service_policy[IPVS_SVC_ATTR_MAX + 1] = {
 	[IPVS_SVC_ATTR_AF]		= { .type = NLA_U16 },
 	[IPVS_SVC_ATTR_PROTOCOL]	= { .type = NLA_U16 },
@@ -87,7 +94,7 @@ static struct nla_policy ipvs_service_policy[IPVS_SVC_ATTR_MAX + 1] = {
 	[IPVS_SVC_ATTR_PORT]		= { .type = NLA_U16 },
 	[IPVS_SVC_ATTR_FWMARK]		= { .type = NLA_U32 },
 	[IPVS_SVC_ATTR_SCHED_NAME]	= { .type = NLA_STRING,
-					    .maxlen = IP_VS_SCHEDNAME_MAXLEN },
+					    .maxlen = IP_VS_SCHEDNAME_MAXLEN - 1 },
 	[IPVS_SVC_ATTR_FLAGS]		= { .type = NLA_UNSPEC,
 					    .minlen = sizeof(struct ip_vs_flags),
 					    .maxlen = sizeof(struct ip_vs_flags) },
@@ -236,61 +243,162 @@ static struct nl_msg *ipvs_nl_message(uint8_t cmd, int flags)
 	return msg;
 }
 
+#ifdef LIBNL_DEBUG
+static void
+dump_nl_msg(const char *msg, struct nl_msg *nlmsg)
+{
+	FILE *fp;
+
+	fp = fopen(KA_TMP_DIR "/nlmsg.dmp", "a");
+	fprintf(fp, "\n%s\n\n", msg);
+	if (nlmsg)
+		nl_msg_dump(nlmsg, fp);
+	fclose(fp);
+}
+#endif
+
 static int ipvs_nl_noop_cb(__attribute__((unused)) struct nl_msg *msg, __attribute__((unused)) void *arg)
 {
+#ifdef LIBNL_DEBUG
+	dump_nl_msg("Noop CB", msg);
+#endif
+
 	return NL_OK;
 }
 
+#ifdef LIBNL_DEBUG
+static int recv_cb(struct nl_msg *msg, __attribute__((unused)) void *arg)
+{
+	dump_nl_msg("Receive message", msg);
+
+	return NL_OK;
+}
+#endif
+
+static int recv_ack_cb(__attribute__((unused)) struct nl_msg *msg, void *arg)
+{
+	int *ack_flag = arg;
+
+#ifdef LIBNL_DEBUG
+	dump_nl_msg("That was an ACK message", NULL);
+#endif
+
+	*ack_flag = 1;
+
+	return NL_STOP;
+}
+
+static int finish_cb(__attribute__((unused)) struct nl_msg *msg, void *arg)
+{
+	int *ack_flag = arg;
+
+#ifdef LIBNL_DEBUG
+	dump_nl_msg("That was a multi done message", NULL);
+#endif
+
+	*ack_flag = 1;
+
+	return NL_STOP;
+}
+
+static int
+ipvs_nl_err_cb(__attribute__((unused)) struct sockaddr_nl *nla, __attribute__((unused)) struct nlmsgerr *nlerr, void *arg)
+{
+	int *ack_flag = arg;
+
+#ifdef LIBNL_DEBUG
+	dump_nl_msg("That was an ERROR message", NULL);
+#endif
+
+	*ack_flag = 1;
+
+	return NL_STOP;
+}
+
+static int
+open_nl_sock(void)
+{
+	if (!(sock = nl_socket_alloc()))
+		return -1;
+
+	if (
+#if HAVE_DECL_CLONE_NEWNET
+	    nl_ipvs_connect(global_data->network_namespace_ipvs, sock)
+#else
+	    genl_connect(sock)
+#endif
+				    < 0 ||
+	    (family = genl_ctrl_resolve(sock, IPVS_GENL_NAME)) < 0) {
+		nl_socket_free(sock);
+		sock = NULL;
+
+		return -1;
+	}
+
+	/* We finish receiving if we get an error, an ACK, or a DONE for a multipart message */
+#ifndef _HAVE_LIBNL1_
+	if (nl_socket_modify_err_cb(sock, NL_CB_CUSTOM, ipvs_nl_err_cb, &nl_ack_flag))
+#else
+	if (nl_cb_err(nl_socket_get_cb(sock), NL_CB_CUSTOM, ipvs_nl_err_cb, &nl_ack_flag))
+#endif
+		log_message(LOG_INFO, "Setting err_cb failed");
+
+	nl_socket_modify_cb(sock, NL_CB_ACK, NL_CB_CUSTOM, recv_ack_cb, &nl_ack_flag);
+	nl_socket_modify_cb(sock, NL_CB_FINISH, NL_CB_CUSTOM, finish_cb, &nl_ack_flag);
+
+#ifdef LIBNL_DEBUG
+	nl_socket_modify_cb(sock, NL_CB_MSG_IN, NL_CB_CUSTOM, recv_cb, 0);
+#endif
+
+	return 0;
+}
+
 static int ipvs_nl_send_message(struct nl_msg *msg, nl_recvmsg_msg_cb_t func, void *arg)
 {
 	int err = EINVAL;
+	int ret = 0;
 
-	sock = nl_socket_alloc();
-	if (!sock) {
-		if (msg)
-			nlmsg_free(msg);
+	if (!sock && open_nl_sock()) {
+		nlmsg_free(msg);
 		return -1;
 	}
 
-	if (genl_connect(sock) < 0)
-		goto fail_genl;
-
-	family = genl_ctrl_resolve(sock, IPVS_GENL_NAME);
-	if (family < 0)
-		goto fail_genl;
-
-	/* To test connections and set the family */
-	if (msg == NULL) {
-		nl_socket_free(sock);
-		sock = NULL;
+	if (!msg)
 		return 0;
+
+	if (func != cur_nl_sock_cb_func) {
+		if (!nl_socket_modify_cb(sock, NL_CB_VALID, NL_CB_CUSTOM, func, arg))
+			cur_nl_sock_cb_func = func;
+		else
+			log_message(LOG_INFO, "Setting libnl callback function failed");
 	}
 
-	if (nl_socket_modify_cb(sock, NL_CB_VALID, NL_CB_CUSTOM, func, arg) != 0)
-		goto fail_genl;
-
-	if (nl_send_auto_complete(sock, msg) < 0)
-		goto fail_genl;
-
-	if ((err = -nl_recvmsgs_default(sock)) > 0)
-		goto fail_genl;
-
-	nlmsg_free(msg);
-
-	nl_socket_free(sock);
-
-	return 0;
-
-fail_genl:
-	nl_socket_free(sock);
-	sock = NULL;
-	nlmsg_free(msg);
-#ifdef _HAVE_LIBNL1_
-	errno = err;
-#else
-	errno = nlerr2syserr(err);
+#ifdef LIBNL_DEBUG
+	dump_nl_msg("Sending message", msg);
 #endif
-	return -1;
+
+#ifndef _HAVE_LIBNL1_
+	if (nl_send_auto(sock, msg) >= 0) {
+#else
+	if (nl_send_auto_complete(sock, msg) >= 0) {
+#endif
+		nl_ack_flag = 0;
+		do {
+			if ((err = -nl_recvmsgs_default(sock)) > 0) {
+#ifdef _HAVE_LIBNL1_
+				errno = err;
+#else
+				errno = nlerr2syserr(err);
+#endif
+
+				ret = -1;
+			}
+		} while (!nl_ack_flag);
+	}
+
+	nlmsg_free(msg);
+
+	return ret;
 }
 #endif
 
@@ -301,11 +409,11 @@ static int ipvs_getinfo_parse_cb(struct nl_msg *msg, __attribute__((unused)) voi
 	struct nlattr *attrs[IPVS_INFO_ATTR_MAX + 1];
 
 	if (genlmsg_parse(nlh, 0, attrs, IPVS_INFO_ATTR_MAX, ipvs_info_policy) != 0)
-		return -1;
+		return NL_STOP;
 
 	if (!(attrs[IPVS_INFO_ATTR_VERSION] &&
 	      attrs[IPVS_INFO_ATTR_CONN_TAB_SIZE]))
-		return -1;
+		return NL_STOP;
 
 	return NL_OK;
 }
@@ -319,10 +427,10 @@ static int ipvs_getinfo(void)
 
 	if (try_nl) {
 		struct nl_msg *msg;
-		msg = ipvs_nl_message(IPVS_CMD_GET_INFO, 0);
-		if (msg)
-			return ipvs_nl_send_message(msg, ipvs_getinfo_parse_cb, NULL);
-		return -1;
+		if (!(msg = ipvs_nl_message(IPVS_CMD_GET_INFO, 0)))
+			return -1;
+
+		return ipvs_nl_send_message(msg, ipvs_getinfo_parse_cb, NULL);
 	}
 
 	len = sizeof(ipvs_info);
@@ -353,12 +461,18 @@ int ipvs_init(void)
 	try_nl = false;
 #endif
 
-	if ((sockfd = socket(AF_INET, SOCK_RAW | SOCK_CLOEXEC, IPPROTO_RAW)) == -1)
+#if HAVE_DECL_CLONE_NEWNET
+	sockfd = socket_netns_name(global_data->network_namespace_ipvs, AF_INET, SOCK_RAW | SOCK_CLOEXEC, IPPROTO_RAW);
+#else
+	sockfd = socket(AF_INET, SOCK_RAW | SOCK_CLOEXEC, IPPROTO_RAW);
+#endif
+	if (sockfd == -1)
 		return -1;
 
 #if !HAVE_DECL_SOCK_CLOEXEC
 	if (set_sock_flags(sockfd, F_SETFD, FD_CLOEXEC)) {
 		close(sockfd);
+		sockfd = -1;
 		return -1;
 	}
 #endif
@@ -366,6 +480,7 @@ int ipvs_init(void)
 	len = sizeof(ipvs_info);
 	if (getsockopt(sockfd, IPPROTO_IP, IP_VS_SO_GET_INFO, (char *)&ipvs_info, &len)) {
 		close(sockfd);
+		sockfd = -1;
 		return -1;
 	}
 
@@ -490,7 +605,7 @@ out_err:
 	return -1;
 }
 
-
+#ifdef _INCLUDE_UNUSED_CODE_
 int ipvs_zero_service(ipvs_service_t *svc)
 {
 	ipvs_func = ipvs_zero_service;
@@ -516,6 +631,7 @@ int ipvs_zero_service(ipvs_service_t *svc)
 out_err:
 	return -1;
 }
+#endif
 
 #ifdef LIBIPVS_USE_NL
 static int ipvs_nl_fill_dest_attr(struct nl_msg *msg, ipvs_dest_t *dst)
@@ -650,21 +766,72 @@ out_err:
 	return -1;
 }
 
+#ifdef LIBIPVS_USE_NL
+static int ipvs_timeout_parse_cb(struct nl_msg *msg, void *arg)
+{
+	struct nlmsghdr *nlh = nlmsg_hdr(msg);
+	struct nlattr *attrs[IPVS_CMD_ATTR_MAX + 1];
+	ipvs_timeout_t *u = PTR_CAST(ipvs_timeout_t, arg);
 
-int ipvs_set_timeout(ipvs_timeout_t *to)
+	if (genlmsg_parse(nlh, 0, attrs, IPVS_CMD_ATTR_MAX, ipvs_cmd_policy) != 0)
+		return -1;
+
+	if (attrs[IPVS_CMD_ATTR_TIMEOUT_TCP])
+		u->tcp_timeout = nla_get_u32(attrs[IPVS_CMD_ATTR_TIMEOUT_TCP]);
+	if (attrs[IPVS_CMD_ATTR_TIMEOUT_TCP_FIN])
+		u->tcp_fin_timeout = nla_get_u32(attrs[IPVS_CMD_ATTR_TIMEOUT_TCP_FIN]);
+	if (attrs[IPVS_CMD_ATTR_TIMEOUT_UDP])
+		u->udp_timeout = nla_get_u32(attrs[IPVS_CMD_ATTR_TIMEOUT_UDP]);
+
+	return NL_OK;
+}
+#endif
+
+static void
+ipvs_get_timeout(void)
+{
+	socklen_t len;
+
+	ipvs_func = ipvs_get_timeout;
+#ifdef LIBIPVS_USE_NL
+	if (try_nl) {
+		struct nl_msg *msg;
+		msg = ipvs_nl_message(IPVS_CMD_GET_CONFIG, 0);
+		if (!msg || ipvs_nl_send_message(msg, ipvs_timeout_parse_cb, &orig_ipvs_timeouts))
+			log_message(LOG_INFO, "Failed to get IPVS timeouts");
+
+		return;
+	}
+#endif
+
+	len = sizeof(orig_ipvs_timeouts);
+	if (getsockopt(sockfd, IPPROTO_IP, IP_VS_SO_GET_TIMEOUT,
+		       (char *)&orig_ipvs_timeouts, &len)) {
+		log_message(LOG_INFO, "Failed to get IPVS timeouts");
+		return;
+	}
+}
+
+int ipvs_set_timeout(const ipvs_timeout_t *to)
 {
 	ipvs_func = ipvs_set_timeout;
+
+	/* If we are altering the timeouts, ensure we can restore the original values */
+	if (!orig_ipvs_timeouts.tcp_timeout) {
+		if (!to)
+			return 0;
+
+		ipvs_get_timeout();
+	}
+
 #ifdef LIBIPVS_USE_NL
 	if (try_nl) {
 		struct nl_msg *msg = ipvs_nl_message(IPVS_CMD_SET_CONFIG, 0);
 		if (!msg) return -1;
 
-		if (to->tcp_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP, (uint32_t)to->tcp_timeout);
-		if (to->tcp_fin_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP_FIN, (uint32_t)to->tcp_fin_timeout);
-		if (to->udp_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_UDP, (uint32_t)to->udp_timeout);
+		NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP, to && to->tcp_timeout ? (uint32_t)to->tcp_timeout : (uint32_t)orig_ipvs_timeouts.tcp_timeout);
+		NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP_FIN, to && to->tcp_fin_timeout ? (uint32_t)to->tcp_fin_timeout : (uint32_t)orig_ipvs_timeouts.tcp_fin_timeout);
+		NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_UDP, to && to->udp_timeout ? (uint32_t)to->udp_timeout : (uint32_t)orig_ipvs_timeouts.udp_timeout);
 		return ipvs_nl_send_message(msg, ipvs_nl_noop_cb, NULL);
 
 nla_put_failure:
@@ -672,7 +839,7 @@ nla_put_failure:
 		return -1;
 	}
 #endif
-	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_TIMEOUT, (char *)to,
+	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_TIMEOUT, (const char *)to,
 			  sizeof(*to));
 }
 
@@ -718,9 +885,9 @@ nla_put_failure:
 	}
 #endif
 	memset(&dmk, 0, sizeof(dmk));
-	dmk.state = (int)dm->state;
+	dmk.state = dm->state;
 	strcpy(dmk.mcast_ifn, dm->mcast_ifn);
-	dmk.syncid = (int)dm->syncid;
+	dmk.syncid = dm->syncid;
 	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_STARTDAEMON,
 			  (char *)&dmk, sizeof(dmk));
 }
@@ -742,7 +909,6 @@ int ipvs_stop_daemon(ipvs_daemon_t *dm)
 			goto nla_put_failure;
 
 		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_STATE, dm->state);
-		NLA_PUT_STRING(msg, IPVS_DAEMON_ATTR_MCAST_IFN, dm->mcast_ifn);
 		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_SYNC_ID, dm->syncid);
 
 		nla_nest_end(msg, nl_daemon);
@@ -755,7 +921,7 @@ nla_put_failure:
 	}
 #endif
 	memset(&dmk, 0, sizeof(dmk));
-	dmk.state = (int)dm->state;
+	dmk.state = dm->state;
 	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_STOPDAEMON,
 			  (char *)&dmk, sizeof(dmk));
 }
@@ -836,8 +1002,8 @@ static int ipvs_services_parse_cb(struct nl_msg *msg, void *arg)
 	struct nlmsghdr *nlh = nlmsg_hdr(msg);
 	struct nlattr *attrs[IPVS_CMD_ATTR_MAX + 1];
 	struct nlattr *svc_attrs[IPVS_SVC_ATTR_MAX + 1];
-	struct ip_vs_get_services_app **getp = (struct ip_vs_get_services_app **)arg;
-	struct ip_vs_get_services_app *get = (struct ip_vs_get_services_app *)*getp;
+	struct ip_vs_get_services_app **getp = PTR_CAST(struct ip_vs_get_services_app *, arg);
+	struct ip_vs_get_services_app *get = PTR_CAST(struct ip_vs_get_services_app, *getp);
 	struct ip_vs_flags flags;
 	unsigned i = get->user.num_services;
 
@@ -906,7 +1072,7 @@ static int ipvs_services_parse_cb(struct nl_msg *msg, void *arg)
 	i++;
 
 	get->user.num_services = i;
-	get = realloc(get, sizeof(*get)
+	get = REALLOC(get, sizeof(*get)
 	      + sizeof(ipvs_service_entry_t) * (get->user.num_services + 1));
 	*getp = get;
 	return 0;
@@ -921,8 +1087,8 @@ static int ipvs_dests_parse_cb(struct nl_msg *msg, void *arg)
 #if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
 	struct nlattr *attr_addr_family = NULL;
 #endif
-	struct ip_vs_get_dests_app **dp = (struct ip_vs_get_dests_app **)arg;
-	struct ip_vs_get_dests_app *d = (struct ip_vs_get_dests_app *)*dp;
+	struct ip_vs_get_dests_app **dp = PTR_CAST(struct ip_vs_get_dests_app *, arg);
+	struct ip_vs_get_dests_app *d = PTR_CAST(struct ip_vs_get_dests_app, *dp);
 	unsigned i = d->user.num_dests;
 
 	if (genlmsg_parse(nlh, 0, attrs, IPVS_CMD_ATTR_MAX, ipvs_cmd_policy) != 0)
@@ -982,7 +1148,7 @@ static int ipvs_dests_parse_cb(struct nl_msg *msg, void *arg)
 	i++;
 
 	d->user.num_dests = i;
-	d = realloc(d, sizeof(*d) + sizeof(ipvs_dest_entry_t) * (d->user.num_dests + 1));
+	d = REALLOC(d, sizeof(*d) + sizeof(ipvs_dest_entry_t) * (d->user.num_dests + 1));
 	*dp = d;
 	return 0;
 }
@@ -1006,7 +1172,7 @@ struct ip_vs_get_dests_app *ipvs_get_dests(ipvs_service_entry_t *svc)
 		struct nl_msg *msg;
 		struct nlattr *nl_service;
 		if (svc->user.num_dests == 0)
-			d = realloc(d,sizeof(*d) + sizeof(ipvs_dest_entry_t));
+			d = REALLOC(d,sizeof(*d) + sizeof(ipvs_dest_entry_t));
 		d->user.fwmark = svc->user.fwmark;
 		d->user.protocol = svc->user.protocol;
 		d->nf_addr = svc->nf_addr;
@@ -1107,6 +1273,7 @@ ipvs_get_service(__u32 fwmark, __u16 af, __u16 protocol, union nf_inet_addr *add
 		tsvc.user.fwmark = fwmark;
 		tsvc.af = af;
 		tsvc.user.protocol= protocol;
+// ?IPv6
 		tsvc.nf_addr = *addr;
 		tsvc.user.port = port;
 
@@ -1169,8 +1336,13 @@ out_err:
 void ipvs_close(void)
 {
 #ifdef LIBIPVS_USE_NL
-	if (try_nl)
+	if (try_nl) {
+		if (sock) {
+			nl_socket_free(sock);
+			sock = NULL;
+		}
 		return;
+	}
 #endif
 	if (sockfd != -1) {
 		close(sockfd);
@@ -1188,29 +1360,20 @@ const char *ipvs_strerror(int err)
 	} table [] = {
 		{ ipvs_add_service, EEXIST, "Service already exists" },
 		{ ipvs_add_service, ENOENT, "Scheduler or persistence engine not found" },
-		{ ipvs_update_service, ESRCH, "No such service" },
 		{ ipvs_update_service, ENOENT, "Scheduler or persistence engine not found" },
-		{ ipvs_del_service, ESRCH, "No such service" },
-		{ ipvs_zero_service, ESRCH, "No such service" },
-		{ ipvs_add_dest, ESRCH, "Service not defined" },
 		{ ipvs_add_dest, EEXIST, "Destination already exists" },
-		{ ipvs_update_dest, ESRCH, "Service not defined" },
 		{ ipvs_update_dest, ENOENT, "No such destination" },
-		{ ipvs_del_dest, ESRCH, "Service not defined" },
 		{ ipvs_del_dest, ENOENT, "No such destination" },
 		{ ipvs_start_daemon, EEXIST, "Daemon has already run" },
 		{ ipvs_stop_daemon, ESRCH, "No daemon is running" },
-#ifdef _WITH_SNMP_CHECKER_
-		{ ipvs_get_dests, ESRCH, "No such service" },
-		{ ipvs_get_service, ESRCH, "No such service" },
-#endif
-		{ 0, EPERM, "Permission denied (you must be root)" },
-		{ 0, EINVAL, "Invalid operation.  Possibly wrong module version, address not unicast, ..." },
-		{ 0, ENOPROTOOPT, "Protocol not available" },
-		{ 0, ENOMEM, "Memory allocation problem" },
-		{ 0, EOPNOTSUPP, "Operation not supported with IPv6" },
-		{ 0, EAFNOSUPPORT, "Operation not supported with specified address family" },
-		{ 0, EMSGSIZE, "Module is wrong version" },
+		{ NULL, ESRCH, "No such service" },
+		{ NULL, EPERM, "Permission denied (you must be root)" },
+		{ NULL, EINVAL, "Invalid operation.  Possibly wrong module version, address not unicast, ..." },
+		{ NULL, ENOPROTOOPT, "Protocol not available" },
+		{ NULL, ENOMEM, "Memory allocation problem" },
+		{ NULL, EOPNOTSUPP, "Operation not supported with IPv6" },
+		{ NULL, EAFNOSUPPORT, "Operation not supported with specified address family" },
+		{ NULL, EMSGSIZE, "Module is wrong version" },
 	};
 
 	for (i = 0; i < sizeof(table)/sizeof(struct table_struct); i++) {
