@@ -47,10 +47,14 @@
 #include "smtp.h"
 #include "utils.h"
 #include "logger.h"
+#include "bitops.h"
 #ifdef _WITH_FIREWALL_
 #include "vrrp_firewall.h"
 #endif
 #include "memory.h"
+#ifdef _WITH_VRRP_
+#include "vrrp_daemon.h"
+#endif
 
 #if HAVE_DECL_CLONE_NEWNET
 #include "namespaces.h"
@@ -74,6 +78,7 @@ use_polling_handler(const vector_t *strvec)
 	global_data->linkbeat_use_polling = true;
 }
 #endif
+
 static void
 save_process_name(char const **dest, const char *src)
 {
@@ -121,10 +126,18 @@ vrrp_process_name_handler(const vector_t *strvec)
 #endif
 #ifdef _WITH_LVS_
 static void
-lvs_process_name_handler(const vector_t *strvec)
+checker_process_name_handler(const vector_t *strvec)
 {
 	save_process_name(&global_data->lvs_process_name, strvec_slot(strvec, 1));
 }
+static void
+lvs_process_name_handler(const vector_t *strvec)
+{
+	/* Deprecated since 12/07/20 */
+	log_message(LOG_INFO, "'lvs_process_name' is deprecated - please use 'checker_process_name'");
+
+	checker_process_name_handler(strvec);
+}
 #endif
 #ifdef _WITH_BFD_
 static void
@@ -417,6 +430,7 @@ checker_log_all_failures_handler(const vector_t *strvec)
 	global_data->checker_log_all_failures = res;
 }
 #endif
+
 #ifdef _WITH_VRRP_
 static void
 default_interface_handler(const vector_t *strvec)
@@ -428,6 +442,16 @@ default_interface_handler(const vector_t *strvec)
 	FREE_CONST_PTR(global_data->default_ifname);
 	global_data->default_ifname = set_value(strvec);
 }
+static void
+disable_local_igmp_handler(__attribute__((unused)) const vector_t *strvec)
+{
+	if (access(igmp_link_local_mcast_reports, W_OK)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "kernel does not support %s", igmp_link_local_mcast_reports);
+		return;
+	}
+
+	global_data->disable_local_igmp = true;
+}
 #endif
 #ifdef _WITH_LVS_
 static void
@@ -575,8 +599,8 @@ lvs_syncd_handler(const vector_t *strvec)
 			if (inet_stosockaddr(strvec_slot(strvec, i+1), NULL, &global_data->lvs_syncd.mcast_group))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_sync_daemon group (%s) - ignoring", strvec_slot(strvec, i+1));
 
-			if ((global_data->lvs_syncd.mcast_group.ss_family == AF_INET  && !IN_MULTICAST(htonl(((struct sockaddr_in *)&global_data->lvs_syncd.mcast_group)->sin_addr.s_addr))) ||
-			    (global_data->lvs_syncd.mcast_group.ss_family == AF_INET6 && !IN6_IS_ADDR_MULTICAST(&((struct sockaddr_in6 *)&global_data->lvs_syncd.mcast_group)->sin6_addr))) {
+			if ((global_data->lvs_syncd.mcast_group.ss_family == AF_INET  && !IN_MULTICAST(htonl(PTR_CAST(struct sockaddr_in, &global_data->lvs_syncd.mcast_group)->sin_addr.s_addr))) ||
+			    (global_data->lvs_syncd.mcast_group.ss_family == AF_INET6 && !IN6_IS_ADDR_MULTICAST(&PTR_CAST(struct sockaddr_in6, &global_data->lvs_syncd.mcast_group)->sin6_addr))) {
 				report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon group address %s is not multicast - ignoring", strvec_slot(strvec, i+1));
 				global_data->lvs_syncd.mcast_group.ss_family = AF_UNSPEC;
 			}
@@ -596,7 +620,7 @@ lvs_syncd_handler(const vector_t *strvec)
 			if (!read_unsigned_strvec(strvec, 3, &val, 0, 255, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid syncid (%s) - defaulting to vrid", strvec_slot(strvec, 3));
 			else {
-				report_config_error(CONFIG_GENERAL_ERROR, "Please use keyword \"id\" before lvs_sync_daemon syncid value");
+				report_config_error(CONFIG_GENERAL_ERROR, "Please use keyword \"id\" before lvs_sync_daemon SYNCID");
 				global_data->lvs_syncd.syncid = val;
 			}
 
@@ -615,14 +639,14 @@ lvs_flush_handler(__attribute__((unused)) const vector_t *strvec)
 }
 
 static void
-lvs_flush_onstop_handler(const vector_t *strvec)
+lvs_flush_on_stop_handler(const vector_t *strvec)
 {
 	if (vector_size(strvec) == 1)
-		global_data->lvs_flush_onstop = LVS_FLUSH_FULL;
+		global_data->lvs_flush_on_stop = LVS_FLUSH_FULL;
 	else if (!strcmp(strvec_slot(strvec, 1), "VS"))
-		global_data->lvs_flush_onstop = LVS_FLUSH_VS;
+		global_data->lvs_flush_on_stop = LVS_FLUSH_VS;
 	else
-		report_config_error(CONFIG_GENERAL_ERROR, "Unknown lvs_flush_onstop type %s", strvec_slot(strvec, 1));
+		report_config_error(CONFIG_GENERAL_ERROR, "Unknown lvs_flush_on_stop type %s", strvec_slot(strvec, 1));
 }
 #endif
 
@@ -741,7 +765,7 @@ vrrp_mcast_group4_handler(const vector_t *strvec)
 {
 	struct sockaddr_in *mcast = &global_data->vrrp_mcast_group4;
 
-	if (inet_stosockaddr(strvec_slot(strvec, 1), 0, (struct sockaddr_storage *)mcast))
+	if (inet_stosockaddr(strvec_slot(strvec, 1), 0, PTR_CAST(struct sockaddr_storage, mcast)))
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: Cant parse vrrp_mcast_group4 [%s]. Skipping"
 				   , strvec_slot(strvec, 1));
 }
@@ -750,7 +774,7 @@ vrrp_mcast_group6_handler(const vector_t *strvec)
 {
 	struct sockaddr_in6 *mcast = &global_data->vrrp_mcast_group6;
 
-	if (inet_stosockaddr(strvec_slot(strvec, 1), 0, (struct sockaddr_storage *)mcast))
+	if (inet_stosockaddr(strvec_slot(strvec, 1), 0, PTR_CAST(struct sockaddr_storage, mcast)))
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: Cant parse vrrp_mcast_group6 [%s]. Skipping"
 				   , strvec_slot(strvec, 1));
 }
@@ -897,6 +921,30 @@ vrrp_min_garp_handler(const vector_t *strvec)
 	if (global_data->vrrp_garp_delay == VRRP_GARP_DELAY)
 		global_data->vrrp_garp_delay = 0;
 }
+#ifdef _HAVE_VRRP_VMAC_
+static void
+vrrp_vmac_garp_intvl_handler(const vector_t *strvec)
+{
+	unsigned delay = 0;
+	unsigned index;
+
+	for (index = 1; index < vector_size(strvec); index++) {
+		if (!strcmp(strvec_slot(strvec, index), "all"))
+			global_data->vrrp_vmac_garp_all_if = true;
+		else if (!read_unsigned_strvec(strvec, index, &delay, 1, 86400, true)) {
+			report_config_error(CONFIG_GENERAL_ERROR, "vrrp_vmac_garp_intvl '%s' invalid - ignoring", strvec_slot(strvec, index));
+			return;
+		}
+	}
+
+	if (!delay) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vrrp_vmac_garp_intvl specified without time - ignoring");
+		return;
+	}
+
+	global_data->vrrp_vmac_garp_intvl = delay;
+}
+#endif
 static void
 vrrp_lower_prio_no_advert_handler(const vector_t *strvec)
 {
@@ -1911,6 +1959,30 @@ vrrp_log_unknown_vrids_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	global_data->log_unknown_vrids = true;
 }
+
+#ifdef _HAVE_VRRP_VMAC_
+static void
+vrrp_vmac_prefix_handler(const vector_t *strvec)
+{
+	if (global_data->vmac_prefix) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vmac prefix has already been specified - ignoring %s", strvec_slot(strvec, 1));
+		return;
+	}
+
+	global_data->vmac_prefix = STRDUP(strvec_slot(strvec, 1));
+}
+
+static void
+vrrp_vmac_addr_prefix_handler(const vector_t *strvec)
+{
+	if (global_data->vmac_addr_prefix) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vmac_addr prefix has already been specified - ignoring %s", strvec_slot(strvec, 1));
+		return;
+	}
+
+	global_data->vmac_addr_prefix = STRDUP(strvec_slot(strvec, 1));
+}
+#endif
 #endif
 
 static void
@@ -1927,17 +1999,32 @@ random_seed_handler(const vector_t *strvec)
 }
 
 #ifndef _ONE_PROCESS_DEBUG_
+static void
+include_check_handler(const vector_t *strvec)
+{
+	include_check_set(strvec);
+}
+
+static void
+reload_check_config_handler(const vector_t *strvec)
+{
+	if (vector_size(strvec) >= 2) {
+		FREE_CONST_PTR(global_data->reload_check_config);
+		global_data->reload_check_config = set_value(strvec);
+
+		/* Check file can be written */
+	} else
+		global_data->reload_check_config = STRDUP("/dev/null");
+}
+
 static void
 reload_time_file_handler(const vector_t *strvec)
 {
-	char *str;
-
 	if (vector_size(strvec) != 2) {
 		report_config_error(CONFIG_GENERAL_ERROR, "reload_time_file invalid");
 		return;
 	}
-	global_data->reload_time_file = str = MALLOC(strlen(strvec_slot(strvec, 1)) + 1);
-	strcpy(str, strvec_slot(strvec, 1));
+	global_data->reload_time_file = STRDUP(strvec_slot(strvec, 1));
 }
 
 static void
@@ -1945,8 +2032,38 @@ reload_repeat_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	global_data->reload_repeat = true;
 }
+
+static void
+reload_file_handler(const vector_t *strvec)
+{
+	if (global_data->reload_file && global_data->reload_file != DEFAULT_RELOAD_FILE)
+		FREE_CONST_PTR(global_data->reload_file);
+
+	if (vector_size(strvec) >= 2)
+		global_data->reload_file = STRDUP(strvec_slot(strvec, 1));
+	else
+		global_data->reload_file = DEFAULT_RELOAD_FILE;
+}
 #endif
 
+static void
+config_copy_directory_handler(const vector_t *strvec)
+{
+	if (global_data->config_directory) {
+		report_config_error(CONFIG_GENERAL_ERROR, "%s already specified - ignoring", strvec_slot(strvec, 0));
+		return;
+	}
+
+	if (vector_size(strvec) >= 2) {
+		global_data->config_directory = STRDUP(strvec_slot(strvec, 1));
+
+		/* Copy the configuration read so far to the new location */
+		if (!reload && !__test_bit(CONFIG_TEST_BIT, &debug))
+			use_disk_copy_for_config(global_data->config_directory);
+	} else
+		report_config_error(CONFIG_GENERAL_ERROR, "%s missing directory name", strvec_slot(strvec, 0));
+}
+
 void
 init_global_keywords(bool global_active)
 {
@@ -1969,7 +2086,8 @@ init_global_keywords(bool global_active)
 	install_keyword("vrrp_process_name", &vrrp_process_name_handler);
 #endif
 #ifdef _WITH_LVS_
-	install_keyword("lvs_process_name", &lvs_process_name_handler);
+	install_keyword("checker_process_name", &checker_process_name_handler);
+	install_keyword("lvs_process_name", &lvs_process_name_handler);		/* Deprecated since 12/07/20 */
 #endif
 #ifdef _WITH_BFD_
 	install_keyword("bfd_process_name", &bfd_process_name_handler);
@@ -1998,11 +2116,13 @@ init_global_keywords(bool global_active)
 	install_keyword("dynamic_interfaces", &dynamic_interfaces_handler);
 	install_keyword("no_email_faults", &no_email_faults_handler);
 	install_keyword("default_interface", &default_interface_handler);
+	install_keyword("disable_local_igmp", &disable_local_igmp_handler);
 #endif
 #ifdef _WITH_LVS_
 	install_keyword("lvs_timeouts", &lvs_timeouts);
 	install_keyword("lvs_flush", &lvs_flush_handler);
-	install_keyword("lvs_flush_onstop", &lvs_flush_onstop_handler);
+	install_keyword("lvs_flush_on_stop", &lvs_flush_on_stop_handler);
+	install_keyword("lvs_flush_onstop", &lvs_flush_on_stop_handler);		/* Deprecated after v2.1.5 */
 #ifdef _WITH_VRRP_
 	install_keyword("lvs_sync_daemon", &lvs_syncd_handler);
 #endif
@@ -2019,6 +2139,9 @@ init_global_keywords(bool global_active)
 	install_keyword("vrrp_garp_interval", &vrrp_garp_interval_handler);
 	install_keyword("vrrp_gna_interval", &vrrp_gna_interval_handler);
 	install_keyword("vrrp_min_garp", &vrrp_min_garp_handler);
+#ifdef _HAVE_VRRP_VMAC_
+	install_keyword("vrrp_vmac_garp_intvl", &vrrp_vmac_garp_intvl_handler);
+#endif
 	install_keyword("vrrp_lower_prio_no_advert", &vrrp_lower_prio_no_advert_handler);
 	install_keyword("vrrp_higher_prio_send_advert", &vrrp_higher_prio_send_advert_handler);
 	install_keyword("vrrp_version", &vrrp_version_handler);
@@ -2126,11 +2249,19 @@ init_global_keywords(bool global_active)
 	install_keyword("vrrp_rx_bufs_multiplier", &vrrp_rx_bufs_multiplier_handler);
 	install_keyword("vrrp_startup_delay", &vrrp_startup_delay_handler);
 	install_keyword("log_unknown_vrids", &vrrp_log_unknown_vrids_handler);
+#ifdef _HAVE_VRRP_VMAC_
+	install_keyword("vmac_prefix", &vrrp_vmac_prefix_handler);
+	install_keyword("vmac_addr_prefix", &vrrp_vmac_addr_prefix_handler);
+#endif
 #endif
 	install_keyword("umask", &umask_handler);
 	install_keyword("random_seed", &random_seed_handler);
 #ifndef _ONE_PROCESS_DEBUG_
+	install_keyword("reload_check_config", &reload_check_config_handler);
 	install_keyword("reload_time_file", &reload_time_file_handler);
 	install_keyword("reload_repeat", &reload_repeat_handler);
+	install_keyword("reload_file", &reload_file_handler);
+	install_keyword("include_check", &include_check_handler);
 #endif
+	install_keyword("tmp_config_directory", &config_copy_directory_handler);
 }
