@@ -83,7 +83,7 @@
 #include "scheduler.h"
 #include "keepalived_netlink.h"
 #include "git-commit.h"
-#if defined THREAD_DUMP || defined _EPOLL_DEBUG_ || defined _EPOLL_THREAD_DUMP_
+#if defined THREAD_DUMP || defined _EPOLL_DEBUG_ || defined _EPOLL_THREAD_DUMP_ || defined _SCRIPT_DEBUG_
 #include "scheduler.h"
 #endif
 #include "process.h"
@@ -99,7 +99,7 @@
 #if defined _NETWORK_TIMESTAMP_ || defined _CHECKSUM_DEBUG_
 #include "vrrp.h"
 #endif
-#if defined _TSM_DEBUG_ || defined _SCRIPT_DEBUG_
+#if defined _TSM_DEBUG_ || defined _RECVMSG_DEBUG_
 #include "vrrp_scheduler.h"
 #endif
 #if defined _PARSER_DEBUG_ || defined _DUMP_KEYWORDS_
@@ -114,16 +114,11 @@
 #ifndef _ONE_PROCESS_DEBUG_
 #include "reload_monitor.h"
 #endif
+#ifdef _USE_SYSTEMD_
+#include "systemd.h"
+#endif
 #include "warnings.h"
 
-/* musl libc doesn't define the following */
-#ifndef	W_EXITCODE
-#define	W_EXITCODE(ret, sig)	((ret) << 8 | (sig))
-#endif
-#ifndef	WCOREFLAG
-#define	WCOREFLAG		((int32_t)WCOREDUMP(0xffffffff))
-#endif
-
 #define CHILD_WAIT_SECS	5
 
 /* Structure used for handling termination of children */
@@ -199,13 +194,16 @@ static struct {
 /* umask settings */
 bool umask_cmdline;
 
+/* Reload control */
+static unsigned num_reloading;
+
 /* Control producing core dumps */
 static bool set_core_dump_pattern = false;
 static bool create_core_dump = false;
 static const char *core_dump_pattern = "core";
 static char *orig_core_dump_pattern = NULL;
 
-static const char *dump_file = "/tmp/keepalived_parent.data";
+static const char *dump_file = KA_TMP_DIR "/keepalived_parent.data";
 
 /* debug flags */
 #if defined _TIMER_CHECK_ || \
@@ -225,6 +223,7 @@ static const char *dump_file = "/tmp/keepalived_parent.data";
     defined _DUMP_KEYWORDS_ || \
     defined _CHECKER_DEBUG_ || \
     defined _MEM_ERR_DEBUG_ || \
+    defined _RECVMSG_DEBUG_ || \
     defined _EINTR_DEBUG_ || \
     defined _SCRIPT_DEBUG_
 #define WITH_DEBUG_OPTIONS 1
@@ -279,6 +278,10 @@ static char checker_debug;
 #ifdef _MEM_ERR_DEBUG_
 static char mem_err_debug;
 #endif
+#ifdef _RECVMSG_DEBUG_
+static char recvmsg_debug;
+static char recvmsg_debug_dump;
+#endif
 #ifdef _EINTR_DEBUG_
 static char eintr_debug;
 #endif
@@ -370,35 +373,6 @@ make_syslog_ident(const char* name)
 	return ident;
 }
 
-static char *
-make_pidfile_name(const char* start, const char* instance, const char* extn)
-{
-	size_t len;
-	char *name;
-
-	len = strlen(start) + 1;
-	if (instance)
-		len += strlen(instance) + 1;
-	if (extn)
-		len += strlen(extn);
-
-	name = MALLOC(len);
-	if (!name) {
-		log_message(LOG_INFO, "Unable to make pidfile name for %s", start);
-		return NULL;
-	}
-
-	strcpy(name, start);
-	if (instance) {
-		strcat(name, "_");
-		strcat(name, instance);
-	}
-	if (extn)
-		strcat(name, extn);
-
-	return name;
-}
-
 #ifdef _WITH_VRRP_
 bool __attribute__ ((pure))
 running_vrrp(void)
@@ -471,9 +445,48 @@ global_init_keywords(void)
 }
 
 static void
-read_config_file(void)
+create_reload_file(void)
 {
-	init_data(conf_file, global_init_keywords);
+	int fd;
+
+	if (!global_data->reload_file || __test_bit(CONFIG_TEST_BIT, &debug))
+		return;
+
+	/* We want to create the reloading file with permissions rw-r--r-- */
+	if (umask_val & (S_IRGRP | S_IROTH))
+		umask(umask_val & ~(S_IRGRP | S_IROTH));
+
+	if ((fd = creat(global_data->reload_file, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) != -1)
+		close(fd);
+	else
+		log_message(LOG_INFO, "Failed to create reload file (%s) %d - %m", global_data->reload_file, errno);
+
+	/* Restore the default umask */
+	if (umask_val & (S_IRGRP | S_IROTH))
+		umask(umask_val);
+}
+
+static inline  void
+remove_reload_file(void)
+{
+	if (global_data->reload_file && !__test_bit(CONFIG_TEST_BIT, &debug))
+		unlink(global_data->reload_file);
+}
+
+static void
+read_config_file(bool write_config_copy)
+{
+#ifdef _ONE_PROCESS_DEBUG_
+	write_config_copy = false;
+#endif
+
+	if (write_config_copy)
+		create_reload_file();
+
+	init_data(conf_file, global_init_keywords, write_config_copy);
+
+	if (write_config_copy)
+		remove_reload_file();
 }
 
 /* Daemon stop sequence */
@@ -522,6 +535,7 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 	if (running_checker()) {
 		start_check_child();
 		have_child = true;
+		num_reloading++;
 	}
 #endif
 #ifdef _WITH_VRRP_
@@ -529,6 +543,7 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 	if (running_vrrp()) {
 		start_vrrp_child();
 		have_child = true;
+		num_reloading++;
 	}
 #endif
 #ifdef _WITH_BFD_
@@ -536,6 +551,7 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 	if (running_bfd()) {
 		start_bfd_child();
 		have_child = true;
+		num_reloading++;
 	}
 #endif
 
@@ -550,64 +566,71 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 }
 
 static bool
-startup_shutdown_script_completed(thread_ref_t thread, bool startup)
+handle_child_timeout(thread_ref_t thread, const char *type)
 {
-	const char *type = startup ? "startup" : "shutdown";
-	int wait_status;
 	pid_t pid;
 	int sig_num;
-	unsigned timeout = 0;
 	void *next_arg;
+	unsigned timeout = 0;
 
-	if (thread->type == THREAD_CHILD_TIMEOUT) {
-		pid = THREAD_CHILD_PID(thread);
+	pid = THREAD_CHILD_PID(thread);
 
-		if (thread->arg == (void *)0) {
-			next_arg = (void *)1;
-			sig_num = SIGTERM;
-			timeout = 2;
-			log_message(LOG_INFO, "%s script timed out", type);
-		} else if (thread->arg == (void *)1) {
-			next_arg = (void *)2;
-			sig_num = SIGKILL;
-			timeout = 2;
-		} else if (thread->arg == (void *)2) {
-			log_message(LOG_INFO, "%s script (PID %d) failed to terminate after kill", type, pid);
-			next_arg = (void *)3;
-			sig_num = SIGKILL;
-			timeout = 10;	/* Give it longer to terminate */
-		} else if (thread->arg == (void *)3) {
-			/* We give up trying to kill the script */
-			return true;
-		}
-
-		if (timeout) {
-			/* If kill returns an error, we can't kill the process since either the process has terminated,
-			 * or we don't have permission. If we can't kill it, there is no point trying again. */
-			if (kill(-pid, sig_num)) {
-				if (errno == ESRCH) {
-					/* The process does not exist, and we should
-					 * have reaped its exit status, otherwise it
-					 * would exist as a zombie process. */
-					log_message(LOG_INFO, "%s script (PID %d) lost", type, pid);
-					timeout = 0;
-				} else {
-					log_message(LOG_INFO, "kill -%d of %s script (%d) with new state %p failed with errno %d", sig_num, type, pid, next_arg, errno);
-					timeout = 1000;
-				}
-			}
-		} else {
-			log_message(LOG_INFO, "%s script %d timeout with unknown script state %p", type, pid, thread->arg);
-			next_arg = thread->arg;
-			timeout = 10;	/* We need some timeout */
-		}
-
-		if (timeout)
-			thread_add_child(thread->master, thread->func, next_arg, pid, timeout * TIMER_HZ);
-
-		return false;
+	if (thread->arg == (void *)0) {
+		next_arg = (void *)1;
+		sig_num = SIGTERM;
+		timeout = 2;
+		log_message(LOG_INFO, "%s timed out", type);
+	} else if (thread->arg == (void *)1) {
+		next_arg = (void *)2;
+		sig_num = SIGKILL;
+		timeout = 2;
+	} else if (thread->arg == (void *)2) {
+		log_message(LOG_INFO, "%s (PID %d) failed to terminate after kill", type, pid);
+		next_arg = (void *)3;
+		sig_num = SIGKILL;
+		timeout = 10;	/* Give it longer to terminate */
+	} else if (thread->arg == (void *)3) {
+		/* We give up trying to kill the script */
+		return true;
 	}
 
+	if (timeout) {
+		/* If kill returns an error, we can't kill the process since either the process has terminated,
+		 * or we don't have permission. If we can't kill it, there is no point trying again. */
+		if (kill(-pid, sig_num)) {
+			if (errno == ESRCH) {
+				/* The process does not exist, and we should
+				 * have reaped its exit status, otherwise it
+				 * would exist as a zombie process. */
+				log_message(LOG_INFO, "%s (PID %d) lost", type, pid);
+				timeout = 0;
+			} else {
+				log_message(LOG_INFO, "kill -%d of %s (%d) with new state %p failed with errno %d", sig_num, type, pid, next_arg, errno);
+				timeout = 1000;
+			}
+		}
+	} else {
+		log_message(LOG_INFO, "%s %d timeout with unknown script state %p", type, pid, thread->arg);
+		next_arg = thread->arg;
+		timeout = 10;	/* We need some timeout */
+	}
+
+	if (timeout)
+		thread_add_child(thread->master, thread->func, next_arg, pid, timeout * TIMER_HZ);
+
+	return false;
+}
+
+static bool
+startup_shutdown_script_completed(thread_ref_t thread, bool startup)
+{
+	const char *type = startup ? "startup script" : "shutdown script";
+	int wait_status;
+	pid_t pid;
+
+	if (thread->type == THREAD_CHILD_TIMEOUT)
+		return handle_child_timeout(thread, type);
+
 	wait_status = THREAD_CHILD_STATUS(thread);
 
 	if (WIFEXITED(wait_status)) {
@@ -716,20 +739,20 @@ static bool reload_config(void)
 		stop_reload_monitor();
 #endif
 
+	/* Clear any config errors from previous loads */
+	clear_config_status();
+
 	/* Make sure there isn't an attempt to change the network namespace or instance name */
 	old_global_data = global_data;
 	global_data = NULL;
 	global_data = alloc_global_data();
 
-	read_config_file();
+	/* If reload_check_config the process checking the config will read the config files,
+	 * otherwise this process needs to. */
+	read_config_file(!old_global_data->reload_check_config);
 
 	init_global_data(global_data, old_global_data, false);
 
-	/* Update process name if necessary */
-	if (!global_data->process_name != !old_global_data->process_name ||
-	    (global_data->process_name && strcmp(global_data->process_name, old_global_data->process_name)))
-		set_process_name(global_data->process_name);
-
 #if HAVE_DECL_CLONE_NEWNET
 	if (override_namespace) {
 		FREE_CONST_PTR(global_data->network_namespace);
@@ -757,13 +780,32 @@ static bool reload_config(void)
 	}
 #endif
 
+	if (!!old_global_data->config_directory != !!global_data->config_directory ||
+	    (global_data->config_directory && strcmp(old_global_data->config_directory, global_data->config_directory))) {
+		log_message(LOG_INFO, "Cannot change config_directory at a reload - please restart %s", PACKAGE);
+		unsupported_change = true;
+	}
+
+#ifdef _WITH_VRRP_
+	if (old_global_data->disable_local_igmp != global_data->disable_local_igmp) {
+		log_message(LOG_INFO, "Cannot change disable_local_igmp at a reload - please restart %s", PACKAGE);
+		unsupported_change = true;
+	}
+#endif
+
 	if (unsupported_change) {
 		/* We cannot reload the configuration, so continue with the old config */
 		free_global_data (global_data);
 		global_data = old_global_data;
 	}
-	else
+	else {
+		/* Update process name if necessary */
+		if (!global_data->process_name != !old_global_data->process_name ||
+		    (global_data->process_name && strcmp(global_data->process_name, old_global_data->process_name)))
+			set_process_name(global_data->process_name);
+
 		free_global_data (old_global_data);
+	}
 
 #ifndef _ONE_PROCESS_DEBUG_
 	if (global_data->reload_time_file)
@@ -799,11 +841,6 @@ print_parent_data(__attribute__((unused)) thread_ref_t thread)
 static void
 propagate_signal(__attribute__((unused)) void *v, int sig)
 {
-	if (sig == SIGHUP) {
-		if (!reload_config())
-			return;
-	}
-
 	/* Signal child processes */
 #ifdef _WITH_VRRP_
 	if (vrrp_child > 0)
@@ -837,6 +874,169 @@ propagate_signal(__attribute__((unused)) void *v, int sig)
 		thread_add_event(master, print_parent_data, NULL, 0);
 }
 
+#ifndef _ONE_PROCESS_DEBUG_
+static void
+child_reloaded(__attribute__((unused)) void *one, __attribute__((unused)) int sig_num)
+{
+	if (num_reloading) {
+		num_reloading--;
+
+		if (!num_reloading) {
+			truncate_config_copy();
+#ifdef _USE_SYSTEMD_
+			systemd_notify_running();
+#endif
+		}
+	}
+}
+
+static void
+do_reload(void)
+{
+	if (!reload_config())
+		return;
+
+#ifdef _USE_SYSTEMD_
+	systemd_notify_reloading();
+#endif
+
+	propagate_signal(NULL, SIGHUP);
+
+#ifdef _WITH_VRRP_
+	if (vrrp_child > 0)
+		num_reloading++;
+#endif
+#ifdef _WITH_LVS_
+	if (checkers_child > 0)
+		num_reloading++;
+#endif
+#ifdef _WITH_BFD_
+	if (bfd_child > 0)
+		num_reloading++;
+#endif
+}
+
+static void
+reload_check_child_thread(thread_ref_t thread)
+{
+	if (thread->type == THREAD_CHILD_TIMEOUT) {
+		handle_child_timeout(thread, "config check");
+		return;
+	}
+
+	/* The config files have been read now */
+	remove_reload_file();
+
+	if (WIFEXITED(thread->u.c.status)) {
+		if (WEXITSTATUS(thread->u.c.status)) {
+			log_message(LOG_INFO, "New config failed validation, see %s for details", global_data->reload_check_config);
+			return;
+		}
+
+		do_reload();
+	} else
+		report_child_status(thread->u.c.status, thread->u.c.pid, "reload_check");
+}
+
+static void
+start_validate_reload_conf_child(void)
+{
+	notify_script_t script;
+	int i;
+	int ret;
+	int argc;
+	const char **argv;
+	char * const *sav_argv;
+	char *config_test_str;
+	char *config_fd_str = NULL;
+	int fd;
+	int len;
+	char exe_buf[128];
+
+	/* Inherits the original parameters and adds new parameters "--config-test and --config-fd" */
+	sav_argv = get_cmd_line_options(&argc);
+	argv = MALLOC((argc + 3) * sizeof(char *));
+
+	exe_buf[sizeof(exe_buf) - 1] = '\0';
+	ret = readlink("/proc/self/exe", exe_buf, sizeof(exe_buf));
+	if (ret == sizeof(exe_buf))
+		strcpy(exe_buf, "/proc/self/exe");
+	else {
+		exe_buf[ret] = '\0';
+		len = strlen(exe_buf);
+		/* If keepalived has been recompiled, the original file will
+		 * be marked as deleted, but we can use the new one. */
+		if (len > 10 && !strcmp(exe_buf + len - 10, " (deleted)"))
+			exe_buf[len - 10] = '\0';
+	}
+	argv[0] = exe_buf;
+
+	/* copy old parameters */
+	for (i = 1; i < argc; i++)
+		argv[i] = sav_argv[i];
+
+	/* add --config-test */
+	config_test_str = MALLOC(14 + strlen(global_data->reload_check_config) + 1);
+	strcpy(config_test_str, "--config-test=");
+	strcat(config_test_str, global_data->reload_check_config);
+	argv[argc++] = config_test_str;
+
+	/* add --config-fd */
+	if ((fd = get_config_fd()) != -1) {
+		len = 13 + 6 + 1;	/* --config-fd=XXXXXX */
+		config_fd_str = MALLOC(len);
+		snprintf(config_fd_str, len, "--config-fd=%d", fd);
+		argv[argc++] = config_fd_str;
+
+		/* Allow fd to be inherited by exec'd process */
+		fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) & ~FD_CLOEXEC);
+	}
+
+	argv[argc] = NULL;
+
+	script.args = argv;
+	script.num_args = argc;
+	script.flags = SC_EXECABLE;
+	script.uid = 0;
+	script.gid = 0;
+
+	if (truncate(global_data->reload_check_config, 0) && errno != ENOENT)
+		log_message(LOG_INFO, "truncate of config check log %s failed (%d) - %m", global_data->reload_check_config, errno);
+
+	create_reload_file();
+
+	/* Execute the script in a child process. Parent returns, child doesn't */
+	ret = system_call_script(master, reload_check_child_thread,
+				  NULL, 5 * TIMER_HZ, &script);
+
+	if (ret)
+		log_message(LOG_INFO, "Could not run config_test");
+
+	/* Restore CLOEXEC on config_copy fd */
+	fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
+
+	FREE(argv);
+	FREE(config_test_str);
+	FREE_PTR(config_fd_str);
+}
+
+static void
+process_reload_signal(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
+{
+
+	/* if reload_check_config is configured, validate the new config before reload */
+	if (!global_data->reload_check_config) {
+		do_reload();
+		return;
+	}
+
+	if (__test_bit(LOG_DETAIL_BIT, &debug))
+		log_message(LOG_INFO, "validate conf before Reload");
+
+	start_validate_reload_conf_child();
+}
+#endif
+
 #ifdef THREAD_DUMP
 void
 thread_dump_signal(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
@@ -876,6 +1076,10 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 
 	log_message(LOG_INFO, "Stopping");
 
+#ifdef _USE_SYSTEMD_
+	systemd_notify_stopping();
+#endif
+
 #ifndef _ONE_PROCESSS_DEBUG_
 	if (global_data->reload_time_file)
 		stop_reload_monitor();
@@ -1041,10 +1245,11 @@ static void
 signal_init(void)
 {
 #ifndef _ONE_PROCESS_DEBUG_
-	signal_set(SIGHUP, propagate_signal, NULL);
+	signal_set(SIGHUP, process_reload_signal, NULL);
 	signal_set(SIGUSR1, propagate_signal, NULL);
 	signal_set(SIGUSR2, propagate_signal, NULL);
 	signal_set(SIGSTATS_CLEAR, propagate_signal, NULL);
+	signal_set(SIGPWR, child_reloaded, NULL);
 #ifdef _WITH_JSON_
 	signal_set(SIGJSON, propagate_signal, NULL);
 #endif
@@ -1064,6 +1269,7 @@ signals_ignore(void) {
 	signal_ignore(SIGUSR1);
 	signal_ignore(SIGUSR2);
 	signal_ignore(SIGSTATS_CLEAR);
+	signal_ignore(SIGPWR);
 #ifdef _WITH_JSON_
 	signal_ignore(SIGJSON);
 #endif
@@ -1239,6 +1445,10 @@ initialise_debug_options(void)
 #ifdef _MEM_ERR_DEBUG_
 	do_mem_err_debug = !!(mem_err_debug & mask);
 #endif
+#ifdef _RECVMSG_DEBUG_
+	do_recvmsg_debug = !!(recvmsg_debug & mask);
+	do_recvmsg_debug_dump = !!(recvmsg_debug_dump & mask);
+#endif
 #ifdef _EINTR_DEBUG_
 	do_eintr_debug = !!(eintr_debug & mask);
 #endif
@@ -1325,6 +1535,10 @@ set_debug_options(const char *options)
 #ifdef _MEM_ERR_DEBUG_
 		mem_err_debug = all_processes;
 #endif
+#ifdef _RECVMSG_DEBUG_
+		recvmsg_debug = all_processes;
+		recvmsg_debug_dump = all_processes;
+#endif
 #ifdef _EINTR_DEBUG_
 		eintr_debug = all_processes;
 #endif
@@ -1382,7 +1596,7 @@ set_debug_options(const char *options)
 		}
 #endif
 
-		/* Letters used - ABCDEFIHKMNOPRSTUVXZ */
+		/* Letters used - ABCDEFGHIJKMNOPRSTUVXZ */
 		switch (opt) {
 #ifdef _TIMER_CHECK_
 		case 'T':
@@ -1467,6 +1681,14 @@ set_debug_options(const char *options)
 			mem_err_debug = processes;
 			break;
 #endif
+#ifdef _RECVMSG_DEBUG_
+		case 'G':
+			recvmsg_debug = processes;
+			break;
+		case 'J':
+			recvmsg_debug_dump = processes;
+			break;
+#endif
 #ifdef _EINTR_DEBUG_
 		case 'I':
 			eintr_debug = processes;
@@ -1490,6 +1712,40 @@ set_debug_options(const char *options)
 }
 #endif
 
+static void
+report_distro(void)
+{
+	FILE *fp = fopen("/etc/os-release", "r");
+	char buf[128];
+	const char * const var = "PRETTY_NAME=";
+	const size_t var_len = strlen(var);
+	char *distro_name;
+	size_t distro_len;
+
+	if (!fp)
+		return;
+
+	while (fgets(buf, sizeof(buf), fp)) {
+		if (!strncmp(buf, var, var_len)) {
+			distro_name = buf + var_len;
+
+			/* Remove "'s and trailing \n */
+			if (*distro_name == '"')
+				distro_name++;
+			distro_len = strlen(distro_name);
+			if (distro_len && distro_name[distro_len - 1] == '\n')
+				distro_name[--distro_len] = '\0';
+			if (distro_len && distro_name[distro_len - 1] == '"')
+				distro_name[--distro_len] = '\0';
+
+			fprintf(stderr, "Distro: %s\n", distro_name);
+			break;
+		}
+	}
+
+	fclose(fp);
+}
+
 /* Usage function */
 static void
 usage(const char *prog)
@@ -1508,7 +1764,7 @@ usage(const char *prog)
 	fprintf(stderr, "  -D, --log-detail             Detailed log messages\n");
 	fprintf(stderr, "  -S, --log-facility=[0-7]     Set syslog facility to LOG_LOCAL[0-7]\n");
 #ifdef ENABLE_LOG_TO_FILE
-	fprintf(stderr, "  -g, --log-file=FILE          Also log to FILE (default /tmp/keepalived.log)\n");
+	fprintf(stderr, "  -g, --log-file=FILE          Also log to FILE (default " KA_TMP_DIR "/keepalived.log)\n");
 	fprintf(stderr, "      --flush-log-file         Flush log file on write\n");
 #endif
 	fprintf(stderr, "  -G, --no-syslog              Don't log via syslog\n");
@@ -1543,9 +1799,13 @@ usage(const char *prog)
 #endif
 	fprintf(stderr, "  -m, --core-dump              Produce core dump if terminate abnormally\n");
 	fprintf(stderr, "  -M, --core-dump-pattern=PATN Also set /proc/sys/kernel/core_pattern to PATN (default 'core')\n");
+#ifdef _MEM_CHECK_
+	fprintf(stderr, "      --no-mem-check           disable malloc() etc mem-checks\n");
+#endif
 #ifdef _MEM_CHECK_LOG_
 	fprintf(stderr, "  -L, --mem-check-log          Log malloc/frees to syslog\n");
 #endif
+	fprintf(stderr, "  -e, --all-config             Error if any configuration file missing (same as includet)\n");
 	fprintf(stderr, "  -i, --config-id id           Skip any configuration lines beginning '@' that don't match id\n"
 			"                                or any lines beginning @^ that do match.\n"
 			"                                The config-id defaults to the node name if option not used\n");
@@ -1559,6 +1819,7 @@ usage(const char *prog)
 								"\n");
 	fprintf(stderr, "  -t, --config-test[=LOG_FILE] Check the configuration for obvious errors, output to\n"
 			"                                stderr by default\n");
+/*	fprintf(stderr, "      --config-fd=fd_num       File descriptor to write consolidated config to\n");	*/ // Internal use only
 #ifdef _WITH_PERF_
 	fprintf(stderr, "      --perf[=PERF_TYPE]       Collect perf data, PERF_TYPE=all, run(default) or end\n");
 #endif
@@ -1614,6 +1875,10 @@ usage(const char *prog)
 #ifdef _MEM_ERR_DEBUG_
 	fprintf(stderr, "                                   Z - memory alloc/free error debug\n");
 #endif
+#ifdef _RECVMSG_DEBUG_
+	fprintf(stderr, "                                   G - VRRP recvmsg() debug\n");
+	fprintf(stderr, "                                   J - VRRP recvmsg() log rx data\n");
+#endif
 #ifdef _EINTR_DEBUG_
 	fprintf(stderr, "                                   I - EINTR debugging\n");
 #endif
@@ -1657,6 +1922,7 @@ parse_cmdline(int argc, char **argv)
 		{"log-detail",		no_argument,		NULL, 'D'},
 		{"log-facility",	required_argument,	NULL, 'S'},
 		{"log-file",		optional_argument,	NULL, 'g'},
+		{"all-config",		no_argument,		NULL, 'e'},
 #ifdef ENABLE_LOG_TO_FILE
 		{"flush-log-file",	no_argument,		NULL,  2 },
 #endif
@@ -1689,6 +1955,9 @@ parse_cmdline(int argc, char **argv)
 #endif
 		{"core-dump",		no_argument,		NULL, 'm'},
 		{"core-dump-pattern",	optional_argument,	NULL, 'M'},
+#ifdef _MEM_CHECK_
+		{"no-mem-check",	no_argument,		NULL,  7 },
+#endif
 #ifdef _MEM_CHECK_LOG_
 		{"mem-check-log",	no_argument,		NULL, 'L'},
 #endif
@@ -1698,6 +1967,7 @@ parse_cmdline(int argc, char **argv)
 		{"config-id",		required_argument,	NULL, 'i'},
 		{"signum",		required_argument,	NULL,  4 },
 		{"config-test",		optional_argument,	NULL, 't'},
+		{"config-fd",		required_argument,	NULL,  8 },
 #ifdef _WITH_PERF_
 		{"perf",		optional_argument,	NULL,  5 },
 #endif
@@ -1714,7 +1984,8 @@ parse_cmdline(int argc, char **argv)
 	 * of longindex, so we need to ensure that before calling getopt_long(), longindex
 	 * is set to a known invalid value */
 	curind = optind;
-	while (longindex = -1, (c = getopt_long(argc, argv, ":vhlndu:DRS:f:p:i:mM::g::Gt::"
+	/* Used short options: ABCDGILMPRSVXabcdefghilmnprstuvx */
+	while (longindex = -1, (c = getopt_long(argc, argv, ":vhlndu:DRS:f:p:i:emM::g::Gt::"
 #if defined _WITH_VRRP_ && defined _WITH_LVS_
 					    "PC"
 #endif
@@ -1755,7 +2026,9 @@ parse_cmdline(int argc, char **argv)
 						(LINUX_VERSION_CODE >>  8) & 0xff,
 						(LINUX_VERSION_CODE      ) & 0xff);
 			uname(&uname_buf);
-			fprintf(stderr, "Running on %s %s %s\n\n", uname_buf.sysname, uname_buf.release, uname_buf.version);
+			fprintf(stderr, "Running on %s %s %s\n", uname_buf.sysname, uname_buf.release, uname_buf.version);
+			report_distro();
+			fprintf(stderr, "\n");
 			fprintf(stderr, "configure options: %s\n\n", KEEPALIVED_CONFIGURE_OPTIONS);
 			fprintf(stderr, "Config options: %s\n\n", CONFIGURATION_OPTIONS);
 			fprintf(stderr, "System options: %s\n", SYSTEM_OPTIONS);
@@ -1812,7 +2085,7 @@ parse_cmdline(int argc, char **argv)
 			if (optarg && optarg[0])
 				log_file_name = optarg;
 			else
-				log_file_name = "/tmp/keepalived.log";
+				log_file_name = KA_TMP_DIR "/keepalived.log";
 			open_log_file(log_file_name, NULL, NULL, NULL);
 #else
 			fprintf(stderr, "-g requires configure option --enable-log-file\n");
@@ -1841,7 +2114,7 @@ parse_cmdline(int argc, char **argv)
 			if (optarg && optarg[0]) {
 				int fd = open(optarg, O_WRONLY | O_APPEND | O_CREAT | O_NOFOLLOW, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 				if (fd == -1) {
-					fprintf(stderr, "Unable to open config-test log file %s\n", optarg);
+					fprintf(stderr, "Unable to open config-test log file %s %d - %m\n", optarg, errno);
 					exit(EXIT_FAILURE);
 				}
 				dup2(fd, STDERR_FILENO);
@@ -1911,6 +2184,9 @@ parse_cmdline(int argc, char **argv)
 			override_namespace = optarg;
 			break;
 #endif
+		case 'e':
+			include_check_set(NULL);
+			break;
 		case 'i':
 			FREE_CONST_PTR(config_id);
 			config_id = STRDUP(optarg);
@@ -1960,6 +2236,14 @@ parse_cmdline(int argc, char **argv)
 			set_debug_options(optarg && optarg[0] ? optarg : NULL);
 			break;
 #endif
+#ifdef _MEM_CHECK_
+		case 7:
+			__clear_bit(MEM_CHECK_BIT, &debug);
+			break;
+#endif
+		case 8:
+			set_config_fd(atoi(optarg));
+			break;
 		case '?':
 			if (optopt && argv[curind][1] != '-')
 				fprintf(stderr, "Unknown option -%c\n", optopt);
@@ -1991,6 +2275,11 @@ parse_cmdline(int argc, char **argv)
 	if (bad_option)
 		exit(1);
 
+#ifdef _MEM_CHECK_
+	if (__test_bit(CONFIG_TEST_BIT, &debug))
+		__clear_bit(MEM_CHECK_BIT, &debug);
+#endif
+
 	return reopen_log;
 }
 
@@ -2054,6 +2343,10 @@ keepalived_main(int argc, char **argv)
 	struct rusage usage;
 	struct rusage child_usage;
 
+#ifdef _MEM_CHECK_
+	__set_bit(MEM_CHECK_BIT, &debug);
+#endif
+
 	/* Ignore reloading signals till signal_init call */
 	signals_ignore();
 
@@ -2064,8 +2357,9 @@ keepalived_main(int argc, char **argv)
 	/* Save command line options in case need to log them later */
 	save_cmd_line_options(argc, argv);
 
-	/* Init debugging level */
-	debug = 0;
+#ifdef _USE_SYSTEMD_
+	check_parent_systemd();
+#endif
 
 	/* We are the parent process */
 #ifndef _ONE_PROCESS_DEBUG_
@@ -2145,14 +2439,14 @@ keepalived_main(int argc, char **argv)
 
 	if (os_major) {
 		if (KERNEL_VERSION(os_major, os_minor, os_release) < LINUX_VERSION_CODE) {
-			/* keepalived was build for a later kernel version */
-			log_message(LOG_INFO, "WARNING - keepalived was build for newer Linux %d.%d.%d, running on %s %s %s",
+			/* keepalived was built for a later kernel version */
+			log_message(LOG_INFO, "WARNING - keepalived was built for newer Linux %d.%d.%d, running on %s %s %s",
 					(LINUX_VERSION_CODE >> 16) & 0xff,
 					(LINUX_VERSION_CODE >>  8) & 0xff,
 					(LINUX_VERSION_CODE      ) & 0xff,
 					uname_buf.sysname, uname_buf.release, uname_buf.version);
 		} else {
-			/* keepalived was build for a later kernel version */
+			/* keepalived was built for a later kernel version */
 			log_message(LOG_INFO, "Running on %s %s %s (built for Linux %d.%d.%d)",
 					uname_buf.sysname, uname_buf.release, uname_buf.version,
 					(LINUX_VERSION_CODE >> 16) & 0xff,
@@ -2180,7 +2474,13 @@ keepalived_main(int argc, char **argv)
 
 	global_data = alloc_global_data();
 
-	read_config_file();
+// Change here so don't need check_conf_file()
+	read_config_file(true);
+
+	if (had_config_file_error()) {
+		exit_code = KEEPALIVED_EXIT_NO_CONFIG;
+		goto end;
+	}
 
 	init_global_data(global_data, NULL, false);
 
@@ -2239,13 +2539,13 @@ keepalived_main(int argc, char **argv)
 			/* Create the directory for pid files */
 			create_pid_dir();
 		}
-	}
 
-	/* If we want to monitor processes, we have to do it before calling
-	 * setns() */
+		/* If we want to monitor processes, we have to do it before calling
+		 * setns() */
 #ifdef _WITH_CN_PROC_
-	open_track_processes();
+		open_track_processes();
 #endif
+	}
 
 #if HAVE_DECL_CLONE_NEWNET
 	if (global_data->network_namespace) {
@@ -2269,7 +2569,7 @@ keepalived_main(int argc, char **argv)
 				free_vrrp_pidfile = true;
 #endif
 #ifdef _WITH_BFD_
-			if (!bfd_pidfile && (bfd_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR VRRP_PID_FILE, global_data->instance_name, PID_EXTENSION)))
+			if (!bfd_pidfile && (bfd_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR BFD_PID_FILE, global_data->instance_name, PID_EXTENSION)))
 				free_bfd_pidfile = true;
 #endif
 		}
@@ -2308,6 +2608,9 @@ keepalived_main(int argc, char **argv)
 #endif
 		}
 
+		/* We have set the namespaces, so we can do this now */
+		remove_reload_file();
+
 		/* Check if keepalived is already running */
 		if (keepalived_running(daemon_mode)) {
 			log_message(LOG_INFO, "daemon is already running");
@@ -2318,7 +2621,7 @@ keepalived_main(int argc, char **argv)
 
 	/* daemonize process */
 	if (!__test_bit(DONT_FORK_BIT, &debug) &&
-	    xdaemon(false, false, true) > 0) {
+	    xdaemon() > 0) {
 		closelog();
 		FREE_CONST_PTR(config_id);
 		FREE_PTR(orig_core_dump_pattern);
