@@ -168,11 +168,12 @@
 #include <sys/socket.h>
 
 #ifdef LIBIPVS_USE_NL
+#ifdef _HAVE_LIBNL1_
+#include <linux/types.h>
+#endif
+#include <linux/netlink.h>	/* Required for netlink/headers.h prior to libnl v3.3.0 */
 #include <netlink/socket.h>
 #include <netlink/genl/genl.h>
-#ifdef _HAVE_LIBNL1_
-#define nl_sock		nl_handle
-#endif
 #ifdef _LIBNL_DYNAMIC_
 #include "libnl_link.h"
 #endif
@@ -207,10 +208,14 @@ setns(int fd, int nstype)
 #include "memory.h"
 #include "logger.h"
 #include "pidfile.h"
+#include "utils.h"
+#include "bitops.h"
+
 
 /* Local data */
 static const char *netns_dir = RUN_DIR "netns/";
 static char *mount_dirname;
+static bool run_mount_set;
 
 void
 free_dirname(void)
@@ -222,6 +227,8 @@ free_dirname(void)
 static void
 set_run_mount(const char *net_namespace)
 {
+	bool error;
+
 	/* /run/keepalived/NAMESPACE */
 	mount_dirname = MALLOC(strlen(KEEPALIVED_PID_DIR) + 1 + strlen(net_namespace));
 	if (!mount_dirname) {
@@ -232,7 +239,17 @@ set_run_mount(const char *net_namespace)
 	strcpy(mount_dirname, KEEPALIVED_PID_DIR);
 	strcat(mount_dirname, net_namespace);
 
-	if (mkdir(mount_dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) && errno != EEXIST) {
+	/* We want the directory to have rwxr-xr-x permissions */
+	if (umask_val & (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
+		umask(umask_val & ~(S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH));
+
+	error = mkdir(mount_dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) && errno != EEXIST;
+
+	/* Restore our default umask */
+	if (umask_val & (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
+		umask(umask_val);
+
+	if (error) {
 		log_message(LOG_INFO, "Unable to create directory %s", mount_dirname);
 		free_dirname();
 		return;
@@ -251,11 +268,16 @@ set_run_mount(const char *net_namespace)
 
 	if (mount(mount_dirname, pid_directory, NULL, MS_BIND, NULL))
 		log_message(LOG_INFO, "Mount failed, error (%d) '%s'", errno, strerror(errno));
+
+	run_mount_set = true;
 }
 
 static void
 unmount_run(void)
 {
+	if (!run_mount_set)
+		return;
+
 	if (umount(pid_directory))
 		log_message(LOG_INFO, "unmount of %s failed - errno %d", pid_directory, errno);
 	if (mount_dirname) {
@@ -293,7 +315,8 @@ set_namespaces(const char* net_namespace)
 
 	close(fd);
 
-	set_run_mount(net_namespace);
+	if (!__test_bit(CONFIG_TEST_BIT, &debug))
+		set_run_mount(net_namespace);
 
 	FREE_PTR(netns_path);
 	netns_path = NULL;
