@@ -42,6 +42,14 @@
 #ifdef THREAD_DUMP
 #include "scheduler.h"
 #endif
+
+/* If it suspected that one of subject, body, buffer or email_to
+ * is overflowing in the smtp_t structure, defining SMTP_MSG_ALLOC_DEBUG
+ * when using --enable-mem-check should help identity the issue
+ */
+//#define SMTP_MSG_ALLOC_DEBUG
+
+
 #ifdef _SMTP_ALERT_DEBUG_
 bool do_smtp_alert_debug;
 #endif
@@ -50,16 +58,16 @@ bool do_smtp_connect_debug;
 #endif
 
 /* SMTP FSM definition */
-static int connection_error(thread_ref_t);
-static int connection_in_progress(thread_ref_t);
-static int connection_timeout(thread_ref_t);
-static int connection_success(thread_ref_t);
-static int helo_cmd(thread_ref_t);
-static int mail_cmd(thread_ref_t);
-static int rcpt_cmd(thread_ref_t);
-static int data_cmd(thread_ref_t);
-static int body_cmd(thread_ref_t);
-static int quit_cmd(thread_ref_t);
+static void connection_error(thread_ref_t);
+static void connection_in_progress(thread_ref_t);
+static void connection_timeout(thread_ref_t);
+static void connection_success(thread_ref_t);
+static void helo_cmd(thread_ref_t);
+static void mail_cmd(thread_ref_t);
+static void rcpt_cmd(thread_ref_t);
+static void data_cmd(thread_ref_t);
+static void body_cmd(thread_ref_t);
+static void quit_cmd(thread_ref_t);
 
 static int connection_code(thread_ref_t , int);
 static int helo_code(thread_ref_t , int);
@@ -69,11 +77,11 @@ static int data_code(thread_ref_t , int);
 static int body_code(thread_ref_t , int);
 static int quit_code(thread_ref_t , int);
 
-static int smtp_read_thread(thread_ref_t);
-static int smtp_send_thread(thread_ref_t);
+static void smtp_read_thread(thread_ref_t);
+static void smtp_send_thread(thread_ref_t);
 
 struct {
-	int (*send) (thread_ref_t);
+	void (*send) (thread_ref_t);
 	int (*read) (thread_ref_t, int);
 } SMTP_FSM[SMTP_MAX_FSM_STATE] = {
 /*       Code			  Stream Write Handlers		Stream Read handlers *
@@ -91,38 +99,65 @@ struct {
 	[QUIT]			= {quit_cmd,			quit_code}
 };
 
-static void
-free_smtp_all(smtp_t * smtp)
+static inline void
+free_smtp_msg_data(smtp_t * smtp)
 {
+#ifdef SMTP_MSG_ALLOC_DEBUG
 	FREE(smtp->buffer);
 	FREE(smtp->subject);
 	FREE(smtp->body);
 	FREE(smtp->email_to);
+#endif
 	FREE(smtp);
 }
 
+static smtp_t *
+alloc_smtp_msg_data(void)
+{
+	smtp_t *smtp;
+
+	/* allocate & initialize smtp argument data structure.
+	 * NOTE: this MALLOC may not have been freed when
+	 * keepalived terminates, if keepalived is unable to
+	 * connect to the SMTP server, since it is waiting for
+	 * the connection attempt to time out. */
+#ifdef SMTP_MSG_ALLOC_DEBUG
+	PMALLOC(smtp);
+	smtp->subject = (char *)MALLOC(MAX_HEADERS_LENGTH);
+	smtp->body = (char *)MALLOC(MAX_BODY_LENGTH);
+	smtp->buffer = (char *)MALLOC(SMTP_BUFFER_MAX);
+	smtp->email_to = (char *)MALLOC(SMTP_BUFFER_MAX);
+#else
+	smtp = MALLOC(sizeof(smtp_t) + MAX_HEADERS_LENGTH + MAX_BODY_LENGTH + SMTP_BUFFER_MAX + SMTP_BUFFER_MAX);
+	smtp->subject = (char *)smtp + sizeof(smtp_t);
+	smtp->body = smtp->subject + MAX_HEADERS_LENGTH;
+	smtp->buffer = smtp->body + MAX_BODY_LENGTH;
+	smtp->email_to = smtp->buffer + SMTP_BUFFER_MAX;
+#endif
+
+	return smtp;
+}
+
 /* layer4 connection handlers */
-static int
+static void
 connection_error(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
 
 	log_message(LOG_INFO, "SMTP connection ERROR to %s."
 			    , FMT_SMTP_HOST());
-	free_smtp_all(smtp);
-	return 0;
+	free_smtp_msg_data(smtp);
 }
-static int
+static void
 connection_timeout(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
 
 	log_message(LOG_INFO, "Timeout connecting SMTP server %s."
 			    , FMT_SMTP_HOST());
-	free_smtp_all(smtp);
-	return 0;
+	free_smtp_msg_data(smtp);
 }
-static int
+static void
 connection_in_progress(thread_ref_t thread)
 {
 	int status;
@@ -140,10 +175,8 @@ connection_in_progress(thread_ref_t thread)
 
 	if (status != connect_in_progress)
 		SMTP_FSM_SEND(status, thread);
-
-	return 0;
 }
-static int
+static void
 connection_success(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
@@ -154,11 +187,10 @@ connection_success(thread_ref_t thread)
 	smtp->stage = connect_success;
 	thread_add_read(thread->master, smtp_read_thread, smtp,
 			smtp->fd, global_data->smtp_connection_to, true);
-	return 0;
 }
 
 /* SMTP protocol handlers */
-static int
+static void
 smtp_read_thread(thread_ref_t thread)
 {
 	smtp_t *smtp;
@@ -173,7 +205,7 @@ smtp_read_thread(thread_ref_t thread)
 		log_message(LOG_INFO, "Timeout reading data to remote SMTP server %s."
 				    , FMT_SMTP_HOST());
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return -1;
+		return;
 	}
 
 	buffer = smtp->buffer;
@@ -185,20 +217,20 @@ smtp_read_thread(thread_ref_t thread)
 		if (check_EAGAIN(errno)) {
 			thread_add_read(thread->master, smtp_read_thread, smtp,
 					thread->u.f.fd, global_data->smtp_connection_to, true);
-			return 0;
+			return;
 		}
 
 		log_message(LOG_INFO, "Error reading data from remote SMTP server %s."
 				    , FMT_SMTP_HOST());
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return 0;
+		return;
 	}
 
 	if (rcv_buffer_size == 0) {
 		log_message(LOG_INFO, "Remote SMTP server %s has closed the connection."
 				    , FMT_SMTP_HOST());
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return 0;
+		return;
 	}
 
 	/* received data overflow buffer size ? */
@@ -207,7 +239,7 @@ smtp_read_thread(thread_ref_t thread)
 				      " overflow our get read buffer length."
 				    , FMT_SMTP_HOST());
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return 0;
+		return;
 	}
 
 	smtp->buflen += (size_t)rcv_buffer_size;
@@ -228,7 +260,7 @@ smtp_read_thread(thread_ref_t thread)
 			thread_add_read(thread->master, smtp_read_thread,
 					smtp, thread->u.f.fd,
 					global_data->smtp_connection_to, true);
-			return 0;
+			return;
 		}
 
 		if (reply[3] == '-') {
@@ -250,7 +282,7 @@ smtp_read_thread(thread_ref_t thread)
 	if (status == -1) {
 		thread_add_read(thread->master, smtp_read_thread, smtp,
 				thread->u.f.fd, global_data->smtp_connection_to, true);
-		return 0;
+		return;
 	}
 
 	SMTP_FSM_READ(smtp->stage, thread, status);
@@ -265,10 +297,10 @@ smtp_read_thread(thread_ref_t thread)
 		SMTP_FSM_READ(QUIT, thread, 0);
 	}
 
-	return 0;
+	return;
 }
 
-static int
+static void
 smtp_send_thread(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
@@ -277,7 +309,7 @@ smtp_send_thread(thread_ref_t thread)
 		log_message(LOG_INFO, "Timeout sending data to remote SMTP server %s."
 				    , FMT_SMTP_HOST());
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return 0;
+		return;
 	}
 
 	SMTP_FSM_SEND(smtp->stage, thread);
@@ -285,7 +317,7 @@ smtp_send_thread(thread_ref_t thread)
 	/* Handle END command */
 	if (smtp->stage == END) {
 		SMTP_FSM_READ(QUIT, thread, 0);
-		return 0;
+		return;
 	}
 
 	/* Registering next smtp command processing thread */
@@ -299,7 +331,7 @@ smtp_send_thread(thread_ref_t thread)
 		SMTP_FSM_READ(QUIT, thread, 0);
 	}
 
-	return 0;
+	return;
 }
 
 static int
@@ -321,19 +353,17 @@ connection_code(thread_ref_t thread, int status)
 }
 
 /* HELO command processing */
-static int
+static void
 helo_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
 	char *buffer;
 
-	buffer = (char *) MALLOC(SMTP_BUFFER_MAX);
+	buffer = (char *)MALLOC(SMTP_BUFFER_MAX);
 	snprintf(buffer, SMTP_BUFFER_MAX, SMTP_HELO_CMD, (global_data->smtp_helo_name) ? global_data->smtp_helo_name : "localhost");
 	if (send(thread->u.f.fd, buffer, strlen(buffer), 0) == -1)
 		smtp->stage = ERROR;
 	FREE(buffer);
-
-	return 0;
 }
 static int
 helo_code(thread_ref_t thread, int status)
@@ -354,19 +384,17 @@ helo_code(thread_ref_t thread, int status)
 }
 
 /* MAIL command processing */
-static int
+static void
 mail_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
 	char *buffer;
 
-	buffer = (char *) MALLOC(SMTP_BUFFER_MAX);
+	buffer = (char *)MALLOC(SMTP_BUFFER_MAX);
 	snprintf(buffer, SMTP_BUFFER_MAX, SMTP_MAIL_CMD, global_data->email_from);
 	if (send(thread->u.f.fd, buffer, strlen(buffer), 0) == -1)
 		smtp->stage = ERROR;
 	FREE(buffer);
-
-	return 0;
 }
 static int
 mail_code(thread_ref_t thread, int status)
@@ -387,26 +415,26 @@ mail_code(thread_ref_t thread, int status)
 }
 
 /* RCPT command processing */
-static int
+static void
 rcpt_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
+	email_t *email = smtp->next_email_element;
 	char *buffer;
-	char *fetched_email;
 
-	buffer = (char *) MALLOC(SMTP_BUFFER_MAX);
+	buffer = (char *)MALLOC(SMTP_BUFFER_MAX);
 	/* We send RCPT TO command multiple time to add all our email receivers.
 	 * --rfc821.3.1
 	 */
-	fetched_email = ELEMENT_DATA(smtp->next_email_element);
-	ELEMENT_NEXT(smtp->next_email_element);
+	if (list_is_last(&smtp->next_email_element->e_list, &global_data->email))
+		smtp->next_email_element = NULL;
+	else
+		smtp->next_email_element = list_entry(email->e_list.next, email_t, e_list);
 
-	snprintf(buffer, SMTP_BUFFER_MAX, SMTP_RCPT_CMD, fetched_email);
+	snprintf(buffer, SMTP_BUFFER_MAX, SMTP_RCPT_CMD, email->addr);
 	if (send(thread->u.f.fd, buffer, strlen(buffer), 0) == -1)
 		smtp->stage = ERROR;
 	FREE(buffer);
-
-	return 0;
 }
 static int
 rcpt_code(thread_ref_t thread, int status)
@@ -428,14 +456,13 @@ rcpt_code(thread_ref_t thread, int status)
 }
 
 /* DATA command processing */
-static int
+static void
 data_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
 
 	if (send(thread->u.f.fd, SMTP_DATA_CMD, strlen(SMTP_DATA_CMD), 0) == -1)
 		smtp->stage = ERROR;
-	return 0;
 }
 static int
 data_code(thread_ref_t thread, int status)
@@ -459,7 +486,7 @@ data_code(thread_ref_t thread, int status)
  * Do we need to use mutli-thread for multi-part body
  * handling ? Don t really think :)
  */
-static int
+static void
 body_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
@@ -468,7 +495,7 @@ body_cmd(thread_ref_t thread)
 	time_t now;
 	struct tm *t;
 
-	buffer = (char *) MALLOC(SMTP_BUFFER_MAX);
+	buffer = (char *)MALLOC(SMTP_BUFFER_MAX);
 
 	time(&now);
 	t = localtime(&now);
@@ -493,7 +520,6 @@ body_cmd(thread_ref_t thread)
 		smtp->stage = ERROR;
 
 	FREE(buffer);
-	return 0;
 }
 static int
 body_code(thread_ref_t thread, int status)
@@ -515,7 +541,7 @@ body_code(thread_ref_t thread, int status)
 }
 
 /* QUIT command processing */
-static int
+static void
 quit_cmd(thread_ref_t thread)
 {
 	smtp_t *smtp = THREAD_ARG(thread);
@@ -524,7 +550,6 @@ quit_cmd(thread_ref_t thread)
 		smtp->stage = ERROR;
 	else
 		smtp->stage++;
-	return 0;
 }
 
 static int
@@ -533,7 +558,7 @@ quit_code(thread_ref_t thread, __attribute__((unused)) int status)
 	smtp_t *smtp = THREAD_ARG(thread);
 
 	/* final state, we are disconnected from the remote host */
-	free_smtp_all(smtp);
+	free_smtp_msg_data(smtp);
 	thread_close_fd(thread);
 	return 0;
 }
@@ -544,14 +569,14 @@ smtp_connect(smtp_t *smtp)
 {
 	enum connect_result status;
 
-	smtp->next_email_element = LIST_HEAD(global_data->email);
+	smtp->next_email_element = list_first_entry(&global_data->email, email_t, e_list);
 
 	if ((smtp->fd = socket(global_data->smtp_server.ss_family, SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_TCP)) == -1) {
 #ifdef _SMTP_CONNECT_DEBUG_
 		if (do_smtp_connect_debug)
 			log_message(LOG_DEBUG, "SMTP connect fail to create socket.");
 #endif
-		free_smtp_all(smtp);
+		free_smtp_msg_data(smtp);
 		return;
 	}
 
@@ -575,26 +600,28 @@ smtp_connect(smtp_t *smtp)
 static void
 smtp_log_to_file(smtp_t *smtp)
 {
-	FILE *fp = fopen_safe("/tmp/smtp-alert.log", "a");
+	FILE *fp = fopen_safe(KA_TMP_DIR "/smtp-alert.log", "a");
 	time_t now;
 	struct tm tm;
 	char time_buf[25];
 	int time_buf_len;
 
-	time(&now);
-	localtime_r(&now, &tm);
-	time_buf_len = strftime(time_buf, sizeof time_buf, "%a %b %e %X %Y", &tm);
+	if (fp) {
+		time(&now);
+		localtime_r(&now, &tm);
+		time_buf_len = strftime(time_buf, sizeof time_buf, "%a %b %e %X %Y", &tm);
 
-	fprintf(fp, "%s: %s -> %s\n"
-		    "%*sSubject: %s\n"
-		    "%*sBody:    %s\n\n",
-		    time_buf, global_data->email_from, smtp->email_to,
-		    time_buf_len - 7, "", smtp->subject,
-		    time_buf_len - 7, "", smtp->body);
+		fprintf(fp, "%s: %s -> %s\n"
+			    "%*sSubject: %s\n"
+			    "%*sBody:    %s\n\n",
+			    time_buf, global_data->email_from, smtp->email_to,
+			    time_buf_len - 7, "", smtp->subject,
+			    time_buf_len - 7, "", smtp->body);
 
-	fclose(fp);
+		fclose(fp);
+	}
 
-	free_smtp_all(smtp);
+	free_smtp_msg_data(smtp);
 }
 #endif
 
@@ -605,20 +632,19 @@ smtp_log_to_file(smtp_t *smtp)
 static void
 build_to_header_rcpt_addrs(smtp_t *smtp)
 {
-	const char *fetched_email;
 	char *email_to_addrs;
 	size_t bytes_available = SMTP_BUFFER_MAX - 1;
 	size_t bytes_to_write;
 	bool done_addr = false;
-	element e;
+	email_t *email;
 
 	if (smtp == NULL)
 		return;
 
 	email_to_addrs = smtp->email_to;
 
-	LIST_FOREACH(global_data->email, fetched_email, e) {
-		bytes_to_write = strlen(fetched_email);
+	list_for_each_entry(email, &global_data->email, e_list) {
+		bytes_to_write = strlen(email->addr);
 		if (done_addr) {
 			if (bytes_available < 2)
 				break;
@@ -634,7 +660,7 @@ build_to_header_rcpt_addrs(smtp_t *smtp)
 		if (bytes_available < bytes_to_write)
 			break;
 
-		strcpy(email_to_addrs, fetched_email);
+		strcpy(email_to_addrs, email->addr);
 
 		email_to_addrs += bytes_to_write;
 		bytes_available -= bytes_to_write;
@@ -657,20 +683,16 @@ smtp_alert(smtp_msg_t msg_type, void* data, const char *subject, const char *bod
 #endif
 
 	/* Only send mail if email specified */
-	if (LIST_ISEMPTY(global_data->email) || !global_data->smtp_server.ss_family)
+	if (list_empty(&global_data->email) || !global_data->smtp_server.ss_family)
 		return;
 
 	/* allocate & initialize smtp argument data structure */
-	smtp = (smtp_t *) MALLOC(sizeof(smtp_t));
-	smtp->subject = (char *) MALLOC(MAX_HEADERS_LENGTH);
-	smtp->body = (char *) MALLOC(MAX_BODY_LENGTH);
-	smtp->buffer = (char *) MALLOC(SMTP_BUFFER_MAX);
-	smtp->email_to = (char *) MALLOC(SMTP_BUFFER_MAX);
+	smtp = alloc_smtp_msg_data();
 
 	/* format subject if rserver is specified */
 #ifdef _WITH_LVS_
 	if (msg_type == SMTP_MSG_RS) {
-		checker = (checker_t *)data;
+		checker = PTR_CAST(checker_t, data);
 		snprintf(smtp->subject, MAX_HEADERS_LENGTH, "[%s] Realserver %s of virtual server %s - %s",
 					global_data->router_id,
 					FMT_RS(checker->rs, checker->vs),
@@ -678,14 +700,14 @@ smtp_alert(smtp_msg_t msg_type, void* data, const char *subject, const char *bod
 					checker->rs->alive ? "UP" : "DOWN");
 	}
 	else if (msg_type == SMTP_MSG_VS) {
-		vs = (virtual_server_t *)data;
+		vs = PTR_CAST(virtual_server_t, data);
 		snprintf(smtp->subject, MAX_HEADERS_LENGTH, "[%s] Virtualserver %s - %s",
 					global_data->router_id,
 					FMT_VS(vs),
 					subject);
 	}
 	else if (msg_type == SMTP_MSG_RS_SHUT) {
-		rs_info = (smtp_rs *)data;
+		rs_info = PTR_CAST(smtp_rs, data);
 		snprintf(smtp->subject, MAX_HEADERS_LENGTH, "[%s] Realserver %s of virtual server %s - %s",
 					global_data->router_id,
 					FMT_RS(rs_info->rs, rs_info->vs),
@@ -696,13 +718,13 @@ smtp_alert(smtp_msg_t msg_type, void* data, const char *subject, const char *bod
 #endif
 #ifdef _WITH_VRRP_
 	if (msg_type == SMTP_MSG_VRRP) {
-		vrrp = (vrrp_t *)data;
+		vrrp = PTR_CAST(vrrp_t, data);
 		snprintf(smtp->subject, MAX_HEADERS_LENGTH, "[%s] VRRP Instance %s - %s",
 					global_data->router_id,
 					vrrp->iname,
 					subject);
 	} else if (msg_type == SMTP_MSG_VGROUP) {
-		vgroup = (vrrp_sgroup_t *)data;
+		vgroup = PTR_CAST(vrrp_sgroup_t, data);
 		snprintf(smtp->subject, MAX_HEADERS_LENGTH, "[%s] VRRP Group %s - %s",
 					global_data->router_id,
 					vgroup->gname,
