@@ -37,7 +37,7 @@
 static int
 snmp_keepalived_log(__attribute__((unused)) int major, __attribute__((unused)) int minor, void *serverarg, __attribute__((unused)) void *clientarg)
 {
-	struct snmp_log_message *slm = (struct snmp_log_message*)serverarg;
+	struct snmp_log_message *slm = PTR_CAST(struct snmp_log_message, serverarg);
 	int slm_len = strlen(slm->msg);
 
 	if (slm_len && slm->msg[slm_len-1] == '\n')
@@ -141,7 +141,7 @@ snmp_find_element(struct variable *vp, oid *name, size_t *length,
 		}
 
 		/* Find the list head of the inner list in the outer entry */
-		l1 = (list_head_t *) ((char *)e - offset_outer + offset_inner);
+		l1 = PTR_CAST(list_head_t, ((char *)e - offset_outer + offset_inner));
 
 		current[1] = 0;
 		list_for_each(e1, l1) {
@@ -223,24 +223,24 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 		return ret.p;
 	case SNMP_MAIL_SMTPSERVERADDRESSTYPE:
 		long_ret = (global_data->smtp_server.ss_family == AF_INET6)?2:1;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 	case SNMP_MAIL_SMTPSERVERADDRESS:
 		if (global_data->smtp_server.ss_family == AF_INET6) {
-			struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *)&global_data->smtp_server;
+			struct sockaddr_in6 *addr6 = PTR_CAST(struct sockaddr_in6, &global_data->smtp_server);
 			*var_len = 16;
-			return (u_char *)&addr6->sin6_addr;
+			return PTR_CAST(u_char, &addr6->sin6_addr);
 		} else {
-			struct sockaddr_in *addr4 = (struct sockaddr_in *)&global_data->smtp_server;
+			struct sockaddr_in *addr4 = PTR_CAST(struct sockaddr_in, &global_data->smtp_server);
 			*var_len = 4;
-			return (u_char *)&addr4->sin_addr;
+			return PTR_CAST(u_char, &addr4->sin_addr);
 		}
 		return NULL;
 	case SNMP_MAIL_SMTPSERVERPORT:
 		long_ret = ntohs(inet_sockaddrport(&global_data->smtp_server));
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 	case SNMP_MAIL_SMTPSERVERTIMEOUT:
 		long_ret = global_data->smtp_connection_to / TIMER_HZ;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 	case SNMP_MAIL_EMAILFROM:
 		if (!global_data->email_from) return NULL;
 		*var_len = strlen(global_data->email_from);
@@ -249,24 +249,24 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 #ifdef _WITH_VRRP_
 	case SNMP_MAIL_EMAILFAULTS:
 		long_ret = global_data->no_email_faults?2:1;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 	case SNMP_TRAPS:
 		long_ret = global_data->enable_traps?1:2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #ifdef _WITH_LINKBEAT_
 	case SNMP_LINKBEAT:
 		long_ret = global_data->linkbeat_use_polling?2:1;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 #ifdef _WITH_LVS_
 	case SNMP_LVSFLUSH:
 		long_ret = global_data->lvs_flush?1:2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 	case SNMP_LVSFLUSH_ONSTOP:
-		long_ret = global_data->lvs_flush_onstop == LVS_FLUSH_FULL ? 1 :
-			   global_data->lvs_flush_onstop == LVS_FLUSH_VS ? 3 : 2;
-		return (u_char *)&long_ret;
+		long_ret = global_data->lvs_flush_on_stop == LVS_FLUSH_FULL ? 1 :
+			   global_data->lvs_flush_on_stop == LVS_FLUSH_VS ? 3 : 2;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 	case SNMP_IPVS_64BIT_STATS:
 #ifdef _WITH_LVS_64BIT_STATS_
@@ -274,7 +274,7 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 #else
 		long_ret = 2;
 #endif
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 	case SNMP_NET_NAMESPACE:
 #if HAVE_DECL_CLONE_NEWNET
 		if (global_data->network_namespace) {
@@ -293,24 +293,24 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 		else
 #endif
 			long_ret = 2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #ifdef _WITH_VRRP_
 	case SNMP_DYNAMIC_INTERFACES:
 		long_ret = global_data->dynamic_interfaces ? 1 : 2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 	case SNMP_SMTP_ALERT:
 		long_ret = global_data->smtp_alert == -1 ? 3 : global_data->smtp_alert ? 1 : 2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #ifdef _WITH_VRRP_
 	case SNMP_SMTP_ALERT_VRRP:
 		long_ret = global_data->smtp_alert_vrrp == -1 ? 3 : global_data->smtp_alert_vrrp ? 1 : 2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 #ifdef _WITH_LVS_
 	case SNMP_SMTP_ALERT_CHECKER:
 		long_ret = global_data->smtp_alert_checker == -1 ? 3 : global_data->smtp_alert_checker ? 1 : 2;
-		return (u_char *)&long_ret;
+		return PTR_CAST(u_char, &long_ret);
 #endif
 	default:
 		break;
@@ -335,7 +335,7 @@ snmp_mail(struct variable *vp, oid *name, size_t *length,
 	switch (vp->magic) {
 	case SNMP_MAIL_EMAILADDRESS:
 		*var_len = strlen(email->addr);
-		return (u_char *)email->addr;
+		return PTR_CAST(u_char, email->addr);
 	default:
 		break;
 	}
@@ -414,7 +414,7 @@ void snmp_register_mib(oid *myoid, size_t len, const char *name,
 {
 	char name_buf[80];
 
-	if (register_mib(name, (struct variable *) variables, varsize,
+	if (register_mib(name, PTR_CAST(struct variable, variables), varsize,
 			 varlen, myoid, len) != MIB_REGISTERED_OK)
 		log_message(LOG_WARNING, "Unable to register %s MIB", name);
 
@@ -474,7 +474,7 @@ snmp_agent_init(const char *snmp_socket_name, bool base_mib)
 	init_agent(global_name);
 	if (base_mib)
 		snmp_register_mib(global_oid, OID_LENGTH(global_oid), global_name,
-				  (struct variable *)global_vars,
+				  PTR_CAST(struct variable, global_vars),
 				  sizeof(struct variable8),
 				  sizeof(global_vars)/sizeof(struct variable8));
 	init_snmp(global_name);
