@@ -49,13 +49,15 @@
 #define BFD_PID_FILE		"bfd"
 #endif
 #define	PID_EXTENSION		".pid"
+#define	RELOAD_EXTENSION	".reload"
 
 extern const char *pid_directory;
 
 /* Prototypes */
 extern void create_pid_dir(void);
 extern void remove_pid_dir(void);
-extern int pidfile_write(const char *, int);
+extern char *make_pidfile_name(const char *, const char *, const char *);
+extern bool pidfile_write(const char *, int);
 extern void pidfile_rm(const char *);
 extern bool keepalived_running(unsigned long);
 
