@@ -110,6 +110,7 @@ typedef struct _vrrp_sgroup {
 	unsigned		num_member_fault;	/* Number of members of group in fault state */
 	unsigned		num_member_init;	/* Number of members of group in pending state */
 	int			state;			/* current stable state */
+	bool			state_same_at_reload;	/* State prior to reload */
 	bool			sgroup_tracking_weight;	/* Use floating priority and scripts
 							 * Used if need different priorities needed on a track object in a sync group.
 							 * It probably won't work properly. */
@@ -132,6 +133,7 @@ typedef struct _vrrp_sgroup {
 	notify_script_t		*script;
 	int			smtp_alert;
 	int			last_email_state;
+	int			notify_priority_changes;
 
 	/* linked list member */
 	list_head_t		e_list;
@@ -273,6 +275,11 @@ typedef struct _vrrp_t {
 	unsigned		garp_lower_prio_rep;	/* Number of ARP messages to send at a time */
 	unsigned		lower_prio_no_advert;	/* Don't send advert after lower prio advert received */
 	unsigned		higher_prio_send_advert; /* Send advert after higher prio advert received */
+#ifdef _HAVE_VRRP_VMAC_
+	timeval_t		vmac_garp_intvl;	/* Interval between GARPs on each VMAC */
+	bool			vmac_garp_all_if;	/* Send GARPs on all i/fs, not just VMACs */
+	timeval_t		vmac_garp_timer;	/* Next scheduled GARP for each VMAC */
+#endif
 	uint8_t			vrid;			/* virtual id. from 1(!) to 255 */
 	uint8_t			base_priority;		/* configured priority value */
 	uint8_t			effective_priority;	/* effective priority value */
@@ -451,7 +458,10 @@ extern void restore_vrrp_interfaces(void);
 extern void shutdown_vrrp_instances(void);
 extern void clear_diff_vrrp(void);
 extern void clear_diff_script(void);
+extern void set_previous_sync_group_states(void);
+#ifdef _WITH_BFD_
 extern void clear_diff_bfd(void);
+#endif
 extern void vrrp_restore_interface(vrrp_t *, bool, bool);
 #ifdef THREAD_DUMP
 extern void register_vrrp_fifo_addresses(void);
