@@ -151,7 +151,10 @@ typedef struct _interface {
 #endif
 	struct _interface	*base_ifp;		/* Base interface (if interface is a VMAC interface),
 							   otherwise the physical interface */
+#ifdef _HAVE_VRRP_VMAC_
 	bool			is_ours;		/* keepalived created the interface */
+	bool			deleting;		/* Set when we are deleting the interface */
+#endif
 	bool			seen_interface;		/* The interface has existed at some point since we started */
 	bool			changeable_type;	/* The interface type or underlying interface can be changed */
 #ifdef _HAVE_VRF_
@@ -213,7 +216,8 @@ typedef enum if_lookup {
 	IF_NO_CREATE,
 	IF_CREATE_IF_DYNAMIC,
 	IF_CREATE_ALWAYS,
-	IF_CREATE_NETLINK
+	IF_CREATE_NETLINK,
+	IF_CREATE_NOT_EXIST,
 } if_lookup_t;
 
 /* Global data */
@@ -221,6 +225,9 @@ extern list_head_t garp_delay;
 
 /* prototypes */
 extern interface_t *if_get_by_ifindex(ifindex_t) __attribute__ ((pure));
+#ifdef _HAVE_VRRP_VMAC_
+extern interface_t * if_get_by_vmac(uint8_t, int, const interface_t *) __attribute__ ((pure));
+#endif
 extern interface_t *get_default_if(void);
 extern interface_t *if_get_by_ifname(const char *, if_lookup_t);
 extern sin_addr_t *if_extra_ipaddress_alloc(interface_t *, void *, unsigned char);
