@@ -39,11 +39,12 @@
 
 /* Defines */
 enum vrrp_vmac_bits {
-	VRRP_VMAC_BIT = 0,
-	VRRP_VMAC_UP_BIT = 1,
-	VRRP_VMAC_XMITBASE_BIT = 2,
+	VRRP_VMAC_BIT,
+	VRRP_VMAC_UP_BIT,
+	VRRP_VMAC_XMITBASE_BIT,
+	VRRP_VMAC_ADDR_BIT,
 #ifdef _HAVE_VRRP_IPVLAN_
-	VRRP_IPVLAN_BIT = 3,
+	VRRP_IPVLAN_BIT,
 #endif
 };
 
