@@ -239,7 +239,7 @@ static gchar * __attribute__ ((malloc))
 dbus_object_create_path_instance(const gchar *interface, int vrid, sa_family_t family)
 {
 	gchar *object_path;
-	char standardized_name[sizeof ((vrrp_t*)NULL)->ifp->ifname];
+	char standardized_name[sizeof (PTR_CAST(vrrp_t, NULL))->ifp->ifname];
 	gchar *vrid_str = g_strdup_printf("%d", vrid);
 
 	set_valid_path(standardized_name, interface);
@@ -338,7 +338,7 @@ handle_get_property(__attribute__((unused)) GDBusConnection *connection,
 {
 	GVariant *ret = NULL;
 	dbus_queue_ent_t ent;
-	char ifname_str[sizeof ((vrrp_t*)NULL)->ifp->ifname];
+	char ifname_str[sizeof (PTR_CAST(vrrp_t, NULL))->ifp->ifname];
 	int action;
 
 	if (g_strcmp0(interface_name, DBUS_VRRP_INSTANCE_INTERFACE)) {
@@ -390,7 +390,7 @@ handle_method_call(__attribute__((unused)) GDBusConnection *connection,
 	unsigned family;
 #endif
 	dbus_queue_ent_t ent;
-	char ifname_str[sizeof ((vrrp_t*)NULL)->ifp->ifname];
+	char ifname_str[sizeof (PTR_CAST(vrrp_t, NULL))->ifp->ifname];
 
 	if (!g_strcmp0(interface_name, DBUS_VRRP_INTERFACE)) {
 		if (!g_strcmp0(method_name, "PrintData")) {
