@@ -176,12 +176,20 @@ int
 ip4tables_process_entry(struct iptc_handle *handle, const char *chain_name, unsigned int rulenum,
 			const char *target_name, const ip_address_t *src_ip_address,
 			const ip_address_t *dst_ip_address, const char *in_iface, const char *out_iface,
-			uint16_t protocol, uint8_t type, int cmd, uint8_t flags, bool force)
+#ifdef _INCLUDE_UNUSED_CODE_
+			uint16_t protocol, uint8_t type,
+#else
+			__attribute__((unused)) uint16_t protocol, __attribute__((unused)) uint8_t type,
+#endif
+			int cmd, uint8_t flags, bool force)
 {
 	size_t size;
 	struct ipt_entry *fw;
 	struct xt_entry_target *target;
+#ifdef _INCLUDE_UNUSED_CODE_
 	struct xt_entry_match *match ;
+	struct ipt_icmp *icmpinfo;
+#endif
 	ipt_chainlabel chain;
 	int res;
 	int sav_errno;
@@ -193,10 +201,12 @@ ip4tables_process_entry(struct iptc_handle *handle, const char *chain_name, unsi
 	size = XT_ALIGN (sizeof (struct ipt_entry)) +
 			XT_ALIGN (sizeof (struct xt_entry_target) + 1);
 
+#ifdef _INCLUDE_UNUSED_CODE_
 	if ( protocol == IPPROTO_ICMP )
 		size += XT_ALIGN ( sizeof(struct xt_entry_match) ) + XT_ALIGN ( sizeof(struct ipt_icmp) ) ;
+#endif
 
-	fw = (struct ipt_entry*)MALLOC(size);
+	fw = PTR_CAST(struct ipt_entry, MALLOC(size));
 
 	fw->target_offset = XT_ALIGN ( sizeof ( struct ipt_entry ) ) ;
 
@@ -224,20 +234,22 @@ ip4tables_process_entry(struct iptc_handle *handle, const char *chain_name, unsi
 
 //		fw->ip.flags |= IP6T_F_PROTO ;		// IPv6 only
 
+#ifdef _INCLUDE_UNUSED_CODE_
 		if ( protocol == IPPROTO_ICMP )
 		{
-			match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+			match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 			match->u.match_size = XT_ALIGN(sizeof (struct xt_entry_match)) + XT_ALIGN(sizeof(struct ipt_icmp));
 			match->u.user.revision = 0;
 			fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 			strcpy ( match->u.user.name, "icmp" ) ;
 
-			struct ipt_icmp *icmpinfo = (struct ipt_icmp *) match->data;
+			icmpinfo = PTR_CAST(struct ipt_icmp, match->data);
 			icmpinfo->type = type ;		// type to match
 			icmpinfo->code[0] = 0 ;		// code lower
 			icmpinfo->code[1] = 0xff ;	// code upper
 			icmpinfo->invflags = 0 ;	// don't invert
 		}
+#endif
 	}
 
 // target is XTC_LABEL_DROP/XTC_LABEL_ACCEPT
@@ -323,6 +335,7 @@ ip6tables_process_entry(struct ip6tc_handle *handle, const char *chain_name, uns
 	struct ip6t_entry *fw;
 	struct xt_entry_target *target;
 	struct xt_entry_match *match ;
+	struct ip6t_icmp *icmpinfo;
 	ip6t_chainlabel chain;
 	int res;
 	int sav_errno;
@@ -337,7 +350,7 @@ ip6tables_process_entry(struct ip6tc_handle *handle, const char *chain_name, uns
 	if ( protocol == IPPROTO_ICMPV6 )
 		size += XT_ALIGN ( sizeof(struct xt_entry_match) ) + XT_ALIGN ( sizeof(struct ip6t_icmp) ) ;
 
-	fw = (struct ip6t_entry*)MALLOC(size);
+	fw = PTR_CAST(struct ip6t_entry, MALLOC(size));
 
 	fw->target_offset = XT_ALIGN ( sizeof ( struct ip6t_entry ) ) ;
 
@@ -365,13 +378,13 @@ ip6tables_process_entry(struct ip6tc_handle *handle, const char *chain_name, uns
 
 		if ( protocol == IPPROTO_ICMPV6 )
 		{
-			match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+			match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 			match->u.match_size = XT_ALIGN ( sizeof (struct xt_entry_match) ) + XT_ALIGN ( sizeof (struct ip6t_icmp) ) ;
 			match->u.user.revision = 0;
 			fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 			strcpy ( match->u.user.name, "icmp6" ) ;
 
-			struct ip6t_icmp *icmpinfo = (struct ip6t_icmp *) match->data;
+			icmpinfo = PTR_CAST(struct ip6t_icmp, match->data);
 			icmpinfo->type = type ;		// type to match
 			icmpinfo->code[0] = 0 ;		// code lower
 			icmpinfo->code[1] = 0xff ;	// code upper
@@ -570,7 +583,12 @@ int
 ip4tables_add_rules(struct iptc_handle *handle, const char *chain_name, unsigned int rulenum,
 		    uint8_t dim, uint8_t src_dst, const char* target_name,
 		    const ip_address_t *src_ip_address, const ip_address_t *dst_ip_address, const char *set_name,
-		    uint16_t protocol, uint8_t param, int cmd, bool ignore_errors)
+#ifdef _INCLUDE_UNUSED_CODE_
+		    uint16_t protocol, uint8_t param,
+#else
+		    __attribute__((unused)) uint16_t protocol, __attribute__((unused)) uint8_t param,
+#endif
+		    int cmd, bool ignore_errors)
 {
 	size_t size;
 	struct ipt_entry *fw;
@@ -595,10 +613,12 @@ ip4tables_add_rules(struct iptc_handle *handle, const char *chain_name, unsigned
 			XT_ALIGN(sizeof(struct xt_entry_target) + 1) +
 			XT_ALIGN(sizeof(*setinfo));
 
+#ifdef _INCLUDE_UNUSED_CODE_
 	if (protocol == IPPROTO_ICMP)
 		size += XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(struct ipt_icmp));
+#endif
 
-	fw = (struct ipt_entry*)MALLOC(size);
+	fw = PTR_CAST(struct ipt_entry, MALLOC(size));
 
 	fw->target_offset = XT_ALIGN(sizeof(struct ipt_entry));
 
@@ -615,7 +635,7 @@ ip4tables_add_rules(struct iptc_handle *handle, const char *chain_name, unsigned
 	}
 
 	// set
-	match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+	match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 	match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(*setinfo));
 #ifdef HAVE_XT_SET_INFO_MATCH_V4
 	match->u.user.revision = 4;
@@ -630,13 +650,13 @@ ip4tables_add_rules(struct iptc_handle *handle, const char *chain_name, unsigned
 	strcpy(match->u.user.name, "set");
 
 #ifdef HAVE_XT_SET_INFO_MATCH_V4
-	setinfo = (struct xt_set_info_match_v4 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v4, match->data);
 #elif defined HAVE_XT_SET_INFO_MATCH_V3
-	setinfo = (struct xt_set_info_match_v3 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v3, match->data);
 #elif defined HAVE_XT_SET_INFO_MATCH_V1
-	setinfo = (struct xt_set_info_match_v1 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v1, match->data);
 #else
-	setinfo = (struct xt_set_info_match *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match, match->data);
 #endif
 
 	get_set_byname(set_name, &setinfo->match_set, NFPROTO_IPV4, ignore_errors);
@@ -650,23 +670,25 @@ ip4tables_add_rules(struct iptc_handle *handle, const char *chain_name, unsigned
 
 	if (protocol != IPPROTO_NONE) {
 		fw->ip.proto = protocol;
+#ifdef _INCLUDE_UNUSED_CODE_
 
 //		fw->ip.flags |= IP6T_F_PROTO ;		// IPv6 only
 
 		if (protocol == IPPROTO_ICMP)
 		{
-			match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+			match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 			match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(struct ipt_icmp));
 			match->u.user.revision = 0;
 			fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 			strcpy(match->u.user.name, "icmp");
 
-			struct ipt_icmp *icmpinfo = (struct ipt_icmp *)match->data;
+			struct ipt_icmp *icmpinfo = PTR_CAST(struct ipt_icmp, match->data);
 			icmpinfo->type = param;		// type to match
 			icmpinfo->code[0] = 0;		// code lower
 			icmpinfo->code[1] = 0xff;	// code upper
 			icmpinfo->invflags = 0;		// don't invert
 		}
+#endif
 	}
 
 // target is XTC_LABEL_DROP/XTC_LABEL_ACCEPT
@@ -739,7 +761,7 @@ ip6tables_add_rules(struct ip6tc_handle *handle, const char *chain_name, unsigne
 	if (protocol == IPPROTO_ICMPV6)
 		size += XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(struct ip6t_icmp));
 
-	fw = (struct ip6t_entry*)MALLOC(size);
+	fw = PTR_CAST(struct ip6t_entry, MALLOC(size));
 
 	if (src_ip_address && src_ip_address->ifa.ifa_family != AF_UNSPEC) {
 //		memcpy(&fw->ipv6.src, &src_ip_address->u.sin6_addr, sizeof(src_ip_address->u.sin6_addr));
@@ -756,7 +778,7 @@ ip6tables_add_rules(struct ip6tc_handle *handle, const char *chain_name, unsigne
 	fw->target_offset = XT_ALIGN(sizeof(struct ip6t_entry));
 
 	// set
-	match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+	match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 	match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(*setinfo));
 #ifdef HAVE_XT_SET_INFO_MATCH_V4
 	match->u.user.revision = 4;
@@ -771,13 +793,13 @@ ip6tables_add_rules(struct ip6tc_handle *handle, const char *chain_name, unsigne
 	strcpy(match->u.user.name, "set");
 
 #ifdef HAVE_XT_SET_INFO_MATCH_V4
-	setinfo = (struct xt_set_info_match_v4 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v4, match->data);
 #elif defined HAVE_XT_SET_INFO_MATCH_V3
-	setinfo = (struct xt_set_info_match_v3 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v3, match->data);
 #elif defined HAVE_XT_SET_INFO_MATCH_V1
-	setinfo = (struct xt_set_info_match_v1 *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match_v1, match->data);
 #else
-	setinfo = (struct xt_set_info_match *)match->data;
+	setinfo = PTR_CAST(struct xt_set_info_match, match->data);
 #endif
 
 	get_set_byname (set_name, &setinfo->match_set, NFPROTO_IPV6, ignore_errors);
@@ -796,13 +818,13 @@ ip6tables_add_rules(struct ip6tc_handle *handle, const char *chain_name, unsigne
 
 		if (protocol == IPPROTO_ICMPV6)
 		{
-			match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
+			match = PTR_CAST(struct xt_entry_match, ((char*)fw + fw->target_offset));
 			match->u.match_size = XT_ALIGN(sizeof(struct xt_entry_match)) + XT_ALIGN(sizeof(struct ip6t_icmp));
 			match->u.user.revision = 0;
 			fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 			strcpy(match->u.user.name, "icmp6");
 
-			struct ip6t_icmp *icmpinfo = (struct ip6t_icmp *)match->data;
+			struct ip6t_icmp *icmpinfo = PTR_CAST(struct ip6t_icmp, match->data);
 			icmpinfo->type = param;		// type to match
 			icmpinfo->code[0] = 0;		// code lower
 			icmpinfo->code[1] = 0xff;	// code upper
