@@ -69,14 +69,24 @@
 /* For load testing recvmsg() */
 /* #define DEBUG_RECVMSG */
 
+/* For _RECVMSG_DEBUG_ we want load testing code as well */
+#ifdef _RECVMSG_DEBUG_
+#define DEBUG_RECVMSG	1
+#endif
+
 /* global vars */
 timeval_t garp_next_time;
 thread_ref_t garp_thread;
 bool vrrp_initialised;
+timeval_t vrrp_delayed_start_time;
 
 #ifdef _TSM_DEBUG_
 bool do_tsm_debug;
 #endif
+#ifdef _RECVMSG_DEBUG_
+bool do_recvmsg_debug;
+bool do_recvmsg_debug_dump;
+#endif
 
 /* local variables */
 #ifdef _WITH_BFD_
@@ -184,9 +194,11 @@ vrrp_init_state(list_head_t *l)
 	/* Do notifications for any sync groups in fault or backup state */
 	list_for_each_entry(vgroup, &vrrp_data->vrrp_sync_group, e_list) {
 		/* Init group if needed  */
-		if (vgroup->state == VRRP_STATE_FAULT ||
-		    vgroup->state == VRRP_STATE_BACK)
+		if ((vgroup->state == VRRP_STATE_FAULT ||
+		     vgroup->state == VRRP_STATE_BACK) &&
+		     !vgroup->state_same_at_reload)
 			send_group_notifies(vgroup);
+		vgroup->state_same_at_reload = false;
 	}
 
 	list_for_each_entry(vrrp, l, e_list) {
@@ -297,7 +309,18 @@ vrrp_init_instance_sands(vrrp_t *vrrp)
 		 * time_now plus the Master Down Timer, when a non-preemptable packet is
 		 * received.
 		 */
-		vrrp->sands = timer_add_long(time_now, vrrp->ms_down_timer);
+		if (vrrp_delayed_start_time.tv_sec) {
+			if (timercmp(&time_now, &vrrp_delayed_start_time, <))
+				vrrp->sands = timer_add_long(vrrp_delayed_start_time, vrrp->ms_down_timer);
+			else {
+				/* If we clear the delayed_start_time once past, then
+				 * the code will be slightly more efficient */
+				if (time_now.tv_sec > vrrp_delayed_start_time.tv_sec)
+					vrrp_delayed_start_time.tv_sec = 0;
+				vrrp->sands = timer_add_long(time_now, vrrp->ms_down_timer);
+			}
+		} else
+			vrrp->sands = timer_add_long(time_now, vrrp->ms_down_timer);
 	}
 	else if (vrrp->state == VRRP_STATE_FAULT || vrrp->state == VRRP_STATE_INIT)
 		vrrp->sands.tv_sec = TIMER_DISABLED;
@@ -450,17 +473,17 @@ vrrp_create_sockpool(list_head_t *l)
 	struct sockaddr_storage *unicast_src;
 
 	list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
-		if (list_empty(&vrrp->unicast_peer)) {
-			ifp =
-#ifdef _HAVE_VRRP_VMAC_
-			      (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? vrrp->configured_ifp :
-#endif
-											vrrp->ifp;
+		if (list_empty(&vrrp->unicast_peer))
 			unicast_src = NULL;
-		} else {
+		else
 			unicast_src = &vrrp->saddr;
-			ifp = vrrp->ifp;
-		}
+
+		ifp =
+#ifdef _HAVE_VRRP_VMAC_
+		      (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? vrrp->configured_ifp :
+#endif
+										vrrp->ifp;
+
 		proto = IPPROTO_VRRP;
 #if defined _WITH_VRRP_AUTH_
 		if (vrrp->auth_type == VRRP_AUTH_AH)
@@ -598,13 +621,6 @@ vrrp_lower_prio_gratuitous_arp_thread(thread_ref_t thread)
 	vrrp_send_link_update(vrrp, vrrp->garp_lower_prio_rep);
 }
 
-static void
-vrrp_master(vrrp_t * vrrp)
-{
-	/* Send the VRRP advert */
-	vrrp_state_master_tx(vrrp);
-}
-
 void
 try_up_instance(vrrp_t *vrrp, bool leaving_init)
 {
@@ -677,11 +693,11 @@ try_up_instance(vrrp_t *vrrp, bool leaving_init)
 		ip_addr.ifp = IF_BASE_IFP(vrrp->ifp);
 
 		if (vrrp->saddr.ss_family == AF_INET) {
-			ip_addr.u.sin.sin_addr.s_addr = ((struct sockaddr_in *)&vrrp->saddr)->sin_addr.s_addr;
+			ip_addr.u.sin.sin_addr.s_addr = PTR_CAST(struct sockaddr_in, &vrrp->saddr)->sin_addr.s_addr;
 			send_gratuitous_arp_immediate(ip_addr.ifp, &ip_addr);
 		} else {
 			/* IPv6 */
-			ip_addr.u.sin6_addr = ((struct sockaddr_in6 *)&vrrp->saddr)->sin6_addr;
+			ip_addr.u.sin6_addr = PTR_CAST(struct sockaddr_in6, &vrrp->saddr)->sin6_addr;
 			ndisc_send_unsolicited_na_immediate(ip_addr.ifp, &ip_addr);
 		}
 	}
@@ -793,7 +809,7 @@ vrrp_dispatcher_read_timeout(sock_t *sock)
 			vrrp_goto_master(vrrp);
 		}
 		else if (vrrp->state == VRRP_STATE_MAST)
-			vrrp_master(vrrp);
+			vrrp_state_master_tx(vrrp);
 
 		/* handle instance synchronization */
 #ifdef _TSM_DEBUG_
@@ -820,9 +836,9 @@ vrrp_dispatcher_read(sock_t *sock)
 	struct sockaddr_storage src_addr = { .ss_family = AF_UNSPEC };
 	vrrp_t vrrp_lookup;
 #ifdef _NETWORK_TIMESTAMP_
-	char control_buf[128];
+	char control_buf[128] __attribute__((aligned(__alignof__(struct cmsghdr))));
 #else
-	char control_buf[64];
+	char control_buf[64] __attribute__((aligned(__alignof__(struct cmsghdr))));
 #endif
 	struct iovec iovec = { .iov_base = vrrp_buffer, .iov_len = vrrp_buffer_len };
 	struct msghdr msghdr = { .msg_name = &src_addr, .msg_namelen = sizeof(src_addr),
@@ -849,27 +865,49 @@ vrrp_dispatcher_read(sock_t *sock)
 		       check_EINTR(errno) && eintr_count++ < 10);
 		if (len < 0) {
 #ifdef DEBUG_RECVMSG
-			if (check_EINTR(errno))
-				log_message(LOG_INFO, "recvmsg(%d) looped %u times due to EINTR before terminating loop"
-						    , sock->fd_in, eintr_count);
+#ifdef _RECVMSG_DEBUG_
+			if (do_recvmsg_debug && (!recv_data_count || !check_EAGAIN(errno)))
+				log_message(LOG_INFO, "recvmsg(%d) returned errno %d, %u eintr", sock->fd_in, errno, eintr_count);
+#endif
+
+#ifdef _RECVMSG_DEBUG_
+			if (do_recvmsg_debug)
+#endif
+			{
+				if (check_EINTR(errno))
+					log_message(LOG_INFO, "recvmsg(%d) looped %u times due to EINTR before terminating loop"
+							    , sock->fd_in, eintr_count);
+			}
 #endif
 
 			if (!check_EAGAIN(errno))
 				log_message(LOG_INFO, "recvmsg(%d) returned %d (%m)"
 						    , sock->fd_in, errno);
 #ifdef DEBUG_RECVMSG
-			else if (recv_data_count == 0)
+			else if (
+#ifdef _RECVMSG_DEBUG_
+				 do_recvmsg_debug &&
+#endif
+				 recv_data_count == 0)
 				log_message(LOG_INFO, "recvmsg(%d) returned EAGAIN without any data being received"
 						    , sock->fd_in);
 
-			if (recv_data_count != 1)
-				log_message(LOG_INFO, "recvmsg(%d) loop received %u packets"
-						    , sock->fd_in, recv_data_count);
+#ifdef _RECVMSG_DEBUG_
+			if (do_recvmsg_debug)
+#endif
+			{
+				if (recv_data_count != 1)
+					log_message(LOG_INFO, "recvmsg(%d) loop received %u packets"
+							    , sock->fd_in, recv_data_count);
+			}
 #endif
 			break;
 		}
-
-#ifdef DEBUG_RECVMSG
+#ifdef _RECVMSG_DEBUG_
+		else if (do_recvmsg_debug)
+			log_message(LOG_INFO, "recvmsg(%d) looped %u times due to EINTR before returning %ld bytes from %s"
+					    , sock->fd_in, eintr_count, len, inet_sockaddrtos(&src_addr));
+#elif defined DEBUG_RECVMSG
 		if (eintr_count)
 			log_message(LOG_INFO, "recvmsg(%d) looped %u times due to EINTR before returning %ld"
 					    , sock->fd_in, eintr_count, len);
@@ -881,6 +919,12 @@ vrrp_dispatcher_read(sock_t *sock)
 			continue;
 		}
 
+#ifdef _RECVMSG_DEBUG_
+		if (do_recvmsg_debug_dump) {
+			log_buffer("Received data", vrrp_buffer, len);
+		}
+#endif
+
 #ifdef DEBUG_RECVMSG
 		recv_data_count++;
 #endif
@@ -929,7 +973,7 @@ vrrp_dispatcher_read(sock_t *sock)
 		vrrp->pkt_saddr = src_addr;
 		vrrp->rx_ttl_hop_limit = -1;           /* Default to not received */
 		if (sock->family == AF_INET) {
-			iph = (const struct iphdr *)vrrp_buffer;
+			iph = PTR_CAST_CONST(struct iphdr, vrrp_buffer);
 			vrrp->multicast_pkt = IN_MULTICAST(htonl(iph->daddr));
 			vrrp->rx_ttl_hop_limit = iph->ttl;
 		} else
@@ -942,13 +986,13 @@ vrrp_dispatcher_read(sock_t *sock)
 #ifdef IPV6_RECVHOPLIMIT
 				if (cmsg->cmsg_type == IPV6_HOPLIMIT &&
 				    cmsg->cmsg_len - sizeof(struct cmsghdr) == sizeof(unsigned int))
-					vrrp->rx_ttl_hop_limit = *(unsigned int *)CMSG_DATA(cmsg);
+					vrrp->rx_ttl_hop_limit = *PTR_CAST(unsigned int, CMSG_DATA(cmsg));
 				else
 #endif
 #ifdef IPV6_RECVPKTINFO
 				if (cmsg->cmsg_type == IPV6_PKTINFO &&
 				    cmsg->cmsg_len - sizeof(struct cmsghdr) == sizeof(struct in6_pktinfo))
-					vrrp->multicast_pkt = IN6_IS_ADDR_MULTICAST(&((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_addr);
+					vrrp->multicast_pkt = IN6_IS_ADDR_MULTICAST(&(PTR_CAST(struct in6_pktinfo, CMSG_DATA(cmsg)))->ipi6_addr);
 				else
 #endif
 					expected_cmsg = false;
@@ -1339,7 +1383,7 @@ dump_threads(void)
 	vrrp_t *vrrp;
 	const char *file_name;
 
-	file_name = make_file_name("/tmp/thread_dump.dat",
+	file_name = make_file_name(KA_TMP_DIR "/thread_dump.dat",
 					"vrrp",
 #if HAVE_DECL_CLONE_NEWNET
 					global_data->network_namespace,
