@@ -42,8 +42,10 @@
 #include "bitops.h"
 #include "logger.h"
 #include "scheduler.h"
+#include "process.h"
 
 #ifdef _MEM_CHECK_
+#include "align.h"
 #include "timer.h"
 #include "rbtree.h"
 #include "list_head.h"
@@ -75,9 +77,11 @@ xalloc(unsigned long size)
 	}
 
 #ifdef _MEM_CHECK_
-	mem_allocated += size - sizeof(long);
-	if (mem_allocated > max_mem_allocated)
-		max_mem_allocated = mem_allocated;
+	if (__test_bit(MEM_CHECK_BIT, &debug)) {
+		mem_allocated += size - sizeof(long);
+		if (mem_allocated > max_mem_allocated)
+			max_mem_allocated = mem_allocated;
+	}
 #endif
 
 	return mem;
@@ -163,12 +167,12 @@ typedef struct {
 } MEMCHECK;
 
 /* Last free pointers */
-static LH_LIST_HEAD(free_list);
+static LIST_HEAD_INITIALIZE(free_list);
 static unsigned free_list_size;
 
 /* alloc_list entries used for 1000 VRRP instance each with VMAC interfaces is 33589 */
 static rb_root_t alloc_list = RB_ROOT;
-static LH_LIST_HEAD(bad_list);
+static LIST_HEAD_INITIALIZE(bad_list);
 
 static unsigned number_alloc_list;	/* number of alloc_list allocation entries */
 static unsigned max_alloc_list;
@@ -203,6 +207,9 @@ format_time(void)
 void
 memcheck_log(const char *called_func, const char *param, const char *file, const char *function, int line)
 {
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return;
+
 	int len = strlen(called_func) + (param ? strlen(param) : 0);
 
 	if ((len = 36 - len) < 0)
@@ -222,7 +229,7 @@ get_free_alloc_entry(void)
 		entry = malloc(sizeof *entry);
 	else {
 		entry = list_first_entry(&free_list, MEMCHECK, l);
-		list_head_del(&entry->l);
+		list_del_init(&entry->l);
 		free_list_size--;
 	}
 
@@ -240,7 +247,7 @@ keepalived_malloc_common(size_t size, const char *file, const char *function, in
 	buf = zalloc(size + sizeof (unsigned long));
 
 #ifndef _NO_UNALIGNED_ACCESS_
-	*(unsigned long *) ((char *) buf + size) = size + CHECK_VAL;
+	*(unsigned long *) PTR_CAST_ASSIGN((char *) buf + size) = size + CHECK_VAL;
 #else
 	unsigned long check_val = CHECK_VAL;
 
@@ -285,6 +292,9 @@ keepalived_malloc_common(size_t size, const char *file, const char *function, in
 void *
 keepalived_malloc(size_t size, const char *file, const char *function, int line)
 {
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return zalloc(size);
+
 	return keepalived_malloc_common(size, file, function, line, "zalloc");
 }
 
@@ -293,6 +303,9 @@ keepalived_strdup(const char *str, const char *file, const char *function, int l
 {
 	char *str_p;
 
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return strdup(str);
+
 	str_p = keepalived_malloc_common(strlen(str) + 1, file, function, line, "strdup");
 	return strcpy(str_p, str);
 }
@@ -302,6 +315,9 @@ keepalived_strndup(const char *str, size_t size, const char *file, const char *f
 {
 	char *str_p;
 
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return strndup(str, size);
+
 	str_p = keepalived_malloc_common(size + 1, file, function, line, "strndup");
 	return strncpy(str_p, str, size);
 }
@@ -394,7 +410,7 @@ keepalived_free_realloc_common(void *buffer, size_t size, const char *file, cons
 
 	check = entry->size + CHECK_VAL;
 #ifndef _NO_UNALIGNED_ACCESS_
-	if (*(unsigned long *)((char *)buffer + entry->size) != check) {
+	if (*(unsigned long *) PTR_CAST_ASSIGN((char *)buffer + entry->size) != check) {
 #else
 	if (memcmp((unsigned char *)buffer + entry->size, (unsigned char *)&check_val, sizeof(check_val))) {
 #endif
@@ -493,7 +509,7 @@ keepalived_free_realloc_common(void *buffer, size_t size, const char *file, cons
 #endif
 
 #ifndef _NO_UNALIGNED_ACCESS_
-	*(unsigned long *) ((char *) buffer + size) = size + CHECK_VAL;
+	*(unsigned long *) PTR_CAST_ASSIGN((char *) buffer + size) = size + CHECK_VAL;
 #else
 	memcpy((unsigned char *)buffer + size, (unsigned char *)&check_val, sizeof(check_val));
 #endif
@@ -518,6 +534,11 @@ keepalived_free_realloc_common(void *buffer, size_t size, const char *file, cons
 void
 keepalived_free(void *buffer, const char *file, const char *function, int line)
 {
+	if (!__test_bit(MEM_CHECK_BIT, &debug)) {
+		free(buffer);
+		return;
+	}
+
 	keepalived_free_realloc_common(buffer, 0, file, function, line, false);
 }
 
@@ -525,6 +546,9 @@ void *
 keepalived_realloc(void *buffer, size_t size, const char *file,
 		   const char *function, int line)
 {
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return realloc(buffer, size);
+
 	return keepalived_free_realloc_common(buffer, size, file, function, line, true);
 }
 
@@ -535,6 +559,9 @@ keepalived_alloc_log(bool final)
 	size_t sum = 0;
 	MEMCHECK *entry;
 
+	if (!__test_bit(MEM_CHECK_BIT, &debug))
+		return;
+
 	if (final) {
 		/* If this is a forked child, we don't want the dump */
 		if (skip_mem_check_final)
@@ -669,7 +696,7 @@ mem_log_init(const char* prog_name, const char *banner)
 	if (log_op)
 		fclose(log_op);
 
-	log_name_len = 5 + strlen(prog_name) + 5 + 7 + 4 + 1;	/* "/tmp/" + prog_name + "_mem." + PID + ".log" + '\0" */
+	log_name_len = strlen(KA_TMP_DIR) + 1 + strlen(prog_name) + 5 + PID_MAX_DIGITS + 4 + 1;	/* KA_TMP_DIR + "/" + prog_name + "_mem." + PID + ".log" + '\0" */
 	log_name = malloc(log_name_len);
 	if (!log_name) {
 		log_message(LOG_INFO, "Unable to malloc log file name");
@@ -677,7 +704,7 @@ mem_log_init(const char* prog_name, const char *banner)
 		return;
 	}
 
-	snprintf(log_name, log_name_len, "/tmp/%s_mem.%d.log", prog_name, getpid());
+	snprintf(log_name, log_name_len, KA_TMP_DIR "/%s_mem.%d.log", prog_name, getpid());
 	log_op = fopen_safe(log_name, "w");
 	if (log_op == NULL) {
 		log_message(LOG_INFO, "Unable to open %s for appending", log_name);
@@ -719,4 +746,15 @@ update_mem_check_log_perms(mode_t umask_bits)
 	if (log_op)
 		fchmod(fileno(log_op), (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH) & ~umask_bits);
 }
+
+void
+log_mem_check_message(const char *format, ...)
+{
+	va_list args;
+
+	va_start(args, format);
+	vfprintf(log_op, format, args);
+	va_end(args);
+	fprintf(log_op, "\n");
+}
 #endif
