@@ -78,25 +78,28 @@ prog_type_t prog_type;		/* Parent/VRRP/Checker process */
 #ifdef _WITH_SNMP_
 bool snmp_running;		/* True if this process is running SNMP */
 #endif
-
-/* local variables */
-static bool shutting_down;
-static int sav_argc;
-static char * const *sav_argv;
 #ifdef _EPOLL_DEBUG_
 bool do_epoll_debug;
 #endif
 #ifdef _EPOLL_THREAD_DUMP_
 bool do_epoll_thread_dump;
 #endif
+#ifdef _SCRIPT_DEBUG_
+bool do_script_debug;
+#endif
+
+/* local variables */
+static bool shutting_down;
+static int sav_argc;
+static char * const *sav_argv;
 #ifdef THREAD_DUMP
 static rb_root_t funcs = RB_ROOT;
 #endif
 #ifdef _VRRP_FD_DEBUG_
 static void (*extra_threads_debug)(void);
 #endif
-#ifdef _SCRIPT_DEBUG_
-bool do_script_debug;
+#ifndef _ONE_PROCESS_DEBUG_
+static void (*shutdown_function)(int);
 #endif
 
 /* Function that returns prog_name if pid is a known child */
@@ -179,7 +182,7 @@ register_thread_address(const char *func_name, thread_func_t func)
 {
 	func_det_t *func_det;
 
-	func_det = (func_det_t *) MALLOC(sizeof(func_det_t));
+	PMALLOC(func_det);
 	if (!func_det)
 		return;
 
@@ -219,6 +222,22 @@ set_extra_threads_debug(void (*func)(void))
 }
 #endif
 
+#ifndef _ONE_PROCESS_DEBUG_
+/* The shutdown function is called if the scheduler gets repeated errors calling
+ * epoll_wait() and so is unable to continue.
+ * github issue 1809 reported the healthchecker process getting error EINVAL with
+ * a particular configuration; this looks as though it was memory corruption but
+ * we have no way of tracking down how that happened. This provides a way to escape
+ * the error if it happens again, by the process terminating, and it will then be
+ * restarted by the parent process. */
+void
+register_shutdown_function(void (*func)(int))
+{
+	/* The function passed here must not use the scheduler to shutdown */
+	shutdown_function = func;
+}
+#endif
+
 /* Move ready thread into ready queue */
 static int
 thread_move_ready(thread_master_t *m, rb_root_cached_t *root, thread_t *thread, int type)
@@ -364,6 +383,14 @@ save_cmd_line_options(int argc, char * const *argv)
 	sav_argv = argv;
 }
 
+char * const *
+get_cmd_line_options(int *argc)
+{
+	*argc = sav_argc;
+	return sav_argv;
+}
+
+
 static const char *
 get_end(const char *str, size_t max_len)
 {
@@ -450,6 +477,18 @@ calc_restart_delay(const timeval_t *last_start_time, unsigned *next_restart_dela
 		return 0;
 	}
 
+#if 0
+	/* If it ran for longer than the last restart delay, we can start
+	 * again immediately. */
+	if (restart_delay &&
+	    (time_now.tv_sec - last_start_time->tv_sec > restart_delay ||
+	     (time_now.tv_sec - last_start_time->tv_sec == restart_delay &&
+	      time_now.tv_usec >= last_start_time->tv_usec))) {
+		*next_restart_delay = 0;
+		return 0;
+	}
+#endif
+
 	/* next restart delay starts at 1, double each subsequent time,
 	 * up to a limit of 1 minute. */
 	if (!restart_delay)
@@ -464,6 +503,15 @@ calc_restart_delay(const timeval_t *last_start_time, unsigned *next_restart_dela
 	return restart_delay;
 }
 
+void
+log_child_died(const char *process, pid_t pid)
+{
+	log_message(LOG_ALERT, "%s child process(%d) died: Respawning", process, pid);
+	log_message(LOG_INFO, "  Please log an issue at https://github.com/acassen/keepalived/issues/");
+	log_message(LOG_INFO, "  and include a full copy of your keepalived configuration files, and");
+	log_message(LOG_INFO, "  copies of the keepalived system log entries around the time this happened");
+}
+
 /* report_child_status returns true if the exit is a hard error, so unable to continue */
 bool
 report_child_status(int status, pid_t pid, char const *prog_name)
@@ -528,7 +576,8 @@ report_child_status(int status, pid_t pid, char const *prog_name)
 //				segv_termination = true;
 		}
 		else
-			log_message(LOG_INFO, "%s exited due to signal %d (%s)", prog_id, WTERMSIG(status), strsignal(WTERMSIG(status)));
+			log_message(LOG_INFO, "%s exited due to signal %d (%s)%s", prog_id, WTERMSIG(status), strsignal(WTERMSIG(status)),
+					WTERMSIG(status) == SIGKILL ? " - has rlimit_rttime been exceeded?" : "");
 	}
 
 	return false;
@@ -575,7 +624,7 @@ thread_event_new(thread_master_t *m, int fd)
 {
 	thread_event_t *event;
 
-	event = (thread_event_t *) MALLOC(sizeof(thread_event_t));
+	PMALLOC(event);
 	if (!event)
 		return NULL;
 
@@ -691,7 +740,7 @@ thread_make_master(void)
 {
 	thread_master_t *new;
 
-	new = (thread_master_t *) MALLOC(sizeof (thread_master_t));
+	PMALLOC(new);
 
 #if HAVE_EPOLL_CREATE1
 	new->epoll_fd = epoll_create1(EPOLL_CLOEXEC);
@@ -883,10 +932,24 @@ thread_destroy_list(thread_master_t *m, list_head_t *l)
 	thread_t *thread, *thread_tmp;
 
 	list_for_each_entry_safe(thread, thread_tmp, l, e_list) {
-		if (thread->event) {
-			thread_del_read(thread);
-			thread_del_write(thread);
+		/* The following thread types are relevant for the ready list */
+		if (thread->type == THREAD_READY_READ_FD ||
+		    thread->type == THREAD_READY_WRITE_FD ||
+		    thread->type == THREAD_READ_TIMEOUT ||
+		    thread->type == THREAD_WRITE_TIMEOUT ||
+		    thread->type == THREAD_READ_ERROR ||
+		    thread->type == THREAD_WRITE_ERROR) {
+			/* Do we have a thread_event, and does it need deleting? */
+			if (thread->event) {
+				thread_del_read(thread);
+				thread_del_write(thread);
+			}
+
+			/* Do we have a file descriptor that needs closing ? */
+			if (thread->u.f.close_on_reload)
+				thread_close_fd(thread);
 		}
+
 		list_del_init(&thread->e_list);
 		thread_add_unuse(m, thread);
 	}
@@ -900,14 +963,9 @@ thread_destroy_rb(thread_master_t *m, rb_root_cached_t *root)
 	rb_for_each_entry_safe_cached(thread, thread_tmp, root, n) {
 		rb_erase_cached(&thread->n, root);
 
+		/* The following are relevant for the read and write rb lists */
 		if (thread->type == THREAD_READ ||
-		    thread->type == THREAD_WRITE ||
-		    thread->type == THREAD_READY_READ_FD ||
-		    thread->type == THREAD_READY_WRITE_FD ||
-		    thread->type == THREAD_READ_TIMEOUT ||
-		    thread->type == THREAD_WRITE_TIMEOUT ||
-		    thread->type == THREAD_READ_ERROR ||
-		    thread->type == THREAD_WRITE_ERROR) {
+		    thread->type == THREAD_WRITE) {
 			/* Do we have a thread_event, and does it need deleting? */
 			if (thread->type == THREAD_READ)
 				thread_del_read(thread);
@@ -1011,7 +1069,7 @@ thread_new(thread_master_t *m)
 	/* If one thread is already allocated return it */
 	new = thread_trim_head(&m->unuse);
 	if (!new) {
-		new = (thread_t *)MALLOC(sizeof(thread_t));
+		PMALLOC(new);
 		m->alloc++;
 	}
 
@@ -1589,7 +1647,7 @@ snmp_epoll_info(thread_master_t *m)
 	/* When SNMP is enabled, we may have to select() on additional
 	 * FDs. snmp_select_info() will add them to `readfd'. The trick
 	 * with this function is its last argument. We need to set it
-	 * true to set its own timer we update * the snmp timer thread timeout */
+	 * true to set its own timer, we then update the snmp timer thread timeout */
 	snmp_select_info(&fdsetsize, &snmp_fdset, &snmp_timer_wait, &snmpblock);
 
 	if (snmpblock)
@@ -1601,15 +1659,19 @@ snmp_epoll_info(thread_master_t *m)
 		return;
 	set_words = (max_fdsetsize - 1) / (sizeof(*old_set) * CHAR_BIT) + 1;
 
-	for (i = 0, old_set = (unsigned long *)&m->snmp_fdset, new_set = (unsigned long *)&snmp_fdset; i < set_words; i++, old_set++, new_set++) {
+	/* coverity[ptr_arith] */
+	for (i = 0, old_set = PTR_CAST(unsigned long, &m->snmp_fdset), new_set = PTR_CAST(unsigned long, &snmp_fdset); i < set_words; i++, old_set++, new_set++) {
 		if (*old_set == *new_set)
 			continue;
 
 		diff = *old_set ^ *new_set;
 		fd = i * sizeof(*old_set) * CHAR_BIT - 1;
-		do {
+		while (diff) {
 			bit = ffsl(diff);
-			diff >>= bit;
+			if (bit == sizeof(diff) * CHAR_BIT)
+				diff = 0;
+			else
+				diff >>= bit;
 			fd += bit;
 			if (FD_ISSET(fd, &snmp_fdset)) {
 				/* Add the fd */
@@ -1620,7 +1682,7 @@ snmp_epoll_info(thread_master_t *m)
 				thread_del_read_fd(m, fd);
 				FD_CLR(fd, &m->snmp_fdset);
 			}
-		} while (diff);
+		}
 	}
 	m->snmp_fdsetsize = fdsetsize;
 }
@@ -1649,36 +1711,42 @@ snmp_epoll_update(thread_master_t *m, bool reset)
 	set_words = m->snmp_fdsetsize ? (m->snmp_fdsetsize - 1) / (sizeof(*old_set) * CHAR_BIT) + 1 : 0;
 
 	/* Clear all the fds that were registered with epoll */
-	for (i = 0, old_set = (unsigned long *)&m->snmp_fdset; i < set_words; i++, old_set++) {
+	for (i = 0, old_set = PTR_CAST(unsigned long, &m->snmp_fdset); i < set_words; i++, old_set++) {
 		bits = *old_set;
 		fd = i * sizeof(*old_set) * CHAR_BIT - 1;
-		do {
+		while (bits) {
 			bit = ffsl(bits);
-			bits >>= bit;
+			if (bit == sizeof(bits) * CHAR_BIT)
+				bits = 0;
+			else
+				bits >>= bit;
 			fd += bit;
 
 			/* Remove the fd */
 			thread_del_read_fd(m, fd);
 			FD_CLR(fd, &m->snmp_fdset);
-		} while (bits);
+		}
 	}
 
 	if (reset) {
 		/* Add the file descriptors that are now in use */
-		set_words = fdsetsize ? (fdsetsize - 1) / (sizeof(*old_set) * CHAR_BIT) + 1 : 0;
+		set_words = fdsetsize ? (fdsetsize - 1) / (sizeof(*new_set) * CHAR_BIT) + 1 : 0;
 
-		for (i = 0, new_set = (unsigned long *)&snmp_fdset; i < set_words; i++, new_set++) {
+		for (i = 0, new_set = PTR_CAST(unsigned long, &snmp_fdset); i < set_words; i++, new_set++) {
 			bits = *new_set;
 			fd = i * sizeof(*new_set) * CHAR_BIT - 1;
-			do {
+			while (bits) {
 				bit = ffsl(bits);
-				bits >>= bit;
+				if (bit == sizeof(bits) * CHAR_BIT)
+					bits = 0;
+				else
+					bits >>= bit;
 				fd += bit;
 
 				/* Add the fd */
 				thread_add_read(m, snmp_read_thread, 0, fd, TIMER_NEVER, false);
 				FD_SET(fd, &m->snmp_fdset);
-			} while (bits);
+			}
 		}
 	} else
 		fdsetsize = 0;
@@ -1706,6 +1774,7 @@ static list_head_t *
 thread_fetch_next_queue(thread_master_t *m)
 {
 	int last_epoll_errno = 0;
+	unsigned last_epoll_errno_count = 0;
 	int ret;
 	int i;
 	timeval_t earliest_timer;
@@ -1765,6 +1834,12 @@ thread_fetch_next_queue(thread_master_t *m)
 
 				/* Log the error first time only */
 				log_message(LOG_INFO, "scheduler: epoll_wait error: %d (%m)", errno);
+
+				last_epoll_errno_count = 1;
+			} else if (++last_epoll_errno_count == 5 && shutdown_function) {
+				/* We aren't goint to be able to recover, so exit and let our parent restart us */
+				log_message(LOG_INFO, "scheduler: epoll_wait has returned errno %d for 5 successive calls - terminating", last_epoll_errno);
+				shutdown_function(KEEPALIVED_EXIT_PROGRAM_ERROR);
 			}
 
 			/* Make sure we don't sit it a tight loop */
@@ -1772,7 +1847,8 @@ thread_fetch_next_queue(thread_master_t *m)
 				sleep(1);
 
 			continue;
-		}
+		} else
+			last_epoll_errno = 0;
 
 		/* Check to see if we are long overdue. This can happen on a very heavily loaded system */
 		if (min_auto_priority_delay && timerisset(&earliest_timer)) {
@@ -1812,6 +1888,11 @@ thread_fetch_next_queue(thread_master_t *m)
 			ep_ev = &m->epoll_events[i];
 			ev = ep_ev->data.ptr;
 
+#ifdef _EPOLL_DEBUG_
+			if (do_epoll_debug)
+				log_message(LOG_INFO, "Handling event 0x%x for fd %d", ep_ev->events, ev->fd);
+#endif
+
 			/* Error */
 			if (ep_ev->events & (EPOLLHUP | EPOLLERR | EPOLLRDHUP)) {
 				if (ev->read) {
