@@ -223,7 +223,7 @@ extern perf_t perf_run;
 
 /* Prototypes defs */
 extern void dump_buffer(const char *, size_t, FILE *, int);
-#ifdef _CHECKSUM_DEBUG_
+#if defined _CHECKSUM_DEBUG_ || defined _RECVMSG_DEBUG_
 extern void log_buffer(const char *, const void *, size_t);
 #endif
 #ifdef _WITH_STACKTRACE_
